import { FilterOperator, FilterService, FilterMatchMode } from 'primevue/api';
import ArrowDownIcon from 'primevue/icons/arrowdown';
import ArrowUpIcon from 'primevue/icons/arrowup';
import SpinnerIcon from 'primevue/icons/spinner';
import Paginator from 'primevue/paginator';
import { DomHandler, ObjectUtils, RowStatus, UniqueComponentId, ZIndexUtils, ConnectedOverlayScrollHandler, DataHandler } from 'primevue/utils';
import VirtualScroller from 'primevue/virtualscroller';
import ChevronDownIcon from 'primevue/icons/chevrondown';
import ChevronRightIcon from 'primevue/icons/chevronright';
import BarsIcon from 'primevue/icons/bars';
import CheckIcon from 'primevue/icons/check';
import PencilIcon from 'primevue/icons/pencil';
import TimesIcon from 'primevue/icons/times';
import OverlayEventBus from 'primevue/overlayeventbus';
import Ripple from 'primevue/ripple';
import dayjs from 'dayjs';
import { resolveComponent, openBlock, createElementBlock, normalizeClass, createElementVNode, createBlock, resolveDynamicComponent, withKeys, withModifiers, resolveDirective, createCommentVNode, normalizeStyle, toDisplayString, Fragment, mergeProps, withDirectives, createTextVNode, renderList, createVNode, withCtx, Transition, computed, renderSlot, createSlots } from 'vue';
import Button from 'primevue/button';
import Dropdown from 'primevue/dropdown';
import FocusTrap from 'primevue/focustrap';
import FilterIcon from 'primevue/icons/filter';
import FilterSlashIcon from 'primevue/icons/filterslash';
import PlusIcon from 'primevue/icons/plus';
import TrashIcon from 'primevue/icons/trash';
import Portal from 'primevue/portal';
import SortAltIcon from 'primevue/icons/sortalt';
import SortAmountDownIcon from 'primevue/icons/sortamountdown';
import SortAmountUpAltIcon from 'primevue/icons/sortamountupalt';

function NumberFormat(value, format){
     
     const formatType = typeof format;
     let locale='zh-Hans-CN';
     let style='decimal';
     let currency='CNY';
     let currencyDisplay='symbol';
     let useGrouping=true;
     let maximumFractionDigits =2;
     let minimumFractionDigits =2; 
    
     if(format===null || format==='')
        return value;

     if(typeof value==='string'){
        value = Number.parseFloat(value);
        if(Number.isNaN(value))
            return 'NaN';
     }

     if(formatType ==='object'){
       const formater = new Intl.NumberFormat(locale, format );

       return formater.format(value || 0 );
     }

     if(formatType ==='string'){
        //如果是货币格式，在前面加货币代码
        let index= format.indexOf(":");

        if(index>0){
            currency = format.substring(0,index).toUpperCase().trim();
            format = format.substring(index+1);
            style ='currency';
        }
        else if(format.endsWith("%")){
            style="percent";
            format =format.substring(0,format.length -1);
        }
     }

     if(format.indexOf(",")<=0){
        useGrouping=false;
     }

     if(currency ==='RMB' || currency==='￥'){
        currency='CNY';
     }
     else if(currency==='$'){
        currency='USD';
     }

     if(format==='#,##0' || format ==='#,###' ){

        if( format ==='#,###'  && !value )
            return "";

        maximumFractionDigits=0;
        minimumFractionDigits =0;
     }
     
     else if(format==='0'){
        maximumFractionDigits=0;
        minimumFractionDigits =0;
     }
   
    const  formater =new Intl.NumberFormat(locale,{style, currency, currencyDisplay , useGrouping ,  maximumFractionDigits,minimumFractionDigits } ); 

    return formater.format(value || 0 );
}


function DateTimeFormat(value, format){
    if(value===null || value===undefined || value==='')
        return "";

    return dayjs(value).format(format);
}

var script$a = {
    name: 'RowCheckbox',
    emits: ['change'],
    props: {
        value: null,
        checked: null,
        rowCheckboxIconTemplate: {
            type: Function,
            default: null
        }
    },
    data() {
        return {
            focused: false
        };
    },
    methods: {
        onClick(event) {
            if (!this.$attrs.disabled) {
                this.$emit('change', {
                    originalEvent: event,
                    data: this.value
                });

                DomHandler.focus(this.$refs.input);
            }

            event.preventDefault();
        },
        onFocus() {
            this.focused = true;
        },
        onBlur() {
            this.focused = false;
        },
        onKeydown(event) {
            switch (event.code) {
                case 'Space': {
                    this.onClick(event);

                    break;
                }
            }
        }
    },
    computed: {
        checkboxAriaLabel() {
            return this.$primevue.config.locale.aria ? (this.checked ? this.$primevue.config.locale.aria.selectRow : this.$primevue.config.locale.aria.unselectRow) : undefined;
        }
    },
    components: {
        CheckIcon: CheckIcon
    }
};

const _hoisted_1$a = { class: "p-hidden-accessible" };
const _hoisted_2$9 = ["checked", "disabled", "tabindex", "aria-label"];

function render$a(_ctx, _cache, $props, $setup, $data, $options) {
  const _component_CheckIcon = resolveComponent("CheckIcon");

  return (openBlock(), createElementBlock("div", {
    class: normalizeClass(['p-checkbox p-component', { 'p-checkbox-focused': $data.focused }]),
    onClick: _cache[3] || (_cache[3] = (...args) => ($options.onClick && $options.onClick(...args)))
  }, [
    createElementVNode("div", _hoisted_1$a, [
      createElementVNode("input", {
        ref: "input",
        type: "checkbox",
        checked: $props.checked,
        disabled: _ctx.$attrs.disabled,
        tabindex: _ctx.$attrs.disabled ? null : '0',
        "aria-label": $options.checkboxAriaLabel,
        onFocus: _cache[0] || (_cache[0] = $event => ($options.onFocus($event))),
        onBlur: _cache[1] || (_cache[1] = $event => ($options.onBlur($event))),
        onKeydown: _cache[2] || (_cache[2] = (...args) => ($options.onKeydown && $options.onKeydown(...args)))
      }, null, 40 /* PROPS, HYDRATE_EVENTS */, _hoisted_2$9)
    ]),
    createElementVNode("div", {
      ref: "box",
      class: normalizeClass(['p-checkbox-box p-component', { 'p-highlight': $props.checked, 'p-disabled': _ctx.$attrs.disabled, 'p-focus': $data.focused }])
    }, [
      ($props.rowCheckboxIconTemplate)
        ? (openBlock(), createBlock(resolveDynamicComponent($props.rowCheckboxIconTemplate), {
            key: 0,
            checked: $props.checked,
            class: "p-checkbox-icon"
          }, null, 8 /* PROPS */, ["checked"]))
        : (openBlock(), createBlock(_component_CheckIcon, {
            key: 1,
            class: "p-checkbox-icon"
          }))
    ], 2 /* CLASS */)
  ], 2 /* CLASS */))
}

script$a.render = render$a;
script$a.__file = "components/lib/datatable/RowCheckbox.vue";

var script$9 = {
    name: 'RowRadioButton',
    inheritAttrs: false,
    emits: ['change'],
    props: {
        value: null,
        checked: null,
        name: null
    },
    data() {
        return {
            focused: false
        };
    },
    methods: {
        onClick(event) {
            if (!this.disabled) {
                if (!this.checked) {
                    this.$emit('change', {
                        originalEvent: event,
                        data: this.value
                    });

                    DomHandler.focus(this.$refs.input);
                }
            }
        },
        onFocus() {
            this.focused = true;
        },
        onBlur() {
            this.focused = false;
        }
    }
};

const _hoisted_1$9 = { class: "p-hidden-accessible" };
const _hoisted_2$8 = ["checked", "disabled", "name"];
const _hoisted_3$5 = /*#__PURE__*/createElementVNode("div", { class: "p-radiobutton-icon" }, null, -1 /* HOISTED */);
const _hoisted_4$5 = [
  _hoisted_3$5
];

function render$9(_ctx, _cache, $props, $setup, $data, $options) {
  return (openBlock(), createElementBlock("div", {
    class: normalizeClass(['p-radiobutton p-component', { 'p-radiobutton-focused': $data.focused }]),
    onClick: _cache[3] || (_cache[3] = (...args) => ($options.onClick && $options.onClick(...args)))
  }, [
    createElementVNode("div", _hoisted_1$9, [
      createElementVNode("input", {
        ref: "input",
        type: "radio",
        checked: $props.checked,
        disabled: _ctx.$attrs.disabled,
        name: $props.name,
        tabindex: "0",
        onFocus: _cache[0] || (_cache[0] = $event => ($options.onFocus($event))),
        onBlur: _cache[1] || (_cache[1] = $event => ($options.onBlur($event))),
        onKeydown: _cache[2] || (_cache[2] = withKeys(withModifiers((...args) => ($options.onClick && $options.onClick(...args)), ["prevent"]), ["space"]))
      }, null, 40 /* PROPS, HYDRATE_EVENTS */, _hoisted_2$8)
    ]),
    createElementVNode("div", {
      ref: "box",
      class: normalizeClass(['p-radiobutton-box p-component', { 'p-highlight': $props.checked, 'p-disabled': _ctx.$attrs.disabled, 'p-focus': $data.focused }])
    }, _hoisted_4$5, 2 /* CLASS */)
  ], 2 /* CLASS */))
}

script$9.render = render$9;
script$9.__file = "components/lib/datatable/RowRadioButton.vue";

var script$8 = {
    name: 'BodyCell',
    emits: ['cell-edit-init', 'cell-edit-complete', 'cell-edit-cancel', 'row-toggle', 'radio-change', 'checkbox-change', 'editing-meta-change','item-change'],
    props: {
        rowData: {
            type: Object,
            default: null
        },
       
        column: {
            type: Object,
            default: null
        },
        frozenRow: {
            type: Boolean,
            default: false
        },
        rowIndex: {
            type: Number,
            default: null
        },
        index: {
            type: Number,
            default: null
        },
        isRowExpanded: {
            type: Boolean,
            default: false
        },
        selected: {
            type: Boolean,
            default: false
        },
        editing: {
            type: Boolean,
            default: false
        },
        responsiveLayout: {
            type: String,
            default: 'stack'
        },
        virtualScrollerContentProps: {
            type: Object,
            default: null
        },
        ariaControls: {
            type: String,
            default: null
        },
        name: {
            type: String,
            default: null
        },
        expandedRowIcon: {
            type: String,
            default: null
        },
        collapsedRowIcon: {
            type: String,
            default: null
        },
        disabled:{
            type:Boolean,
            default:false
        },
        readonly:{
            type:Boolean,
            default:false
        }
    },

    inject:{
        dataTable:"DataTable"
     },

    documentEditListener: null,
    selfClick: false,
    overlayEventListener: null,
    data() {
        return {
            d_editing: this.editing,
            d_edingCellClass:null,
            styleObject: {},
        };
    },

    watch: {
        editing(newValue) {
           this.d_editing = newValue;
        },
    },

    mounted() {
        if (this.columnProp('frozen')) {
            this.updateStickyPosition();
        }
    },

    updated() {
        if (this.columnProp('frozen')) {
            this.updateStickyPosition();
        }

        if (this.d_editing) {
            setTimeout(() => {
                const focusableEl = DomHandler.getFirstFocusableElement(this.$el);
                
                if(focusableEl){
                    //下面这行代码会导致点击单元时，单元在表格中显示的位置发生改变
                    // focusableEl.scrollIntoView && focusableEl.scrollIntoView({ block: 'nearest', inline: 'start' });
                    focusableEl.focus();
                } 
            }, 10);
        }
    },
    beforeUnmount() {
        if (this.overlayEventListener) {
            OverlayEventBus.off('overlay-click', this.overlayEventListener);
            this.overlayEventListener = null;
        }
    },
    methods: {
        columnProp(prop) {
            return ObjectUtils.getVNodeProp(this.column, prop);
        },

        resolveFieldData() {
            const value = ObjectUtils.resolveFieldData(this.rowData, this.field);

            if(value ===null || value ===undefined || value==='' )
               return ( this.index ===0 ) ?' ' : '';   //如果第一个输入的单元内容为空，则输出空格，否则新增的行如果全部字段的值都为NULL，行的显示高度有问题
        
            
            //如果列的编辑格式是下接控件或设置了代码表
            if( this.column.props["codeTable"] || (this.column.props["editor"]==="Dropdown" && this.column.props["editorProps"]) ){
                 const editorProps= this.column.props["editorProps"] || {};
                 const options= ( editorProps.options || this.column.props["codeTable"] ) || [];

                 if(options && options.length){
                     let {optionLabel='value', optionValue='code'} = editorProps;

                     return this.getOptionLabel( value, options,optionValue,optionLabel );
                 }
            }

            let format= this.numberFormat;

            if(format){
                if(typeof format==='function'){
                    return format(value);
                }

                if(( value instanceof Date )|| 
                   ( (( typeof value)==="string" ) && ((new Date(value)) instanceof Date )) ) {
                    return DateTimeFormat(value,format);
                }

                return NumberFormat(value,format);
            }

            return value ;
        },

        toggleRow(event) {
            this.$emit('row-toggle', {
                originalEvent: event,
                data: this.rowData
            });
        },
        toggleRowWithRadio(event, index) {
            this.$emit('radio-change', { originalEvent: event.originalEvent, index: index, data: event.data });
        },
        toggleRowWithCheckbox(event, index) {
            this.$emit('checkbox-change', { originalEvent: event.originalEvent, index: index, data: event.data });
        },
        isEditable() {
            if(this.readonly  )
                return false;

            const editor= ( this.column.props && this.column.props["editor"] )  || '';

            return editor!=='' ||  ( this.column.children && this.column.children.editor != null);
        },
        bindDocumentEditListener() {
            if (!this.documentEditListener) {
                this.documentEditListener = (event) => {
                    if (!this.selfClick) {
                        this.completeEdit(event, 'outside');
                    }

                    this.selfClick = false;
                };

                document.addEventListener('click', this.documentEditListener);
            }
        },
        unbindDocumentEditListener() {
            if (this.documentEditListener) {
                document.removeEventListener('click', this.documentEditListener);
                this.documentEditListener = null;
                this.selfClick = false;
            }
        },

        switchCellToViewMode() {
            this.d_editing = false;
            this.unbindDocumentEditListener();
            OverlayEventBus.off('overlay-click', this.overlayEventListener);
            this.overlayEventListener = null;
        },

        onClick(event) {
            if (this.isEditable()) {
                this.selfClick = true;

                if (!this.d_editing) {
                    this.d_editing = true;
                    this.bindDocumentEditListener();
                    this.$emit('cell-edit-init', { originalEvent: event, data: this.rowData, field: this.field, index: this.rowIndex });

                    this.overlayEventListener = (e) => {
                        if (this.$el && this.$el.contains(e.target)) {
                            this.selfClick = true;
                        }
                    };

                    OverlayEventBus.on('overlay-click', this.overlayEventListener);
                }
            }
        },

        onUpdateModelValue(){
            const itemChangeEvent ={
                rowData: this.rowData,
                field: this.field,
                value:this.rowData[this.field],
            };

            this.$emit('item-change', itemChangeEvent);
        },

        completeEdit(event, type) {
            const completeEvent = {
                originalEvent: event,
                rowData: this.rowData,
                field: this.field,
                value:this.rowData[this.field],
                index: this.rowIndex,
                type: type,
                defaultPrevented: false,
                preventDefault: function () {
                    this.defaultPrevented = true;
                }
            };

             this.$emit('cell-edit-complete', completeEvent);

            if ( type==='outside' || !completeEvent.defaultPrevented) {
                let moveToNewCell = true;

                if(type==='tab'|| ( type==='enter' && !this.enterToNextRow) ){
                    if (event.shiftKey){
                        moveToNewCell=this.moveToPreviousCell(event);
                    }
                     else {
                        moveToNewCell = this.moveToNextCell(event);
                     }
                }
                else if(type==='arrowDown' || (  type==='enter' && this.enterToNextRow) ){
                    moveToNewCell= this.moveToNextRow(event, this.field);
                }
                else if( type==='arrowUp'){
                    moveToNewCell=   this.moveToPreviousRow(event, this.field);
                }

                 moveToNewCell &&  this.switchCellToViewMode();
            }
            else {
                
                if(type!=='outside'){
                    event.preventDefault();
                    event.stopPropagation();
                }
            }
        },

        onKeyDown(event) {
            if (this.isEditable()) {

                switch (event.code) {
                    case 'ArrowUp':
                        this.completeEdit(event, 'arrowUp');
                        break;

                    case 'ArrowDown':
                         this.completeEdit(event, 'arrowDown');    
                         break;

                    case 'Enter':
                    case 'NumpadEnter':
                        this.completeEdit(event, 'enter');
                        break;

                    case 'Escape':
                        this.switchCellToViewMode();
                        this.$emit('cell-edit-cancel', { originalEvent: event, data: this.rowData, field: this.field, index: this.rowIndex });
                         break;

                    case 'Tab':
                        this.completeEdit(event, 'tab');
                        break;
                }
            }
        },
        moveToPreviousCell(event) {
            event.preventDefault();
            event.stopPropagation();

            let result=false;
            let currentCell = this.findCell(event.target);
            let targetCell = this.findPreviousEditableColumn(currentCell);

            if (targetCell) {
                DomHandler.invokeElementMethod(targetCell, 'click');
                result =true;
            }

            return result;
        },
        moveToNextCell(event) {
            event.preventDefault();
            event.stopPropagation();

            let result=false;
            let currentCell = this.findCell(event.target);
            let targetCell = this.findNextEditableColumn(currentCell);

            if (targetCell) {
                DomHandler.invokeElementMethod(targetCell, 'click');
                result =true;
            }

            return result;
        },
        findCell(element) {
            if (element) {
                let cell = element;

                while (cell && !DomHandler.hasClass(cell, 'p-cell-editing')) {
                    cell = cell.parentElement;
                }

                return cell;
            } else {
                return null;
            }
        },
        findPreviousEditableColumn(cell) {
            let prevCell = cell.previousElementSibling;

            if (!prevCell) {
                let previousRow = cell.parentElement.previousElementSibling;

                if (previousRow) {
                    prevCell = previousRow.lastElementChild;
                }
            }

            if (prevCell) {
                if (DomHandler.hasClass(prevCell, 'p-editable-column')) return prevCell;
                else return this.findPreviousEditableColumn(prevCell);
            } else {
                return null;
            }
        },
        findNextEditableColumn(cell) {
            let nextCell = cell.nextElementSibling;

            if (!nextCell) {
                let nextRow = cell.parentElement.nextElementSibling;

                if (nextRow) {
                    nextCell = nextRow.firstElementChild;
                }
            }

            if (nextCell) {
                if (DomHandler.hasClass(nextCell, 'p-editable-column')) return nextCell;
                else return this.findNextEditableColumn(nextCell);
            } else {
                return null;
            }
        },
        isEditingCellValid() {
            return DomHandler.find(this.$el, '.p-invalid').length === 0;
        },

        updateStickyPosition() {
            if (this.columnProp('frozen')) {
                let align = this.columnProp('alignFrozen');

                if (align === 'right') {
                    let right = 0;
                    let next = this.$el.nextElementSibling;

                    if (next) {
                        right = DomHandler.getOuterWidth(next) + parseFloat(next.style.right || 0);
                    }

                    this.styleObject.right = right + 'px';
                } else {
                    let left = 0;
                    let prev = this.$el.previousElementSibling;

                    if (prev) {
                        left = DomHandler.getOuterWidth(prev) + parseFloat(prev.style.left || 0);
                    }

                    this.styleObject.left = left + 'px';
                }
            }
        },
        getVirtualScrollerProp(option) {
            return this.virtualScrollerContentProps ? this.virtualScrollerContentProps[option] : null;
        },

       getOptionLabel( value, options,optionValue,optionLabel ){
            if(options.length<=0)
                return value;

            let option=options[0];

            if(!optionLabel && option.hasOwnProperty("name")){
                optionLabel ="name";
            }

            if(!optionLabel && option.hasOwnProperty("label")){
                optionLabel ="label";
            }

            if(!optionLabel && option.hasOwnProperty("value")){
                optionLabel ="value";
            }

            if(!optionValue && option.hasOwnProperty("code")){
                optionValue ="code";
            }

            if(!optionValue && option.hasOwnProperty("id")){
                optionValue ="id";
            }

            if(!optionValue && option.hasOwnProperty("value")){
                optionValue ="value";
            }

            option= options.find((item)=> item[optionValue] === value);

            return (option)? option[optionLabel] : value;
       },

       getRowElememnt(element){

             if (element.nodeName === 'TR') {
                return element;
            } else {
                let parent = element.parentElement;

                while (parent.nodeName !== 'TR') {
                    parent = parent.parentElement;
                    if (!parent) break;
                }

                return parent;
            }
       },

       moveToPreviousRow(event, field) {
            event.preventDefault();
            event.stopPropagation();

            let result=false;
            const row = this.getRowElememnt(event.target);
            const preRow = row && row.previousElementSibling ;
            const targetCell= preRow && preRow.querySelector(`[data-field=${field}`);

            if (targetCell) {
                DomHandler.invokeElementMethod(targetCell, 'click');
                result =true;
            }

            return result;
        },

        moveToNextRow(event,field) {
            event.preventDefault();
            event.stopPropagation();

            let result=false;
            const row = this.getRowElememnt(event.target);
            const preRow =row && row.nextElementSibling ;
            const targetCell= preRow && preRow.querySelector(`[data-field=${field}`);

            if (targetCell) {
                DomHandler.invokeElementMethod(targetCell, 'click');
                result =true;
            }

            return result;
        },
        onUpdateEditingCellClass(val){
            this.d_edingCellClass =val;
        },
    },
    computed: {
        editingRowData() {
            return this.rowData;
        },

        field() {
            return this.columnProp('field');
        },
        containerClass() {
            //增加列的对齐方式的设置
            const editor= (this.column.props && this.column.props["editor"] ) || '';
            const align= (this.columnProp('align') || '' ).toLowerCase();
            const alignClass= (align==='') ? null : ( (editor==='InputSwitch')?'justify-content-'+ align: 'text-'+align);
            const edingClass =( this.d_editing) ? this.d_edingCellClass : null;
            
            return [
                alignClass,
                edingClass,
                this.columnProp('bodyClass'),
                this.columnProp('class'),
                {
                    'p-selection-column': this.columnProp('selectionMode') != null,
                    'p-editable-column': this.isEditable(),
                    'p-cell-editing': this.d_editing,
                    'p-frozen-column': this.columnProp('frozen'),
                    'p-disabled': this.disabled,
                }
            ];
        },
        containerStyle() {
             //增加列宽度的设置
             let width =  this.columnProp('width') || '';
            let widthStyle =(width==='') ? null : { width }  ;

            let bodyStyle = this.columnProp('bodyStyle');
            let columnStyle = this.columnProp('style');

            //设置的格式是一个函数，则把当前行的值传给函数计算，返回相应的格式
            if( (typeof bodyStyle)==='function')
                bodyStyle = bodyStyle(this.rowData,this.field);

            return this.columnProp('frozen') ? [ widthStyle, columnStyle, bodyStyle, this.styleObject] : [ widthStyle, columnStyle, bodyStyle];
        },
        loading() {
            return this.getVirtualScrollerProp('loading');
        },
        loadingOptions() {
            const getLoaderOptions = this.getVirtualScrollerProp('getLoaderOptions');

            return (
                getLoaderOptions &&
                getLoaderOptions(this.rowIndex, {
                    cellIndex: this.index,
                    cellFirst: this.index === 0,
                    cellLast: this.index === this.getVirtualScrollerProp('columns').length - 1,
                    cellEven: this.index % 2 === 0,
                    cellOdd: this.index % 2 !== 0,
                    column: this.column,
                    field: this.field
                })
            );
        },
        expandButtonAriaLabel() {
            return this.$primevue.config.locale.aria ? (this.isRowExpanded ? this.$primevue.config.locale.aria.expandRow : this.$primevue.config.locale.aria.collapseRow) : undefined;
        },
        initButtonAriaLabel() {
            return this.$primevue.config.locale.aria ? this.$primevue.config.locale.aria.editRow : undefined;
        },
        saveButtonAriaLabel() {
            return this.$primevue.config.locale.aria ? this.$primevue.config.locale.aria.saveEdit : undefined;
        },
        cancelButtonAriaLabel() {
            return this.$primevue.config.locale.aria ? this.$primevue.config.locale.aria.cancelEdit : undefined;
        },

        showEditor(){
            const editor= (this.column.props && this.column.props["editor"] ) || '';

            return (editor!=='' && ((!this.readonly && !this.disabled && this.d_editing )  || editor==='Checkbox' || editor==='RadioButton' ||
                    editor==='InputSwitch'  || editor==='Chip' || editor==='Slider')) ? true:false;

        },

        numberFormat(){
            let format = this.column.props["format"] || '';

            if(format==='' && this.column.props["editor"]==="InputNumber"){
                const editorProps=this.column.props["editorProps"] || {} ;
                const {minFractionDigits:minimumFractionDigits, maxFractionDigits:maximumFractionDigits} = editorProps;

                format =Object.assign( {}, editorProps, { style:'decimal', useGrouping:true , minimumFractionDigits,maximumFractionDigits } );
            }

            return format
        },

        enterToNextRow(){
            return (this.dataTable && this.dataTable.enterToNextRow)? true :false;
        },

       
        
    },
    components: {
        DTRadioButton: script$9,
        DTCheckbox: script$a,
        ChevronDownIcon: ChevronDownIcon,
        ChevronRightIcon: ChevronRightIcon,
        BarsIcon: BarsIcon,
        PencilIcon: PencilIcon,
        CheckIcon: CheckIcon,
        TimesIcon: TimesIcon
    },
    directives: {
        ripple: Ripple
    }
};

const _hoisted_1$8 = { key: 0 };
const _hoisted_2$7 = ["data-field"];
const _hoisted_3$4 = {
  key: 0,
  class: "p-column-title"
};
const _hoisted_4$4 = { key: 6 };
const _hoisted_5$4 = ["aria-expanded", "aria-controls", "aria-label"];

function render$8(_ctx, _cache, $props, $setup, $data, $options) {
  const _component_DTRadioButton = resolveComponent("DTRadioButton");
  const _component_DTCheckbox = resolveComponent("DTCheckbox");
  const _component_ChevronDownIcon = resolveComponent("ChevronDownIcon");
  const _component_ChevronRightIcon = resolveComponent("ChevronRightIcon");
  const _directive_ripple = resolveDirective("ripple");

  return ($props.column.props===undefined || $props.column.props===null)
    ? (openBlock(), createElementBlock("td", _hoisted_1$8, [
        ($props.column.children && $props.column.children.body)
          ? (openBlock(), createBlock(resolveDynamicComponent($props.column.children.body), {
              key: 0,
              data: $options.editingRowData,
              onUpdateEditingCellClass: $options.onUpdateEditingCellClass,
              "onUpdate:modelValue": $options.onUpdateModelValue
            }, null, 40 /* PROPS, HYDRATE_EVENTS */, ["data", "onUpdateEditingCellClass", "onUpdate:modelValue"]))
          : createCommentVNode("v-if", true)
      ]))
    : ($options.loading)
      ? (openBlock(), createElementBlock("td", {
          key: 1,
          style: normalizeStyle($options.containerStyle),
          class: normalizeClass($options.containerClass)
        }, [
          (openBlock(), createBlock(resolveDynamicComponent($props.column.children.loading), {
            data: $props.rowData,
            column: $props.column,
            field: $options.field,
            index: $props.rowIndex,
            frozenRow: $props.frozenRow,
            loadingOptions: $options.loadingOptions
          }, null, 8 /* PROPS */, ["data", "column", "field", "index", "frozenRow", "loadingOptions"]))
        ], 6 /* CLASS, STYLE */))
      : (openBlock(), createElementBlock("td", {
          key: 2,
          style: normalizeStyle($options.containerStyle),
          class: normalizeClass($options.containerClass),
          onClick: _cache[4] || (_cache[4] = (...args) => ($options.onClick && $options.onClick(...args))),
          onKeydown: _cache[5] || (_cache[5] = (...args) => ($options.onKeyDown && $options.onKeyDown(...args))),
          "data-field": $props.column.props.field
        }, [
          ($props.responsiveLayout === 'stack')
            ? (openBlock(), createElementBlock("span", _hoisted_3$4, toDisplayString($options.columnProp('header')), 1 /* TEXT */))
            : createCommentVNode("v-if", true),
          ($props.column.children && $props.column.children.body && !$data.d_editing)
            ? (openBlock(), createBlock(resolveDynamicComponent($props.column.children.body), {
                key: 1,
                data: $props.rowData,
                column: $props.column,
                field: $options.field,
                index: $props.rowIndex,
                frozenRow: $props.frozenRow
              }, null, 8 /* PROPS */, ["data", "column", "field", "index", "frozenRow"]))
            : ($options.showEditor)
              ? (openBlock(), createElementBlock(Fragment, { key: 2 }, [
                  createCommentVNode("支持按编辑控件类型来设置编辑对象"),
                  (openBlock(), createBlock(resolveDynamicComponent($props.column.props.editor), mergeProps({
                    modelValue: $options.editingRowData[$options.field],
                    "onUpdate:modelValue": _cache[0] || (_cache[0] = $event => (($options.editingRowData[$options.field]) = $event))
                  }, $props.column.props.editorProps, {
                    tableCellEditor: true,
                    align: $props.column.props.align,
                    disabled: $props.disabled,
                    readonly: $props.readonly,
                    onUpdateEditingCellClass: $options.onUpdateEditingCellClass,
                    "onUpdate:modelValue": $options.onUpdateModelValue
                  }), null, 16 /* FULL_PROPS */, ["modelValue", "align", "disabled", "readonly", "onUpdateEditingCellClass", "onUpdate:modelValue"]))
                ], 2112 /* STABLE_FRAGMENT, DEV_ROOT_FRAGMENT */))
              : ( $props.column.children && $props.column.children.editor && $data.d_editing )
                ? (openBlock(), createBlock(resolveDynamicComponent($props.column.children.editor), {
                    key: 3,
                    data: $props.rowData,
                    column: $props.column,
                    field: $options.field,
                    index: $props.rowIndex,
                    frozenRow: $props.frozenRow,
                    disabled: $props.disabled,
                    readonly: $props.readonly,
                    onUpdateEditingCellClass: $options.onUpdateEditingCellClass,
                    "onUpdate:modelValue": $options.onUpdateModelValue
                  }, null, 40 /* PROPS, HYDRATE_EVENTS */, ["data", "column", "field", "index", "frozenRow", "disabled", "readonly", "onUpdateEditingCellClass", "onUpdate:modelValue"]))
                : ($props.column.children && $props.column.children.body && !$props.column.children.editor && $data.d_editing  )
                  ? (openBlock(), createBlock(resolveDynamicComponent($props.column.children.body), {
                      key: 4,
                      data: $options.editingRowData,
                      column: $props.column,
                      field: $options.field,
                      index: $props.rowIndex,
                      frozenRow: $props.frozenRow,
                      readonly: $props.readonly
                    }, null, 8 /* PROPS */, ["data", "column", "field", "index", "frozenRow", "readonly"]))
                  : ($options.columnProp('selectionMode'))
                    ? (openBlock(), createElementBlock(Fragment, { key: 5 }, [
                        ($options.columnProp('selectionMode') === 'single')
                          ? (openBlock(), createBlock(_component_DTRadioButton, {
                              key: 0,
                              value: $props.rowData,
                              name: $props.name,
                              checked: $props.selected,
                              onChange: _cache[1] || (_cache[1] = $event => ($options.toggleRowWithRadio($event, $props.rowIndex)))
                            }, null, 8 /* PROPS */, ["value", "name", "checked"]))
                          : ($options.columnProp('selectionMode') === 'multiple')
                            ? (openBlock(), createBlock(_component_DTCheckbox, {
                                key: 1,
                                value: $props.rowData,
                                checked: $props.selected,
                                rowCheckboxIconTemplate: $props.column.children && $props.column.children.rowcheckboxicon,
                                "aria-selected": $props.selected ? true : undefined,
                                onChange: _cache[2] || (_cache[2] = $event => ($options.toggleRowWithCheckbox($event, $props.rowIndex)))
                              }, null, 8 /* PROPS */, ["value", "checked", "rowCheckboxIconTemplate", "aria-selected"]))
                            : createCommentVNode("v-if", true)
                      ], 64 /* STABLE_FRAGMENT */))
                    : ($options.columnProp('rowNumber'))
                      ? (openBlock(), createElementBlock("span", _hoisted_4$4, toDisplayString($props.rowIndex + 1), 1 /* TEXT */))
                      : ($options.columnProp('rowReorder'))
                        ? (openBlock(), createBlock(resolveDynamicComponent($props.column.children && $props.column.children.rowreordericon ? $props.column.children.rowreordericon : $options.columnProp('rowReorderIcon') ? 'i' : 'BarsIcon'), {
                            key: 7,
                            class: normalizeClass(['p-datatable-reorderablerow-handle', $options.columnProp('rowReorderIcon')])
                          }, null, 8 /* PROPS */, ["class"]))
                        : ($options.columnProp('expander'))
                          ? withDirectives((openBlock(), createElementBlock("button", {
                              key: 8,
                              class: "p-row-toggler p-link",
                              type: "button",
                              "aria-expanded": $props.isRowExpanded,
                              "aria-controls": $props.ariaControls,
                              "aria-label": $options.expandButtonAriaLabel,
                              onClick: _cache[3] || (_cache[3] = (...args) => ($options.toggleRow && $options.toggleRow(...args)))
                            }, [
                              ($props.column.children && $props.column.children.rowtogglericon)
                                ? (openBlock(), createBlock(resolveDynamicComponent($props.column.children.rowtogglericon), {
                                    key: 0,
                                    rowExpanded: $props.isRowExpanded
                                  }, null, 8 /* PROPS */, ["rowExpanded"]))
                                : (openBlock(), createElementBlock(Fragment, { key: 1 }, [
                                    ($props.isRowExpanded && $props.expandedRowIcon)
                                      ? (openBlock(), createElementBlock("span", {
                                          key: 0,
                                          class: normalizeClass(['p-row-toggler-icon', $props.expandedRowIcon])
                                        }, null, 2 /* CLASS */))
                                      : ($props.isRowExpanded && !$props.expandedRowIcon)
                                        ? (openBlock(), createBlock(_component_ChevronDownIcon, {
                                            key: 1,
                                            class: "p-row-toggler-icon"
                                          }))
                                        : (!$props.isRowExpanded && $props.collapsedRowIcon)
                                          ? (openBlock(), createElementBlock("span", {
                                              key: 2,
                                              class: normalizeClass(['p-row-toggler-icon', $props.collapsedRowIcon])
                                            }, null, 2 /* CLASS */))
                                          : (!$props.isRowExpanded && !$props.collapsedRowIcon)
                                            ? (openBlock(), createBlock(_component_ChevronRightIcon, {
                                                key: 3,
                                                class: "p-row-toggler-icon"
                                              }))
                                            : createCommentVNode("v-if", true)
                                  ], 64 /* STABLE_FRAGMENT */))
                            ], 8 /* PROPS */, _hoisted_5$4)), [
                              [_directive_ripple]
                            ])
                          : (openBlock(), createElementBlock(Fragment, { key: 9 }, [
                              createTextVNode(toDisplayString($options.resolveFieldData()), 1 /* TEXT */)
                            ], 64 /* STABLE_FRAGMENT */))
        ], 46 /* CLASS, STYLE, PROPS, HYDRATE_EVENTS */, _hoisted_2$7))
}

script$8.render = render$8;
script$8.__file = "components/lib/datatable/BodyCell.vue";

var script$7 = {
    name: 'TableBody',
    emits: [
        'rowgroup-toggle',
        'row-click',
        'row-dblclick',
        'row-rightclick',
        'row-touchend',
        'row-keydown',
        'row-mousedown',
        'row-dragstart',
        'row-dragover',
        'row-dragleave',
        'row-dragend',
        'row-drop',
        'row-toggle',
        'radio-change',
        'checkbox-change',
        'cell-edit-init',
        'cell-edit-complete',
        'cell-edit-cancel',
        'item-change',
        'body-keydown'
    ],
    props: {
        value: {
            type: Array,
            default: null
        },
        columns: {
            type: null,
            default: null
        },
        frozenRow: {
            type: Boolean,
            default: false
        },
        empty: {
            type: Boolean,
            default: false
        },
        rowGroupMode: {
            type: String,
            default: null
        },
        groupRowsBy: {
            type: [Array, String, Function],
            default: null
        },
        expandableRowGroups: {
            type: Boolean,
            default: false
        },
        expandedRowGroups: {
            type: Array,
            default: null
        },

        currentRow: {
            type: Number,
            default: 0
        },

        first: {
            type: Number,
            default: 0
        },
        dataKey: {
            type: String,
            default: null
        },
        expandedRowIcon: {
            type: String,
            default: null
        },
        collapsedRowIcon: {
            type: String,
            default: null
        },
        expandedRows: {
            type: Array,
            default: null
        },
        expandedRowKeys: {
            type: null,
            default: null
        },

        selectionMode: {
            type: String,
            default: null
        },
        contextMenu: {
            type: Boolean,
            default: false
        },
        contextMenuSelection: {
            type: Object,
            default: null
        },
        rowClass: {
            type: null,
            default: null
        },
        rowStyle: {
            type: null,
            default: null
        },
      
        compareSelectionBy: {
            type: String,
            default: 'deepEquals'
        },
        templates: {
            type: null,
            default: null
        },
        scrollable: {
            type: Boolean,
            default: false
        },
        responsiveLayout: {
            type: String,
            default: 'stack'
        },
        virtualScrollerContentProps: {
            type: Object,
            default: null
        },
        isVirtualScrollerDisabled: {
            type: Boolean,
            default: false
        },
        currentRowHighlight:{
            type: Boolean,
            default: true
        },

        disabled:{
             type:Boolean,
             default:null
         },
        readonly:{
            type:Boolean,
            default:false
        }
    },
    
    inject:{
         dataTable:"DataTable"
     },

    data() {
        return {
            rowGroupHeaderStyleObject: {},
            tabindexArray: [],
            isARowSelected: false
        };
    },
    mounted() {
        if (this.frozenRow) {
            this.updateFrozenRowStickyPosition();
        }

        if (this.scrollable && this.rowGroupMode === 'subheader') {
            this.updateFrozenRowGroupHeaderStickyPosition();
        }
    },
    updated() {
        if (this.frozenRow) {
            this.updateFrozenRowStickyPosition();
        }

        if (this.scrollable && this.rowGroupMode === 'subheader') {
            this.updateFrozenRowGroupHeaderStickyPosition();
        }
    },

    methods: {
        columnProp(col, prop) {
            return ObjectUtils.getVNodeProp(col, prop);
        },
        shouldRenderRowGroupHeader(value, rowData, i) {
            let currentRowFieldData = ObjectUtils.resolveFieldData(rowData, this.groupRowsBy);
            let prevRowData = value[i - 1];

            if (prevRowData) {
                let previousRowFieldData = ObjectUtils.resolveFieldData(prevRowData, this.groupRowsBy);

                return currentRowFieldData !== previousRowFieldData;
            } else {
                return true;
            }
        },

        getRowKey(rowData, index) {
           if(rowData && ('rowId' in rowData)){
               return rowData["rowId"];
           }

           return this.dataKey ? ObjectUtils.resolveFieldData(rowData, this.dataKey) : this.getRowIndex(index);
        },

        getRowIndex(index) {
            const getItemOptions = this.getVirtualScrollerProp('getItemOptions');

            //改为数据显示在table中的行序号，而不是在数据表的行数据index
            //return getItemOptions ? getItemOptions(index).index : this.first +index

             return getItemOptions ? getItemOptions(index).index : index;
        },

        getRowStyle(rowData) {
            if (this.rowStyle) {
                return this.rowStyle(rowData);
            }
        },
        getRowClass(rowData, index ) {
            let rowStyleClass = [];

            if(!this.disabled){
                if( this.currentRowHighlight && index === this.currentRow ){
                    if(this.selectionMode)   //如果支持行选择，则用 p-currentRow 的主题颜色
                         rowStyleClass.push('p-currentRow');
                    else
                         rowStyleClass.push('p-highlight');
                }

                rowStyleClass.push({
                    'p-highlight': this.isSelected(rowData) 
                });
            
                if (this.contextMenuSelection) {
                    rowStyleClass.push({
                        'p-highlight-contextmenu': this.isSelectedWithContextMenu(rowData)
                    });
                } 
            }

            if (this.rowClass) {
                let rowClassValue = this.rowClass(rowData);

                if (rowClassValue) {
                    rowStyleClass.push(rowClassValue);
                }
            }

            return rowStyleClass;
        },
        shouldRenderRowGroupFooter(value, rowData, i) {
            if (this.expandableRowGroups && !this.isRowGroupExpanded(rowData)) {
                return false;
            } else {
                let currentRowFieldData = ObjectUtils.resolveFieldData(rowData, this.groupRowsBy);
                let nextRowData = value[i + 1];

                if (nextRowData) {
                    let nextRowFieldData = ObjectUtils.resolveFieldData(nextRowData, this.groupRowsBy);

                    return currentRowFieldData !== nextRowFieldData;
                } else {
                    return true;
                }
            }
        },
        shouldRenderBodyCell(value, column, i) {
            if (this.rowGroupMode) {
                if (this.rowGroupMode === 'subheader') {
                    return this.groupRowsBy !== this.columnProp(column, 'field');
                } else if (this.rowGroupMode === 'rowspan') {
                    if (this.isGrouped(column)) {
                        let prevRowData = value[i - 1];

                        if (prevRowData) {
                            let currentRowFieldData = ObjectUtils.resolveFieldData(value[i], this.columnProp(column, 'field'));
                            let previousRowFieldData = ObjectUtils.resolveFieldData(prevRowData, this.columnProp(column, 'field'));

                            return currentRowFieldData !== previousRowFieldData;
                        } else {
                            return true;
                        }
                    } else {
                        return true;
                    }
                }
            } else {
                return !this.columnProp(column, 'hidden');
            }
        },
        calculateRowGroupSize(value, column, index) {
            if (this.isGrouped(column)) {
                let currentRowFieldData = ObjectUtils.resolveFieldData(value[index], this.columnProp(column, 'field'));
                let nextRowFieldData = currentRowFieldData;
                let groupRowSpan = 0;

                while (currentRowFieldData === nextRowFieldData) {
                    groupRowSpan++;
                    let nextRowData = value[++index];

                    if (nextRowData) {
                        nextRowFieldData = ObjectUtils.resolveFieldData(nextRowData, this.columnProp(column, 'field'));
                    } else {
                        break;
                    }
                }

                return groupRowSpan === 1 ? null : groupRowSpan;
            } else {
                return null;
            }
        },
        isGrouped(column) {
            if (this.groupRowsBy && this.columnProp(column, 'field')) {
                if (Array.isArray(this.groupRowsBy)) return this.groupRowsBy.indexOf(column.props.field) > -1;
                else return this.groupRowsBy === column.props.field;
            } else {
                return false;
            }
        },
        isRowEditing(rowData) {
            return false;
        },
        isRowExpanded(rowData) {
            if (rowData && this.expandedRows) {
                if (this.dataKey) return this.expandedRowKeys ? this.expandedRowKeys[ObjectUtils.resolveFieldData(rowData, this.dataKey)] !== undefined : false;
                else return this.findIndex(rowData, this.expandedRows) > -1;
            }

            return false;
        },
        isRowGroupExpanded(rowData) {
            if (this.expandableRowGroups && this.expandedRowGroups) {
                let groupFieldValue = ObjectUtils.resolveFieldData(rowData, this.groupRowsBy);

                return this.expandedRowGroups.indexOf(groupFieldValue) > -1;
            }

            return false;
        },
        isSelected(rowData) {
           if(rowData && ('isSelected' in rowData)){
               return rowData["isSelected"];
           }

           return false;
        },
        
        isSelectedWithContextMenu(rowData) {
            if (rowData && this.contextMenuSelection) {
                return this.equals(rowData, this.contextMenuSelection, this.dataKey);
            }

            return false;
        },

        isDisabled(rowData, column){
            if(!rowData ||! column)
                return false;

            if(this.disabled)
                return true;

            const propValue = this.columnProp(column,"disabled") ;

            if(propValue===null || propValue===undefined || propValue==='' )
                return false;
            else if( ( typeof propValue) ==='boolean')
                return propValue;
            else if( ( typeof propValue)==='function'){
               return  propValue( rowData, this.columnProp(column,"field"));
            }
            else if(propValue==='notnew'){
                const status=rowData['rowStatus'];

                return  ( status ===RowStatus.New || status===RowStatus.NewModified)? false : true;
            }

            return  (propValue==='1'  ) ? true: false;
        },  

        isReadonly(rowData, column){
            if(!rowData ||! column)
                return false;

            if(this.readonly)
                return true;

            const propValue = this.columnProp(column,"readonly") ;

            if(propValue===null || propValue===undefined || propValue==='' )
                return false;
            else if( ( typeof propValue) ==='boolean')
                return propValue;
            else if( ( typeof propValue)==='function'){
               return  propValue( rowData, this.columnProp(column,"field"));
            }
            else if(propValue==='notnew'){
                const status=rowData['rowStatus'];

                return  ( status ===RowStatus.New || status===RowStatus.NewModified)? false : true;
            }

            return  (propValue==='1'  ) ? true: false;
        },  

        findIndex(rowData, collection) {
            let index = -1;

            if (collection && collection.length) {
                for (let i = 0; i < collection.length; i++) {
                    if (this.equals(rowData, collection[i])) {
                        index = i;
                        break;
                    }
                }
            }

            return index;
        },
        equals(data1, data2) {
            return this.compareSelectionBy === 'equals' ? data1 === data2 : ObjectUtils.equals(data1, data2, this.dataKey);
        },
        onRowGroupToggle(event, data) {
            this.$emit('rowgroup-toggle', { originalEvent: event, data: data });
        },
        onRowClick(event, rowData, rowIndex) {
            this.$emit('row-click', { originalEvent: event, data: rowData, index: rowIndex });
        },
        onRowDblClick(event, rowData, rowIndex) {
            this.$emit('row-dblclick', { originalEvent: event, data: rowData, index: rowIndex });
        },
        onRowRightClick(event, rowData, rowIndex) {
            this.$emit('row-rightclick', { originalEvent: event, data: rowData, index: rowIndex });
        },
        
        onRowTouchEnd(event) {
            this.$emit('row-touchend', event);
        },
        onRowKeyDown(event, rowData, rowIndex) {
           this.$emit('row-keydown', { originalEvent: event, data: rowData, index: rowIndex });
        },
        onRowMouseDown(event) {
            this.$emit('row-mousedown', event);
        },
        onRowDragStart(event, rowIndex) {
            this.$emit('row-dragstart', { originalEvent: event, index: rowIndex });
        },
        onRowDragOver(event, rowIndex) {
            this.$emit('row-dragover', { originalEvent: event, index: rowIndex });
        },
        onRowDragLeave(event) {
            this.$emit('row-dragleave', event);
        },
        onRowDragEnd(event) {
            this.$emit('row-dragend', event);
        },
        onRowDrop(event) {
            this.$emit('row-drop', event);
        },
        onRowToggle(event) {
            this.$emit('row-toggle', event);
        },
        onRadioChange(event) {
            this.$emit('radio-change', event);
        },
        onCheckboxChange(event) {
            this.$emit('checkbox-change', event);
        },
        onCellEditInit(event) {
            this.$emit('cell-edit-init', event);
        },
        onCellEditComplete(event) {
            this.$emit('cell-edit-complete', event);
        },
        onCellEditCancel(event) {
            this.$emit('cell-edit-cancel', event);
        },
        onItemChange(event){
            this.$emit('item-change', event);
        },
               
        updateFrozenRowStickyPosition() {
            this.$el.style.top = DomHandler.getOuterHeight(this.$el.previousElementSibling) + 'px';
        },
        updateFrozenRowGroupHeaderStickyPosition() {
            let tableHeaderHeight = DomHandler.getOuterHeight(this.$el.previousElementSibling);

            this.rowGroupHeaderStyleObject.top = tableHeaderHeight + 'px';
        },
        getVirtualScrollerProp(option, options) {
            options = options || this.virtualScrollerContentProps;

            return options ? options[option] : null;
        },
        bodyRef(el) {
            // For VirtualScroller
            const contentRef = this.getVirtualScrollerProp('contentRef');

            contentRef && contentRef(el);
        },
        setRowTabindex(index) {
            // if (this.selection === null && (this.selectionMode === 'single' || this.selectionMode === 'multiple')) {
            //     return index === 0 ? 0 : -1;
            // }

            return -1;
        },
        
        onBodyKeyDown(event){
            this.$emit('body-keydown',  { originalEvent: event, frozenRow: this.frozenRow  }  );
        }
    },

    computed: {
        columnsLength() {
            let hiddenColLength = 0;

            this.columns.forEach((column) => {
                //if (this.columnProp(column, 'selectionMode') === 'single') hiddenColLength--;
                if (this.columnProp(column, 'hidden')) hiddenColLength++;
            });

            return this.columns ? this.columns.length - hiddenColLength : 0;
        },
        rowGroupHeaderStyle() {
            if (this.scrollable) {
                return { top: this.rowGroupHeaderStyleObject.top };
            }

            return null;
        },
        bodyStyle() {
            return this.getVirtualScrollerProp('contentStyle');
        },
        expandedRowId() {
            return UniqueComponentId();
        },
        nameAttributeSelector() {
            return UniqueComponentId();
        }
    },
    components: {
        DTBodyCell: script$8,
        ChevronDownIcon: ChevronDownIcon,
        ChevronRightIcon: ChevronRightIcon
    }
};

const _hoisted_1$7 = ["tabindex"];
const _hoisted_2$6 = ["colspan"];
const _hoisted_3$3 = ["onClick"];
const _hoisted_4$3 = ["onClick", "onDblclick", "onContextmenu", "onDragstart", "onDragover"];
const _hoisted_5$3 = ["id"];
const _hoisted_6$2 = ["colspan"];
const _hoisted_7$2 = ["colspan"];
const _hoisted_8$1 = {
  key: 1,
  class: "p-datatable-emptymessage",
  role: "row"
};
const _hoisted_9$1 = ["colspan"];
const _hoisted_10 = { key: 1 };

function render$7(_ctx, _cache, $props, $setup, $data, $options) {
  const _component_ChevronDownIcon = resolveComponent("ChevronDownIcon");
  const _component_ChevronRightIcon = resolveComponent("ChevronRightIcon");
  const _component_DTBodyCell = resolveComponent("DTBodyCell");

  return (openBlock(), createElementBlock("tbody", {
    ref: $options.bodyRef,
    class: "p-datatable-tbody",
    role: "rowgroup",
    style: normalizeStyle($options.bodyStyle),
    onKeydown: _cache[12] || (_cache[12] = $event => ($options.onBodyKeyDown($event))),
    tabindex:  $props.frozenRow ? -1: 0 
  }, [
    createCommentVNode("  必须要设置 tabindex，才能响应keyodown事件  "),
    (!$props.empty)
      ? (openBlock(true), createElementBlock(Fragment, { key: 0 }, renderList($props.value, (rowData, index) => {
          return (openBlock(), createElementBlock(Fragment, null, [
            ($props.templates['groupheader'] && $props.rowGroupMode === 'subheader' && $options.shouldRenderRowGroupHeader($props.value, rowData, $options.getRowIndex(index)))
              ? (openBlock(), createElementBlock("tr", {
                  key: $options.getRowKey(rowData, $options.getRowIndex(index)) + '_subheader',
                  class: "p-rowgroup-header",
                  style: normalizeStyle($options.rowGroupHeaderStyle)
                }, [
                  createElementVNode("td", {
                    colspan: $options.columnsLength 
                  }, [
                    ($props.expandableRowGroups)
                      ? (openBlock(), createElementBlock("button", {
                          key: 0,
                          class: "p-row-toggler p-link",
                          onClick: $event => ($options.onRowGroupToggle($event, rowData)),
                          type: "button"
                        }, [
                          ($props.templates['rowgrouptogglericon'])
                            ? (openBlock(), createBlock(resolveDynamicComponent($props.templates['rowgrouptogglericon']), {
                                key: 0,
                                expanded: $options.isRowGroupExpanded(rowData)
                              }, null, 8 /* PROPS */, ["expanded"]))
                            : (openBlock(), createElementBlock(Fragment, { key: 1 }, [
                                ($options.isRowGroupExpanded(rowData) && $props.expandedRowIcon)
                                  ? (openBlock(), createElementBlock("span", {
                                      key: 0,
                                      class: normalizeClass(['p-row-toggler-icon', $props.expandedRowIcon])
                                    }, null, 2 /* CLASS */))
                                  : ($options.isRowGroupExpanded(rowData) && !$props.expandedRowIcon)
                                    ? (openBlock(), createBlock(_component_ChevronDownIcon, {
                                        key: 1,
                                        class: "p-row-toggler-icon"
                                      }))
                                    : (!$options.isRowGroupExpanded(rowData) && $props.collapsedRowIcon)
                                      ? (openBlock(), createElementBlock("span", {
                                          key: 2,
                                          class: normalizeClass(['p-row-toggler-icon', $props.collapsedRowIcon])
                                        }, null, 2 /* CLASS */))
                                      : (!$options.isRowGroupExpanded(rowData) && !$props.collapsedRowIcon)
                                        ? (openBlock(), createBlock(_component_ChevronRightIcon, {
                                            key: 3,
                                            class: "p-row-toggler-icon"
                                          }))
                                        : createCommentVNode("v-if", true)
                              ], 64 /* STABLE_FRAGMENT */))
                        ], 8 /* PROPS */, _hoisted_3$3))
                      : createCommentVNode("v-if", true),
                    (openBlock(), createBlock(resolveDynamicComponent($props.templates['groupheader']), {
                      data: rowData,
                      index: $options.getRowIndex(index)
                    }, null, 8 /* PROPS */, ["data", "index"]))
                  ], 8 /* PROPS */, _hoisted_2$6)
                ], 4 /* STYLE */))
              : createCommentVNode("v-if", true),
            createCommentVNode("\n                      把行的keydown 消息统一 转到body里来进行处理\n                      @keydown=\"onRowKeyDown($event, rowData, getRowIndex(index))\" \n                      :tabindex=\"setRowTabindex(index)\"\n                "),
            ($props.expandableRowGroups ? $options.isRowGroupExpanded(rowData) : true)
              ? (openBlock(), createElementBlock("tr", {
                  key: $options.getRowKey(rowData, $options.getRowIndex(index)),
                  class: normalizeClass($options.getRowClass(rowData,$options.getRowIndex(index) )),
                  style: normalizeStyle($options.getRowStyle(rowData)),
                  onClick: $event => ($options.onRowClick($event, rowData, $options.getRowIndex(index))),
                  onDblclick: $event => ($options.onRowDblClick($event, rowData, $options.getRowIndex(index))),
                  onContextmenu: $event => ($options.onRowRightClick($event, rowData, $options.getRowIndex(index))),
                  onTouchend: _cache[7] || (_cache[7] = $event => ($options.onRowTouchEnd($event))),
                  onMousedown: _cache[8] || (_cache[8] = $event => ($options.onRowMouseDown($event))),
                  onDragstart: $event => ($options.onRowDragStart($event, $options.getRowIndex(index))),
                  onDragover: $event => ($options.onRowDragOver($event, $options.getRowIndex(index))),
                  onDragleave: _cache[9] || (_cache[9] = $event => ($options.onRowDragLeave($event))),
                  onDragend: _cache[10] || (_cache[10] = $event => ($options.onRowDragEnd($event))),
                  onDrop: _cache[11] || (_cache[11] = $event => ($options.onRowDrop($event)))
                }, [
                  (openBlock(true), createElementBlock(Fragment, null, renderList($props.columns, (col, i) => {
                    return (openBlock(), createElementBlock(Fragment, null, [
                      ($options.shouldRenderBodyCell($props.value, col, $options.getRowIndex(index)))
                        ? (openBlock(), createBlock(_component_DTBodyCell, {
                            key: $options.columnProp(col, 'columnKey') || $options.columnProp(col, 'field') || i,
                            rowData: rowData,
                            column: col,
                            rowIndex: $options.getRowIndex(index),
                            index: i,
                            selected: $options.isSelected(rowData),
                            frozenRow: $props.frozenRow,
                            rowspan: $props.rowGroupMode === 'rowspan' ? $options.calculateRowGroupSize($props.value, col, $options.getRowIndex(index)) : null,
                            responsiveLayout: $props.responsiveLayout,
                            virtualScrollerContentProps: $props.virtualScrollerContentProps,
                            ariaControls: $options.expandedRowId + '_' + index + '_expansion',
                            name: $options.nameAttributeSelector,
                            isRowExpanded: $options.isRowExpanded(rowData),
                            expandedRowIcon: $props.expandedRowIcon,
                            collapsedRowIcon: $props.collapsedRowIcon,
                            disabled: $options.isDisabled(rowData,col),
                            readonly: $options.isReadonly(rowData,col),
                            onRadioChange: _cache[0] || (_cache[0] = $event => ($options.onRadioChange($event))),
                            onCheckboxChange: _cache[1] || (_cache[1] = $event => ($options.onCheckboxChange($event))),
                            onRowToggle: _cache[2] || (_cache[2] = $event => ($options.onRowToggle($event))),
                            onCellEditInit: _cache[3] || (_cache[3] = $event => ($options.onCellEditInit($event))),
                            onCellEditComplete: _cache[4] || (_cache[4] = $event => ($options.onCellEditComplete($event))),
                            onCellEditCancel: _cache[5] || (_cache[5] = $event => ($options.onCellEditCancel($event))),
                            onItemChange: _cache[6] || (_cache[6] = $event => ($options.onItemChange($event)))
                          }, null, 8 /* PROPS */, ["rowData", "column", "rowIndex", "index", "selected", "frozenRow", "rowspan", "responsiveLayout", "virtualScrollerContentProps", "ariaControls", "name", "isRowExpanded", "expandedRowIcon", "collapsedRowIcon", "disabled", "readonly"]))
                        : createCommentVNode("v-if", true)
                    ], 64 /* STABLE_FRAGMENT */))
                  }), 256 /* UNKEYED_FRAGMENT */))
                ], 46 /* CLASS, STYLE, PROPS, HYDRATE_EVENTS */, _hoisted_4$3))
              : createCommentVNode("v-if", true),
            ($props.templates['expansion'] && $props.expandedRows && $options.isRowExpanded(rowData))
              ? (openBlock(), createElementBlock("tr", {
                  key: $options.getRowKey(rowData, $options.getRowIndex(index)) + '_expansion',
                  id: $options.expandedRowId + '_' + index + '_expansion',
                  class: "p-datatable-row-expansion"
                }, [
                  createElementVNode("td", { colspan: $options.columnsLength }, [
                    (openBlock(), createBlock(resolveDynamicComponent($props.templates['expansion']), {
                      data: rowData,
                      index: $options.getRowIndex(index)
                    }, null, 8 /* PROPS */, ["data", "index"]))
                  ], 8 /* PROPS */, _hoisted_6$2)
                ], 8 /* PROPS */, _hoisted_5$3))
              : createCommentVNode("v-if", true),
            ($props.templates['groupfooter'] && $props.rowGroupMode === 'subheader' && $options.shouldRenderRowGroupFooter($props.value, rowData, $options.getRowIndex(index)))
              ? (openBlock(), createElementBlock("tr", {
                  key: $options.getRowKey(rowData, $options.getRowIndex(index)) + '_subfooter',
                  class: "p-rowgroup-footer"
                }, [
                  createElementVNode("td", {
                    colspan: $options.columnsLength 
                  }, [
                    (openBlock(), createBlock(resolveDynamicComponent($props.templates['groupfooter']), {
                      data: rowData,
                      index: $options.getRowIndex(index)
                    }, null, 8 /* PROPS */, ["data", "index"]))
                  ], 8 /* PROPS */, _hoisted_7$2)
                ]))
              : createCommentVNode("v-if", true)
          ], 64 /* STABLE_FRAGMENT */))
        }), 256 /* UNKEYED_FRAGMENT */))
      : (openBlock(), createElementBlock("tr", _hoisted_8$1, [
          createElementVNode("td", { colspan: $options.columnsLength }, [
            ($props.templates.empty)
              ? (openBlock(), createBlock(resolveDynamicComponent($props.templates.empty), { key: 0 }))
              : (openBlock(), createElementBlock("div", _hoisted_10, "暂无数据"))
          ], 8 /* PROPS */, _hoisted_9$1)
        ]))
  ], 44 /* STYLE, PROPS, HYDRATE_EVENTS */, _hoisted_1$7))
}

script$7.render = render$7;
script$7.__file = "components/lib/datatable/TableBody.vue";

var script$6 = {
    name: 'FooterCell',
    props: {
        column: {
            type: Object,
            default: null
        }
    },

    inject:{
         dataTable:"DataTable"
     },

    data() {
        return {
            styleObject: {},
            d_value: null  /*保存计算公式返回的值 ，可以实现按计算结果设置样式 */
        };
    },
    mounted() {
        if (this.columnProp('frozen')) {
            this.updateStickyPosition();
        }
    },
    updated() {
        if (this.columnProp('frozen')) {
            this.updateStickyPosition();
        }
    },
    methods: {
        columnProp(prop) {

            if(prop==='footer'){
                 let footer = this.column.props['footer'] || '';

                 if(footer==='')
                    return '';

                 let field= this.column.props['field'] || '';
                 let value =footer;

                 if(typeof footer==='function' ){
                    this.d_value= footer( this.dataTable.primaryData,field );
                    value= NumberFormat(this.d_value, this.numberFormat ) ;
                 } 

                 else if( this.dataTable && field && ( typeof footer==='string' ) &&  footer.startsWith("=") ){
                     const exp=footer.substring(1);

                     if(exp!==''){
                        this.d_value= this.dataTable.evaluate(exp,field);
                        value= NumberFormat(this.d_value, this.numberFormat) ;
                     }
                 }   

                 return value;
            }

            return ObjectUtils.getVNodeProp(this.column, prop);
        },

        updateStickyPosition() {
            if (this.columnProp('frozen')) {
                let align = this.columnProp('alignFrozen');

                if (align === 'right') {
                    let right = 0;
                    let next = this.$el.nextElementSibling;

                    if (next) {
                        right = DomHandler.getOuterWidth(next) + parseFloat(next.style.right || 0);
                    }

                    this.styleObject.right = right + 'px';
                } else {
                    let left = 0;
                    let prev = this.$el.previousElementSibling;

                    if (prev) {
                        left = DomHandler.getOuterWidth(prev) + parseFloat(prev.style.left || 0);
                    }

                    this.styleObject.left = left + 'px';
                }
            }
        }
    },
    computed: {
        containerClass() {
            const align= (this.columnProp('align') || '' ).toLowerCase();
            const alignClass= (align==='')? null : 'text-'+align;

            return [
                this.columnProp('footerClass'),
                this.columnProp('class'),
                alignClass,
                {
                    'p-frozen-column': this.columnProp('frozen')
                }
            ];
        },
        containerStyle() {
            //增加列宽度的设置
            let width =  this.columnProp('width') || '';
            let widthStyle =(width==='') ? null : { width }  ;

            let footerStyle = this.columnProp('footerStyle');
            // let columnStyle = this.columnProp('style');

            //设置的格式是一个函数，则把当前脚本区的值传给函数计算，返回相应的格式
            if((typeof footerStyle)==='function')
            {
                if(this.d_value!==null){
                    footerStyle = footerStyle(this.d_value, this.columnProp('field'));
                }else {
                    footerStyle= null;
                }
            }


            return this.columnProp('frozen') ? [widthStyle, footerStyle, this.styleObject] : [widthStyle, footerStyle];
        },

        numberFormat(){
            let format = this.column.props["format"] || '';

            if(format==='' && this.column.props["editor"]==="InputNumber"){
                const editorProps=this.column.props["editorProps"] || {} ;
                const {minFractionDigits:minimumFractionDigits, maxFractionDigits:maximumFractionDigits} = editorProps;
                
                format =Object.assign( {}, editorProps, { style:'decimal', useGrouping:true , minimumFractionDigits,maximumFractionDigits } );
            }

            return format
        }
    }
};

const _hoisted_1$6 = ["colspan", "rowspan"];

function render$6(_ctx, _cache, $props, $setup, $data, $options) {
  return (openBlock(), createElementBlock("td", {
    style: normalizeStyle($options.containerStyle),
    class: normalizeClass($options.containerClass),
    role: "cell",
    colspan: $options.columnProp('colspan'),
    rowspan: $options.columnProp('rowspan')
  }, [
    ($props.column.children && $props.column.children.footer)
      ? (openBlock(), createBlock(resolveDynamicComponent($props.column.children.footer), {
          key: 0,
          column: $props.column
        }, null, 8 /* PROPS */, ["column"]))
      : createCommentVNode("v-if", true),
    createTextVNode(" " + toDisplayString($options.columnProp('footer')), 1 /* TEXT */)
  ], 14 /* CLASS, STYLE, PROPS */, _hoisted_1$6))
}

script$6.render = render$6;
script$6.__file = "components/lib/datatable/FooterCell.vue";

var script$5 = {
    name: 'TableFooter',
    props: {
        columnGroup: {
            type: null,
            default: null
        },
        columns: {
            type: Object,
            default: null
        }
    },
   
    methods: {
        columnProp(col, prop) {
            return ObjectUtils.getVNodeProp(col, prop);
        },
        getFooterRows() {
            let rows = [];

            let columnGroup = this.columnGroup;

            if (columnGroup.children && columnGroup.children.default) {
                for (let child of columnGroup.children.default()) {
                    if (child.type.name === 'Row') {
                        rows.push(child);
                    } else if (child.children && child.children instanceof Array) {
                        rows = child.children;
                    }
                }

                return rows;
            }
        },
        getFooterColumns(row) {
            let cols = [];

            if (row.children && row.children.default) {
                row.children.default().forEach((child) => {
                    if (child.children && child.children instanceof Array) cols = [...cols, ...child.children];
                    else if (child.type.name === 'Column') cols.push(child);
                });

                return cols;
            }
        }
    },
    computed: {
        hasFooter() {
            let hasFooter = false;

            if (this.columnGroup) {
                hasFooter = true;
            } else if (this.columns) {
                for (let col of this.columns) {
                    if (this.columnProp(col, 'footer') || (col.children && col.children.footer)) {
                        hasFooter = true;
                        break;
                    }
                }
            }

            return hasFooter;
        }
    },
    components: {
        DTFooterCell: script$6
    }
};

const _hoisted_1$5 = {
  key: 0,
  class: "p-datatable-tfoot"
};
const _hoisted_2$5 = { key: 0 };

function render$5(_ctx, _cache, $props, $setup, $data, $options) {
  const _component_DTFooterCell = resolveComponent("DTFooterCell");

  return ($options.hasFooter)
    ? (openBlock(), createElementBlock("tfoot", _hoisted_1$5, [
        (!$props.columnGroup)
          ? (openBlock(), createElementBlock("tr", _hoisted_2$5, [
              (openBlock(true), createElementBlock(Fragment, null, renderList($props.columns, (col, i) => {
                return (openBlock(), createElementBlock(Fragment, {
                  key: $options.columnProp(col, 'columnKey') || $options.columnProp(col, 'field') || i
                }, [
                  (!$options.columnProp(col, 'hidden'))
                    ? (openBlock(), createBlock(_component_DTFooterCell, {
                        key: 0,
                        column: col
                      }, null, 8 /* PROPS */, ["column"]))
                    : createCommentVNode("v-if", true)
                ], 64 /* STABLE_FRAGMENT */))
              }), 128 /* KEYED_FRAGMENT */))
            ]))
          : (openBlock(true), createElementBlock(Fragment, { key: 1 }, renderList($options.getFooterRows(), (row, i) => {
              return (openBlock(), createElementBlock("tr", { key: i }, [
                (openBlock(true), createElementBlock(Fragment, null, renderList($options.getFooterColumns(row), (col, j) => {
                  return (openBlock(), createElementBlock(Fragment, {
                    key: $options.columnProp(col, 'columnKey') || $options.columnProp(col, 'field') || j
                  }, [
                    (!$options.columnProp(col, 'hidden'))
                      ? (openBlock(), createBlock(_component_DTFooterCell, {
                          key: 0,
                          column: col
                        }, null, 8 /* PROPS */, ["column"]))
                      : createCommentVNode("v-if", true)
                  ], 64 /* STABLE_FRAGMENT */))
                }), 128 /* KEYED_FRAGMENT */))
              ]))
            }), 128 /* KEYED_FRAGMENT */))
      ]))
    : createCommentVNode("v-if", true)
}

script$5.render = render$5;
script$5.__file = "components/lib/datatable/TableFooter.vue";

var script$4 = {
    name: 'ColumnFilter',
    emits: ['filter-change', 'filter-apply', 'operator-change', 'matchmode-change', 'constraint-add', 'constraint-remove', 'filter-clear', 'apply-click'],
    props: {
        field: {
            type: String,
            default: null
        },
        type: {
            type: String,
            default: 'text'
        },
        display: {
            type: String,
            default: null
        },
        showMenu: {
            type: Boolean,
            default: true
        },
        matchMode: {
            type: String,
            default: null
        },
        showOperator: {
            type: Boolean,
            default: true
        },
        showClearButton: {
            type: Boolean,
            default: true
        },
        showApplyButton: {
            type: Boolean,
            default: true
        },
        showMatchModes: {
            type: Boolean,
            default: true
        },
        showAddButton: {
            type: Boolean,
            default: true
        },
        matchModeOptions: {
            type: Array,
            default: null
        },
        maxConstraints: {
            type: Number,
            default: 2
        },
        filterElement: {
            type: Function,
            default: null
        },
        filterHeaderTemplate: {
            type: Function,
            default: null
        },
        filterFooterTemplate: {
            type: Function,
            default: null
        },
        filterClearTemplate: {
            type: Function,
            default: null
        },
        filterApplyTemplate: {
            type: Function,
            default: null
        },
        filterIconTemplate: {
            type: Function,
            default: null
        },
        filterAddIconTemplate: {
            type: Function,
            default: null
        },
        filterRemoveIconTemplate: {
            type: Function,
            default: null
        },
        filterClearIconTemplate: {
            type: Function,
            default: null
        },
        filters: {
            type: Object,
            default: null
        },
        filtersStore: {
            type: Object,
            default: null
        },
        filterMenuClass: {
            type: String,
            default: null
        },
        filterMenuStyle: {
            type: null,
            default: null
        },
        filterInputProps: {
            type: null,
            default: null
        }
    },
    data() {
        return {
            overlayVisible: false,
            defaultMatchMode: null,
            defaultOperator: null
        };
    },
    overlay: null,
    selfClick: false,
    overlayEventListener: null,
    beforeUnmount() {
        if (this.overlayEventListener) {
            OverlayEventBus.off('overlay-click', this.overlayEventListener);
            this.overlayEventListener = null;
        }

        if (this.overlay) {
            ZIndexUtils.clear(this.overlay);
            this.onOverlayHide();
        }
    },
    mounted() {
        if (this.filters && this.filters[this.field]) {
            let fieldFilters = this.filters[this.field];

            if (fieldFilters.operator) {
                this.defaultMatchMode = fieldFilters.constraints[0].matchMode;
                this.defaultOperator = fieldFilters.operator;
            } else {
                this.defaultMatchMode = this.filters[this.field].matchMode;
            }
        }
    },
    methods: {
        clearFilter() {
            let _filters = { ...this.filters };

            if (_filters[this.field].operator) {
                _filters[this.field].constraints.splice(1);
                _filters[this.field].operator = this.defaultOperator;
                _filters[this.field].constraints[0] = { value: null, matchMode: this.defaultMatchMode };
            } else {
                _filters[this.field].value = null;
                _filters[this.field].matchMode = this.defaultMatchMode;
            }

            this.$emit('filter-clear');
            this.$emit('filter-change', _filters);
            this.$emit('filter-apply');
            this.hide();
        },
        applyFilter() {
            this.$emit('apply-click', { field: this.field, constraints: this.filters[this.field] });
            this.$emit('filter-apply');
            this.hide();
        },
        hasFilter() {
            if (this.filtersStore) {
                let fieldFilter = this.filtersStore[this.field];

                if (fieldFilter) {
                    if (fieldFilter.operator) return !this.isFilterBlank(fieldFilter.constraints[0].value);
                    else return !this.isFilterBlank(fieldFilter.value);
                }
            }

            return false;
        },
        hasRowFilter() {
            return this.filters[this.field] && !this.isFilterBlank(this.filters[this.field].value);
        },
        isFilterBlank(filter) {
            if (filter !== null && filter !== undefined) {
                if ((typeof filter === 'string' && filter.trim().length == 0) || (filter instanceof Array && filter.length == 0)) return true;
                else return false;
            }

            return true;
        },
        toggleMenu(event) {
            this.overlayVisible = !this.overlayVisible;
            event.preventDefault();
        },
        onToggleButtonKeyDown(event) {
            switch (event.code) {
                case 'Enter':
                case 'Space':
                    this.toggleMenu();
                    event.preventDefault();
                    break;

                case 'Escape':
                    this.overlayVisible = false;
                    break;
            }
        },
        onRowMatchModeChange(matchMode) {
            let _filters = { ...this.filters };

            _filters[this.field].matchMode = matchMode;
            this.$emit('matchmode-change', { field: this.field, matchMode: matchMode });
            this.$emit('filter-change', _filters);
            this.$emit('filter-apply');
            this.hide();
        },
        onRowMatchModeKeyDown(event) {
            let item = event.target;

            switch (event.code) {
                case 'ArrowDown':
                    var nextItem = this.findNextItem(item);

                    if (nextItem) {
                        item.removeAttribute('tabindex');
                        nextItem.tabIndex = '0';
                        nextItem.focus();
                    }

                    event.preventDefault();
                    break;

                case 'ArrowUp':
                    var prevItem = this.findPrevItem(item);

                    if (prevItem) {
                        item.removeAttribute('tabindex');
                        prevItem.tabIndex = '0';
                        prevItem.focus();
                    }

                    event.preventDefault();
                    break;
            }
        },
        isRowMatchModeSelected(matchMode) {
            return this.filters[this.field].matchMode === matchMode;
        },
        onOperatorChange(value) {
            let _filters = { ...this.filters };

            _filters[this.field].operator = value;
            this.$emit('filter-change', _filters);

            this.$emit('operator-change', { field: this.field, operator: value });

            if (!this.showApplyButton) {
                this.$emit('filter-apply');
            }
        },
        onMenuMatchModeChange(value, index) {
            let _filters = { ...this.filters };

            _filters[this.field].constraints[index].matchMode = value;
            this.$emit('matchmode-change', { field: this.field, matchMode: value, index: index });

            if (!this.showApplyButton) {
                this.$emit('filter-apply');
            }
        },
        addConstraint() {
            let _filters = { ...this.filters };
            let newConstraint = { value: null, matchMode: this.defaultMatchMode };

            _filters[this.field].constraints.push(newConstraint);
            this.$emit('constraint-add', { field: this.field, constraing: newConstraint });
            this.$emit('filter-change', _filters);

            if (!this.showApplyButton) {
                this.$emit('filter-apply');
            }
        },
        removeConstraint(index) {
            let _filters = { ...this.filters };
            let removedConstraint = _filters[this.field].constraints.splice(index, 1);

            this.$emit('constraint-remove', { field: this.field, constraing: removedConstraint });
            this.$emit('filter-change', _filters);

            if (!this.showApplyButton) {
                this.$emit('filter-apply');
            }
        },
        filterCallback() {
            this.$emit('filter-apply');
        },
        findNextItem(item) {
            let nextItem = item.nextElementSibling;

            if (nextItem) return DomHandler.hasClass(nextItem, 'p-column-filter-separator') ? this.findNextItem(nextItem) : nextItem;
            else return item.parentElement.firstElementChild;
        },
        findPrevItem(item) {
            let prevItem = item.previousElementSibling;

            if (prevItem) return DomHandler.hasClass(prevItem, 'p-column-filter-separator') ? this.findPrevItem(prevItem) : prevItem;
            else return item.parentElement.lastElementChild;
        },
        hide() {
            this.overlayVisible = false;

            DomHandler.focus(this.$refs.icon);
        },
        onContentClick(event) {
            this.selfClick = true;

            OverlayEventBus.emit('overlay-click', {
                originalEvent: event,
                target: this.overlay
            });
        },
        onContentMouseDown() {
            this.selfClick = true;
        },
        onOverlayEnter(el) {
            if (this.filterMenuStyle) {
                DomHandler.applyStyle(this.overlay, this.filterMenuStyle);
            }

            ZIndexUtils.set('overlay', el, this.$primevue.config.zIndex.overlay);
            DomHandler.absolutePosition(this.overlay, this.$refs.icon);
            this.bindOutsideClickListener();
            this.bindScrollListener();
            this.bindResizeListener();

            this.overlayEventListener = (e) => {
                if (!this.isOutsideClicked(e.target)) {
                    this.selfClick = true;
                }
            };

            OverlayEventBus.on('overlay-click', this.overlayEventListener);
        },
        onOverlayLeave() {
            this.onOverlayHide();
        },
        onOverlayAfterLeave(el) {
            ZIndexUtils.clear(el);
        },
        onOverlayHide() {
            this.unbindOutsideClickListener();
            this.unbindResizeListener();
            this.unbindScrollListener();
            this.overlay = null;
            OverlayEventBus.off('overlay-click', this.overlayEventListener);
            this.overlayEventListener = null;
        },
        overlayRef(el) {
            this.overlay = el;
        },
        isOutsideClicked(target) {
            return !this.isTargetClicked(target) && this.overlay && !(this.overlay.isSameNode(target) || this.overlay.contains(target));
        },
        isTargetClicked(target) {
            return this.$refs.icon && (this.$refs.icon.isSameNode(target) || this.$refs.icon.contains(target));
        },
        bindOutsideClickListener() {
            if (!this.outsideClickListener) {
                this.outsideClickListener = (event) => {
                    if (this.overlayVisible && !this.selfClick && this.isOutsideClicked(event.target)) {
                        this.overlayVisible = false;
                    }

                    this.selfClick = false;
                };

                document.addEventListener('click', this.outsideClickListener);
            }
        },
        unbindOutsideClickListener() {
            if (this.outsideClickListener) {
                document.removeEventListener('click', this.outsideClickListener);
                this.outsideClickListener = null;
                this.selfClick = false;
            }
        },
        bindScrollListener() {
            if (!this.scrollHandler) {
                this.scrollHandler = new ConnectedOverlayScrollHandler(this.$refs.icon, () => {
                    if (this.overlayVisible) {
                        this.hide();
                    }
                });
            }

            this.scrollHandler.bindScrollListener();
        },
        unbindScrollListener() {
            if (this.scrollHandler) {
                this.scrollHandler.unbindScrollListener();
            }
        },
        bindResizeListener() {
            if (!this.resizeListener) {
                this.resizeListener = () => {
                    if (this.overlayVisible && !DomHandler.isTouchDevice()) {
                        this.hide();
                    }
                };

                window.addEventListener('resize', this.resizeListener);
            }
        },
        unbindResizeListener() {
            if (this.resizeListener) {
                window.removeEventListener('resize', this.resizeListener);
                this.resizeListener = null;
            }
        }
    },
    computed: {
        containerClass() {
            return [
                'p-column-filter p-fluid',
                {
                    'p-column-filter-row': this.display === 'row',
                    'p-column-filter-menu': this.display === 'menu'
                }
            ];
        },
        overlayClass() {
            return [
                this.filterMenuClass,
                {
                    'p-column-filter-overlay p-component p-fluid': true,
                    'p-column-filter-overlay-menu': this.display === 'menu',
                    'p-input-filled': this.$primevue.config.inputStyle === 'filled',
                    'p-ripple-disabled': this.$primevue.config.ripple === false
                }
            ];
        },
        showMenuButton() {
            return this.showMenu && (this.display === 'row' ? this.type !== 'boolean' : true);
        },
        overlayId() {
            return UniqueComponentId();
        },
        matchModes() {
            return (
                this.matchModeOptions ||
                this.$primevue.config.filterMatchModeOptions[this.type].map((key) => {
                    return { label: this.$primevue.config.locale[key], value: key };
                })
            );
        },
        isShowMatchModes() {
            return this.type !== 'boolean' && this.showMatchModes && this.matchModes;
        },
        operatorOptions() {
            return [
                { label: this.$primevue.config.locale.matchAll, value: FilterOperator.AND },
                { label: this.$primevue.config.locale.matchAny, value: FilterOperator.OR }
            ];
        },
        noFilterLabel() {
            return this.$primevue.config.locale ? this.$primevue.config.locale.noFilter : undefined;
        },
        isShowOperator() {
            return this.showOperator && this.filters[this.field].operator;
        },
        operator() {
            return this.filters[this.field].operator;
        },
        fieldConstraints() {
            return this.filters[this.field].constraints || [this.filters[this.field]];
        },
        showRemoveIcon() {
            return this.fieldConstraints.length > 1;
        },
        removeRuleButtonLabel() {
            return this.$primevue.config.locale ? this.$primevue.config.locale.removeRule : undefined;
        },
        addRuleButtonLabel() {
            return this.$primevue.config.locale ? this.$primevue.config.locale.addRule : undefined;
        },
        isShowAddConstraint() {
            return this.showAddButton && this.filters[this.field].operator && this.fieldConstraints && this.fieldConstraints.length < this.maxConstraints;
        },
        clearButtonLabel() {
            return this.$primevue.config.locale ? this.$primevue.config.locale.clear : undefined;
        },
        applyButtonLabel() {
            return this.$primevue.config.locale ? this.$primevue.config.locale.apply : undefined;
        },
        filterMenuButtonAriaLabel() {
            return this.$primevue.config.locale ? (this.overlayVisible ? this.$primevue.config.locale.showFilterMenu : this.$primevue.config.locale.hideFilterMenu) : undefined;
        },
        filterOperatorAriaLabel() {
            return this.$primevue.config.locale ? this.$primevue.config.locale.filterOperator : undefined;
        },
        filterConstraintAriaLabel() {
            return this.$primevue.config.locale ? this.$primevue.config.locale.filterConstraint : undefined;
        }
    },
    components: {
        CFDropdown: Dropdown,
        CFButton: Button,
        Portal: Portal,
        FilterSlashIcon: FilterSlashIcon,
        FilterIcon: FilterIcon,
        TrashIcon: TrashIcon,
        PlusIcon: PlusIcon
    },
    directives: {
        focustrap: FocusTrap
    }
};

const _hoisted_1$4 = ["aria-label", "aria-expanded", "aria-controls"];
const _hoisted_2$4 = ["id", "aria-modal"];
const _hoisted_3$2 = {
  key: 0,
  class: "p-column-filter-row-items"
};
const _hoisted_4$2 = ["onClick", "onKeydown", "tabindex"];
const _hoisted_5$2 = /*#__PURE__*/createElementVNode("li", { class: "p-column-filter-separator" }, null, -1 /* HOISTED */);
const _hoisted_6$1 = {
  key: 0,
  class: "p-column-filter-operator"
};
const _hoisted_7$1 = { class: "p-column-filter-constraints" };
const _hoisted_8 = {
  key: 1,
  class: "p-column-filter-add-rule"
};
const _hoisted_9 = { class: "p-column-filter-buttonbar" };

function render$4(_ctx, _cache, $props, $setup, $data, $options) {
  const _component_CFDropdown = resolveComponent("CFDropdown");
  const _component_CFButton = resolveComponent("CFButton");
  const _component_Portal = resolveComponent("Portal");
  const _directive_focustrap = resolveDirective("focustrap");

  return (openBlock(), createElementBlock("div", {
    class: normalizeClass($options.containerClass)
  }, [
    ($props.display === 'row')
      ? (openBlock(), createElementBlock("div", mergeProps({
          key: 0,
          class: "p-fluid p-column-filter-element"
        }, $props.filterInputProps), [
          (openBlock(), createBlock(resolveDynamicComponent($props.filterElement), {
            field: $props.field,
            filterModel: $props.filters[$props.field],
            filterCallback: $options.filterCallback
          }, null, 8 /* PROPS */, ["field", "filterModel", "filterCallback"]))
        ], 16 /* FULL_PROPS */))
      : createCommentVNode("v-if", true),
    ($options.showMenuButton)
      ? (openBlock(), createElementBlock("button", {
          key: 1,
          ref: "icon",
          type: "button",
          class: normalizeClass(["p-column-filter-menu-button p-link", { 'p-column-filter-menu-button-open': $data.overlayVisible, 'p-column-filter-menu-button-active': $options.hasFilter() }]),
          "aria-label": $options.filterMenuButtonAriaLabel,
          "aria-haspopup": "true",
          "aria-expanded": $data.overlayVisible,
          "aria-controls": $options.overlayId,
          onClick: _cache[0] || (_cache[0] = $event => ($options.toggleMenu($event))),
          onKeydown: _cache[1] || (_cache[1] = $event => ($options.onToggleButtonKeyDown($event)))
        }, [
          (openBlock(), createBlock(resolveDynamicComponent($props.filterIconTemplate || 'FilterIcon')))
        ], 42 /* CLASS, PROPS, HYDRATE_EVENTS */, _hoisted_1$4))
      : createCommentVNode("v-if", true),
    ($props.showClearButton && $props.display === 'row')
      ? (openBlock(), createElementBlock("button", {
          key: 2,
          class: normalizeClass([{ 'p-hidden-space': !$options.hasRowFilter() }, "p-column-filter-clear-button p-link"]),
          type: "button",
          onClick: _cache[2] || (_cache[2] = $event => ($options.clearFilter()))
        }, [
          (openBlock(), createBlock(resolveDynamicComponent($props.filterClearIconTemplate || 'FilterSlashIcon')))
        ], 2 /* CLASS */))
      : createCommentVNode("v-if", true),
    createVNode(_component_Portal, null, {
      default: withCtx(() => [
        createVNode(Transition, {
          name: "p-connected-overlay",
          onEnter: $options.onOverlayEnter,
          onLeave: $options.onOverlayLeave,
          onAfterLeave: $options.onOverlayAfterLeave
        }, {
          default: withCtx(() => [
            ($data.overlayVisible)
              ? withDirectives((openBlock(), createElementBlock("div", {
                  key: 0,
                  ref: $options.overlayRef,
                  id: $options.overlayId,
                  "aria-modal": $data.overlayVisible,
                  role: "dialog",
                  class: normalizeClass($options.overlayClass),
                  onKeydown: _cache[10] || (_cache[10] = withKeys((...args) => ($options.hide && $options.hide(...args)), ["escape"])),
                  onClick: _cache[11] || (_cache[11] = (...args) => ($options.onContentClick && $options.onContentClick(...args))),
                  onMousedown: _cache[12] || (_cache[12] = (...args) => ($options.onContentMouseDown && $options.onContentMouseDown(...args)))
                }, [
                  (openBlock(), createBlock(resolveDynamicComponent($props.filterHeaderTemplate), {
                    field: $props.field,
                    filterModel: $props.filters[$props.field],
                    filterCallback: $options.filterCallback
                  }, null, 8 /* PROPS */, ["field", "filterModel", "filterCallback"])),
                  ($props.display === 'row')
                    ? (openBlock(), createElementBlock("ul", _hoisted_3$2, [
                        (openBlock(true), createElementBlock(Fragment, null, renderList($options.matchModes, (matchMode, i) => {
                          return (openBlock(), createElementBlock("li", {
                            key: matchMode.label,
                            class: normalizeClass(["p-column-filter-row-item", { 'p-highlight': $options.isRowMatchModeSelected(matchMode.value) }]),
                            onClick: $event => ($options.onRowMatchModeChange(matchMode.value)),
                            onKeydown: [
                              _cache[3] || (_cache[3] = $event => ($options.onRowMatchModeKeyDown($event))),
                              withKeys(withModifiers($event => ($options.onRowMatchModeChange(matchMode.value)), ["prevent"]), ["enter"])
                            ],
                            tabindex: i === 0 ? '0' : null
                          }, toDisplayString(matchMode.label), 43 /* TEXT, CLASS, PROPS, HYDRATE_EVENTS */, _hoisted_4$2))
                        }), 128 /* KEYED_FRAGMENT */)),
                        _hoisted_5$2,
                        createElementVNode("li", {
                          class: "p-column-filter-row-item",
                          onClick: _cache[4] || (_cache[4] = $event => ($options.clearFilter())),
                          onKeydown: [
                            _cache[5] || (_cache[5] = $event => ($options.onRowMatchModeKeyDown($event))),
                            _cache[6] || (_cache[6] = withKeys($event => (_ctx.onRowClearItemClick()), ["enter"]))
                          ]
                        }, toDisplayString($options.noFilterLabel), 33 /* TEXT, HYDRATE_EVENTS */)
                      ]))
                    : (openBlock(), createElementBlock(Fragment, { key: 1 }, [
                        ($options.isShowOperator)
                          ? (openBlock(), createElementBlock("div", _hoisted_6$1, [
                              createVNode(_component_CFDropdown, {
                                options: $options.operatorOptions,
                                modelValue: $options.operator,
                                "aria-label": $options.filterOperatorAriaLabel,
                                class: "p-column-filter-operator-dropdown",
                                optionLabel: "label",
                                optionValue: "value",
                                "onUpdate:modelValue": _cache[7] || (_cache[7] = $event => ($options.onOperatorChange($event)))
                              }, null, 8 /* PROPS */, ["options", "modelValue", "aria-label"])
                            ]))
                          : createCommentVNode("v-if", true),
                        createElementVNode("div", _hoisted_7$1, [
                          (openBlock(true), createElementBlock(Fragment, null, renderList($options.fieldConstraints, (fieldConstraint, i) => {
                            return (openBlock(), createElementBlock("div", {
                              key: i,
                              class: "p-column-filter-constraint"
                            }, [
                              ($options.isShowMatchModes)
                                ? (openBlock(), createBlock(_component_CFDropdown, {
                                    key: 0,
                                    options: $options.matchModes,
                                    modelValue: fieldConstraint.matchMode,
                                    class: "p-column-filter-matchmode-dropdown",
                                    optionLabel: "label",
                                    optionValue: "value",
                                    "aria-label": $options.filterConstraintAriaLabel,
                                    "onUpdate:modelValue": $event => ($options.onMenuMatchModeChange($event, i))
                                  }, null, 8 /* PROPS */, ["options", "modelValue", "aria-label", "onUpdate:modelValue"]))
                                : createCommentVNode("v-if", true),
                              ($props.display === 'menu')
                                ? (openBlock(), createBlock(resolveDynamicComponent($props.filterElement), {
                                    key: 1,
                                    field: $props.field,
                                    filterModel: fieldConstraint,
                                    filterCallback: $options.filterCallback
                                  }, null, 8 /* PROPS */, ["field", "filterModel", "filterCallback"]))
                                : createCommentVNode("v-if", true),
                              createElementVNode("div", null, [
                                ($options.showRemoveIcon)
                                  ? (openBlock(), createBlock(_component_CFButton, {
                                      key: 0,
                                      type: "button",
                                      class: "p-column-filter-remove-button p-button-text p-button-danger p-button-sm",
                                      onClick: $event => ($options.removeConstraint(i)),
                                      label: $options.removeRuleButtonLabel
                                    }, {
                                      icon: withCtx((iconProps) => [
                                        (openBlock(), createBlock(resolveDynamicComponent($props.filterRemoveIconTemplate || 'TrashIcon'), {
                                          class: normalizeClass(iconProps.class)
                                        }, null, 8 /* PROPS */, ["class"]))
                                      ]),
                                      _: 2 /* DYNAMIC */
                                    }, 1032 /* PROPS, DYNAMIC_SLOTS */, ["onClick", "label"]))
                                  : createCommentVNode("v-if", true)
                              ])
                            ]))
                          }), 128 /* KEYED_FRAGMENT */))
                        ]),
                        ($options.isShowAddConstraint)
                          ? (openBlock(), createElementBlock("div", _hoisted_8, [
                              createVNode(_component_CFButton, {
                                type: "button",
                                label: $options.addRuleButtonLabel,
                                iconPos: "left",
                                class: "p-column-filter-add-button p-button-text p-button-sm",
                                onClick: _cache[8] || (_cache[8] = $event => ($options.addConstraint()))
                              }, {
                                icon: withCtx((iconProps) => [
                                  (openBlock(), createBlock(resolveDynamicComponent($props.filterAddIconTemplate || 'PlusIcon'), {
                                    class: normalizeClass(iconProps.class)
                                  }, null, 8 /* PROPS */, ["class"]))
                                ]),
                                _: 1 /* STABLE */
                              }, 8 /* PROPS */, ["label"])
                            ]))
                          : createCommentVNode("v-if", true),
                        createElementVNode("div", _hoisted_9, [
                          (!$props.filterClearTemplate && $props.showClearButton)
                            ? (openBlock(), createBlock(_component_CFButton, {
                                key: 0,
                                type: "button",
                                class: "p-button-outlined p-button-sm",
                                label: $options.clearButtonLabel,
                                onClick: $options.clearFilter
                              }, null, 8 /* PROPS */, ["label", "onClick"]))
                            : (openBlock(), createBlock(resolveDynamicComponent($props.filterClearTemplate), {
                                key: 1,
                                field: $props.field,
                                filterModel: $props.filters[$props.field],
                                filterCallback: $options.clearFilter
                              }, null, 8 /* PROPS */, ["field", "filterModel", "filterCallback"])),
                          ($props.showApplyButton)
                            ? (openBlock(), createElementBlock(Fragment, { key: 2 }, [
                                (!$props.filterApplyTemplate)
                                  ? (openBlock(), createBlock(_component_CFButton, {
                                      key: 0,
                                      type: "button",
                                      class: "p-button-sm",
                                      label: $options.applyButtonLabel,
                                      onClick: _cache[9] || (_cache[9] = $event => ($options.applyFilter()))
                                    }, null, 8 /* PROPS */, ["label"]))
                                  : (openBlock(), createBlock(resolveDynamicComponent($props.filterApplyTemplate), {
                                      key: 1,
                                      field: $props.field,
                                      filterModel: $props.filters[$props.field],
                                      filterCallback: $options.applyFilter
                                    }, null, 8 /* PROPS */, ["field", "filterModel", "filterCallback"]))
                              ], 64 /* STABLE_FRAGMENT */))
                            : createCommentVNode("v-if", true)
                        ])
                      ], 64 /* STABLE_FRAGMENT */)),
                  (openBlock(), createBlock(resolveDynamicComponent($props.filterFooterTemplate), {
                    field: $props.field,
                    filterModel: $props.filters[$props.field],
                    filterCallback: $options.filterCallback
                  }, null, 8 /* PROPS */, ["field", "filterModel", "filterCallback"]))
                ], 42 /* CLASS, PROPS, HYDRATE_EVENTS */, _hoisted_2$4)), [
                  [_directive_focustrap, { autoFocus: true }]
                ])
              : createCommentVNode("v-if", true)
          ]),
          _: 1 /* STABLE */
        }, 8 /* PROPS */, ["onEnter", "onLeave", "onAfterLeave"])
      ]),
      _: 1 /* STABLE */
    })
  ], 2 /* CLASS */))
}

script$4.render = render$4;
script$4.__file = "components/lib/datatable/ColumnFilter.vue";

var script$3 = {
    name: 'HeaderCheckbox',
    emits: ['change'],
    props: {
        checked: null,
        disabled: null,
        headerCheckboxIconTemplate: {
            type: Function,
            default: null
        }
    },
    data() {
        return {
            focused: false
        };
    },
    methods: {
        onClick(event) {
            if (!this.disabled) {
                this.$emit('change', {
                    originalEvent: event,
                    checked: !this.checked
                });

                DomHandler.focus(this.$refs.input);
            }
        },
        onFocus() {
            this.focused = true;
        },
        onBlur() {
            this.focused = false;
        }
    },
    computed: {
        headerCheckboxAriaLabel() {
            return this.$primevue.config.locale.aria ? (this.checked ? this.$primevue.config.locale.aria.selectAll : this.$primevue.config.locale.aria.unselectAll) : undefined;
        }
    },
    components: {
        CheckIcon: CheckIcon
    }
};

const _hoisted_1$3 = { class: "p-hidden-accessible" };
const _hoisted_2$3 = ["checked", "disabled", "tabindex", "aria-label"];

function render$3(_ctx, _cache, $props, $setup, $data, $options) {
  const _component_CheckIcon = resolveComponent("CheckIcon");

  return (openBlock(), createElementBlock("div", {
    class: normalizeClass(['p-checkbox p-component', { 'p-checkbox-focused': $data.focused, 'p-disabled': $props.disabled }]),
    onClick: _cache[2] || (_cache[2] = (...args) => ($options.onClick && $options.onClick(...args))),
    onKeydown: _cache[3] || (_cache[3] = withKeys(withModifiers((...args) => ($options.onClick && $options.onClick(...args)), ["prevent"]), ["space"]))
  }, [
    createElementVNode("div", _hoisted_1$3, [
      createElementVNode("input", {
        ref: "input",
        type: "checkbox",
        checked: $props.checked,
        disabled: $props.disabled,
        tabindex: $props.disabled ? null : '0',
        "aria-label": $options.headerCheckboxAriaLabel,
        onFocus: _cache[0] || (_cache[0] = $event => ($options.onFocus($event))),
        onBlur: _cache[1] || (_cache[1] = $event => ($options.onBlur($event)))
      }, null, 40 /* PROPS, HYDRATE_EVENTS */, _hoisted_2$3)
    ]),
    createElementVNode("div", {
      ref: "box",
      class: normalizeClass(['p-checkbox-box p-component', { 'p-highlight': $props.checked, 'p-disabled': $props.disabled, 'p-focus': $data.focused }])
    }, [
      ($props.headerCheckboxIconTemplate)
        ? (openBlock(), createBlock(resolveDynamicComponent($props.headerCheckboxIconTemplate), {
            key: 0,
            checked: $props.checked,
            class: "p-checkbox-icon"
          }, null, 8 /* PROPS */, ["checked"]))
        : (openBlock(), createBlock(_component_CheckIcon, {
            key: 1,
            class: "p-checkbox-icon"
          }))
    ], 2 /* CLASS */)
  ], 34 /* CLASS, HYDRATE_EVENTS */))
}

script$3.render = render$3;
script$3.__file = "components/lib/datatable/HeaderCheckbox.vue";

var script$2 = {
    name: 'HeaderCell',
    emits: [
        'column-click',
        'column-mousedown',
        'column-dragstart',
        'column-dragover',
        'column-dragleave',
        'column-drop',
        'column-resizestart',
        'checkbox-change',
        'filter-change',
        'filter-apply',
        'operator-change',
        'matchmode-change',
        'constraint-add',
        'constraint-remove',
        'filter-clear',
        'apply-click'
    ],
    props: {
        column: {
            type: Object,
            default: null
        },
        resizableColumns: {
            type: Boolean,
            default: false
        },
        groupRowsBy: {
            type: [Array, String, Function],
            default: null
        },
        sortMode: {
            type: String,
            default: 'single'
        },
        groupRowSortField: {
            type: [String, Function],
            default: null
        },
        sortField: {
            type: [String, Function],
            default: null
        },
        sortOrder: {
            type: Number,
            default: null
        },
        multiSortMeta: {
            type: Array,
            default: null
        },
        allRowsSelected: {
            type: Boolean,
            default: false
        },
        empty: {
            type: Boolean,
            default: false
        },
        filterDisplay: {
            type: String,
            default: null
        },
        filters: {
            type: Object,
            default: null
        },
        filtersStore: {
            type: Object,
            default: null
        },
        filterColumn: {
            type: Boolean,
            default: false
        },
        reorderableColumns: {
            type: Boolean,
            default: false
        },
        filterInputProps: {
            type: null,
            default: null
        },
        headerCheckboxIconTemplate: {
            type: Function,
            default: null
        }
    },
    data() {
        return {
            styleObject: {}
        };
    },
    mounted() {
        if (this.columnProp('frozen')) {
            this.updateStickyPosition();
        }
    },
    updated() {
        if (this.columnProp('frozen')) {
            this.updateStickyPosition();
        }
    },
    methods: {
        columnProp(prop) {
            return ObjectUtils.getVNodeProp(this.column, prop);
        },
        onClick(event) {
            this.$emit('column-click', { originalEvent: event, column: this.column });
        },
        onKeyDown(event) {
            if ((event.code === 'Enter' || event.code === 'Space') && event.currentTarget.nodeName === 'TH' && DomHandler.hasClass(event.currentTarget, 'p-sortable-column')) {
                this.$emit('column-click', { originalEvent: event, column: this.column });
                event.preventDefault();
            }
        },
        onMouseDown(event) {
            this.$emit('column-mousedown', { originalEvent: event, column: this.column });
        },
        onDragStart(event) {
            this.$emit('column-dragstart', event);
        },
        onDragOver(event) {
            this.$emit('column-dragover', event);
        },
        onDragLeave(event) {
            this.$emit('column-dragleave', event);
        },
        onDrop(event) {
            this.$emit('column-drop', event);
        },
        onResizeStart(event) {
            this.$emit('column-resizestart', event);
        },
        getMultiSortMetaIndex() {
            return this.multiSortMeta.findIndex((meta) => meta.field === this.columnProp('field') || meta.field === this.columnProp('sortField'));
        },
        getBadgeValue() {
            let index = this.getMultiSortMetaIndex();

            return this.groupRowsBy && this.groupRowsBy === this.groupRowSortField && index > -1 ? index : index + 1;
        },
        isMultiSorted() {
            return this.sortMode === 'multiple' && this.columnProp('sortable') && this.getMultiSortMetaIndex() > -1;
        },
        isColumnSorted() {
            return this.sortMode === 'single' ? this.sortField && (this.sortField === this.columnProp('field') || this.sortField === this.columnProp('sortField')) : this.isMultiSorted();
        },
        updateStickyPosition() {
            if (this.columnProp('frozen')) {
                let align = this.columnProp('alignFrozen');

                if (align === 'right') {
                    let right = 0;
                    let next = this.$el.nextElementSibling;

                    if (next) {
                        right = DomHandler.getOuterWidth(next) + parseFloat(next.style.right || 0);
                    }

                    this.styleObject.right = right + 'px';
                } else {
                    let left = 0;
                    let prev = this.$el.previousElementSibling;

                    if (prev) {
                        left = DomHandler.getOuterWidth(prev) + parseFloat(prev.style.left || 0);
                    }

                    this.styleObject.left = left + 'px';
                }

                let filterRow = this.$el.parentElement.nextElementSibling;

                if (filterRow) {
                    let index = DomHandler.index(this.$el);

                    filterRow.children[index].style.left = this.styleObject.left;
                    filterRow.children[index].style.right = this.styleObject.right;
                }
            }
        },
        onHeaderCheckboxChange(event) {
            this.$emit('checkbox-change', event);
        }
    },
    computed: {
        containerClass() {
            return [
                this.filterColumn ? this.columnProp('filterHeaderClass') : this.columnProp('headerClass'),
                this.columnProp('class'),
                {
                    'p-sortable-column': this.columnProp('sortable'),
                    'p-resizable-column': this.resizableColumns,
                    'p-highlight': this.isColumnSorted(),
                    'p-filter-column': this.filterColumn,
                    'p-frozen-column': this.columnProp('frozen'),
                    'p-reorderable-column': this.reorderableColumns
                }
            ];
        },
        containerStyle() {
             //增加列宽度的设置
             let width =  this.columnProp('width') || '';
            let widthStyle =(width==='') ? null : { width }  ;

            let headerStyle = this.filterColumn ? this.columnProp('filterHeaderStyle') : this.columnProp('headerStyle');
            let columnStyle = this.columnProp('style');

            return this.columnProp('frozen') ? [widthStyle, columnStyle, headerStyle, this.styleObject] : [widthStyle, columnStyle, headerStyle];
        },
        sortState() {
            let sorted = false;
            let sortOrder = null;

            if (this.sortMode === 'single') {
                sorted = this.sortField && (this.sortField === this.columnProp('field') || this.sortField === this.columnProp('sortField'));
                sortOrder = sorted ? this.sortOrder : 0;
            } else if (this.sortMode === 'multiple') {
                let metaIndex = this.getMultiSortMetaIndex();

                if (metaIndex > -1) {
                    sorted = true;
                    sortOrder = this.multiSortMeta[metaIndex].order;
                }
            }

            return {
                sorted,
                sortOrder
            };
        },
        sortableColumnIcon() {
            const { sorted, sortOrder } = this.sortState;

            if (!sorted) return SortAltIcon;
            else if (sorted && sortOrder > 0) return SortAmountUpAltIcon;
            else if (sorted && sortOrder < 0) return SortAmountDownIcon;

            return null;
        },
        ariaSort() {
            if (this.columnProp('sortable')) {
                const { sorted, sortOrder } = this.sortState;

                if (sorted && sortOrder < 0) return 'descending';
                else if (sorted && sortOrder > 0) return 'ascending';
                else return 'none';
            } else {
                return null;
            }
        }
    },
    components: {
        DTHeaderCheckbox: script$3,
        DTColumnFilter: script$4,
        SortAltIcon: SortAltIcon,
        SortAmountUpAltIcon: SortAmountUpAltIcon,
        SortAmountDownIcon: SortAmountDownIcon
    }
};

const _hoisted_1$2 = ["tabindex", "colspan", "rowspan", "aria-sort"];
const _hoisted_2$2 = { class: "p-column-header-content" };
const _hoisted_3$1 = {
  key: 1,
  class: "p-column-title"
};
const _hoisted_4$1 = { key: 2 };
const _hoisted_5$1 = {
  key: 3,
  class: "p-sortable-column-badge"
};

function render$2(_ctx, _cache, $props, $setup, $data, $options) {
  const _component_DTHeaderCheckbox = resolveComponent("DTHeaderCheckbox");
  const _component_DTColumnFilter = resolveComponent("DTColumnFilter");

  return (openBlock(), createElementBlock("th", {
    style: normalizeStyle($options.containerStyle),
    class: normalizeClass($options.containerClass),
    tabindex: $options.columnProp('sortable') ? '0' : null,
    colspan: $options.columnProp('colspan'),
    rowspan: $options.columnProp('rowspan'),
    "aria-sort": $options.ariaSort,
    onClick: _cache[8] || (_cache[8] = (...args) => ($options.onClick && $options.onClick(...args))),
    onKeydown: _cache[9] || (_cache[9] = (...args) => ($options.onKeyDown && $options.onKeyDown(...args))),
    onMousedown: _cache[10] || (_cache[10] = (...args) => ($options.onMouseDown && $options.onMouseDown(...args))),
    onDragstart: _cache[11] || (_cache[11] = (...args) => ($options.onDragStart && $options.onDragStart(...args))),
    onDragover: _cache[12] || (_cache[12] = (...args) => ($options.onDragOver && $options.onDragOver(...args))),
    onDragleave: _cache[13] || (_cache[13] = (...args) => ($options.onDragLeave && $options.onDragLeave(...args))),
    onDrop: _cache[14] || (_cache[14] = (...args) => ($options.onDrop && $options.onDrop(...args)))
  }, [
    ($props.resizableColumns && !$options.columnProp('frozen'))
      ? (openBlock(), createElementBlock("span", {
          key: 0,
          class: "p-column-resizer",
          onMousedown: _cache[0] || (_cache[0] = (...args) => ($options.onResizeStart && $options.onResizeStart(...args)))
        }, null, 32 /* HYDRATE_EVENTS */))
      : createCommentVNode("v-if", true),
    createElementVNode("div", _hoisted_2$2, [
      ($props.column.children && $props.column.children.header)
        ? (openBlock(), createBlock(resolveDynamicComponent($props.column.children.header), {
            key: 0,
            column: $props.column
          }, null, 8 /* PROPS */, ["column"]))
        : createCommentVNode("v-if", true),
      ($options.columnProp('header'))
        ? (openBlock(), createElementBlock("span", _hoisted_3$1, toDisplayString($options.columnProp('header')), 1 /* TEXT */))
        : createCommentVNode("v-if", true),
      ($options.columnProp('sortable'))
        ? (openBlock(), createElementBlock("span", _hoisted_4$1, [
            (openBlock(), createBlock(resolveDynamicComponent(($props.column.children && $props.column.children.sorticon) || $options.sortableColumnIcon), {
              sorted: $options.sortState.sorted,
              sortOrder: $options.sortState.sortOrder,
              class: "p-sortable-column-icon"
            }, null, 8 /* PROPS */, ["sorted", "sortOrder"]))
          ]))
        : createCommentVNode("v-if", true),
      ($options.isMultiSorted())
        ? (openBlock(), createElementBlock("span", _hoisted_5$1, toDisplayString($options.getBadgeValue()), 1 /* TEXT */))
        : createCommentVNode("v-if", true),
      ($options.columnProp('selectionMode') === 'multiple' && $props.filterDisplay !== 'row')
        ? (openBlock(), createBlock(_component_DTHeaderCheckbox, {
            key: 4,
            checked: $props.allRowsSelected,
            onChange: $options.onHeaderCheckboxChange,
            disabled: $props.empty,
            headerCheckboxIconTemplate: $props.headerCheckboxIconTemplate
          }, null, 8 /* PROPS */, ["checked", "onChange", "disabled", "headerCheckboxIconTemplate"]))
        : createCommentVNode("v-if", true),
      ($props.filterDisplay === 'menu' && $props.column.children && $props.column.children.filter)
        ? (openBlock(), createBlock(_component_DTColumnFilter, {
            key: 5,
            field: $options.columnProp('filterField') || $options.columnProp('field'),
            type: $options.columnProp('dataType'),
            display: "menu",
            showMenu: $options.columnProp('showFilterMenu'),
            filterElement: $props.column.children && $props.column.children.filter,
            filterHeaderTemplate: $props.column.children && $props.column.children.filterheader,
            filterFooterTemplate: $props.column.children && $props.column.children.filterfooter,
            filterClearTemplate: $props.column.children && $props.column.children.filterclear,
            filterApplyTemplate: $props.column.children && $props.column.children.filterapply,
            filterIconTemplate: $props.column.children && $props.column.children.filtericon,
            filterAddIconTemplate: $props.column.children && $props.column.children.filteraddicon,
            filterRemoveIconTemplate: $props.column.children && $props.column.children.filterremoveicon,
            filterClearIconTemplate: $props.column.children && $props.column.children.filterclearicon,
            filters: $props.filters,
            filtersStore: $props.filtersStore,
            filterInputProps: $props.filterInputProps,
            onFilterChange: _cache[1] || (_cache[1] = $event => (_ctx.$emit('filter-change', $event))),
            onFilterApply: _cache[2] || (_cache[2] = $event => (_ctx.$emit('filter-apply'))),
            filterMenuStyle: $options.columnProp('filterMenuStyle'),
            filterMenuClass: $options.columnProp('filterMenuClass'),
            showOperator: $options.columnProp('showFilterOperator'),
            showClearButton: $options.columnProp('showClearButton'),
            showApplyButton: $options.columnProp('showApplyButton'),
            showMatchModes: $options.columnProp('showFilterMatchModes'),
            showAddButton: $options.columnProp('showAddButton'),
            matchModeOptions: $options.columnProp('filterMatchModeOptions'),
            maxConstraints: $options.columnProp('maxConstraints'),
            onOperatorChange: _cache[3] || (_cache[3] = $event => (_ctx.$emit('operator-change', $event))),
            onMatchmodeChange: _cache[4] || (_cache[4] = $event => (_ctx.$emit('matchmode-change', $event))),
            onConstraintAdd: _cache[5] || (_cache[5] = $event => (_ctx.$emit('constraint-add', $event))),
            onConstraintRemove: _cache[6] || (_cache[6] = $event => (_ctx.$emit('constraint-remove', $event))),
            onApplyClick: _cache[7] || (_cache[7] = $event => (_ctx.$emit('apply-click', $event)))
          }, null, 8 /* PROPS */, ["field", "type", "showMenu", "filterElement", "filterHeaderTemplate", "filterFooterTemplate", "filterClearTemplate", "filterApplyTemplate", "filterIconTemplate", "filterAddIconTemplate", "filterRemoveIconTemplate", "filterClearIconTemplate", "filters", "filtersStore", "filterInputProps", "filterMenuStyle", "filterMenuClass", "showOperator", "showClearButton", "showApplyButton", "showMatchModes", "showAddButton", "matchModeOptions", "maxConstraints"]))
        : createCommentVNode("v-if", true)
    ])
  ], 46 /* CLASS, STYLE, PROPS, HYDRATE_EVENTS */, _hoisted_1$2))
}

script$2.render = render$2;
script$2.__file = "components/lib/datatable/HeaderCell.vue";

var script$1 = {
    name: 'TableHeader',
    emits: [
        'column-click',
        'column-mousedown',
        'column-dragstart',
        'column-dragover',
        'column-dragleave',
        'column-drop',
        'column-resizestart',
        'checkbox-change',
        'filter-change',
        'filter-apply',
        'operator-change',
        'matchmode-change',
        'constraint-add',
        'constraint-remove',
        'filter-clear',
        'apply-click'
    ],
    props: {
        columnGroup: {
            type: null,
            default: null
        },
        columns: {
            type: null,
            default: null
        },
        rowGroupMode: {
            type: String,
            default: null
        },
        groupRowsBy: {
            type: [Array, String, Function],
            default: null
        },
        resizableColumns: {
            type: Boolean,
            default: false
        },
        allRowsSelected: {
            type: Boolean,
            default: false
        },
        empty: {
            type: Boolean,
            default: false
        },
        sortMode: {
            type: String,
            default: 'single'
        },
        groupRowSortField: {
            type: [String, Function],
            default: null
        },
        sortField: {
            type: [String, Function],
            default: null
        },
        sortOrder: {
            type: Number,
            default: null
        },
        multiSortMeta: {
            type: Array,
            default: null
        },
        filterDisplay: {
            type: String,
            default: null
        },
        filters: {
            type: Object,
            default: null
        },
        filtersStore: {
            type: Object,
            default: null
        },
        reorderableColumns: {
            type: Boolean,
            default: false
        },
        filterInputProps: {
            type: null,
            default: null
        },
        headerCheckboxIconTemplate: {
            type: Function,
            default: null
        }
    },
    methods: {
        columnProp(col, prop) {
            return ObjectUtils.getVNodeProp(col, prop);
        },
        getFilterColumnHeaderClass(column) {
            return [
                'p-filter-column',
                this.columnProp(column, 'filterHeaderClass'),
                this.columnProp(column, 'class'),
                {
                    'p-frozen-column': this.columnProp(column, 'frozen')
                }
            ];
        },
        getFilterColumnHeaderStyle(column) {
            return [this.columnProp(column, 'filterHeaderStyle'), this.columnProp(column, 'style')];
        },
        getHeaderRows() {
            let rows = [];

            let columnGroup = this.columnGroup;

            if (columnGroup.children && columnGroup.children.default) {
                for (let child of columnGroup.children.default()) {
                    if (child.type.name === 'Row') {
                        rows.push(child);
                    } else if (child.children && child.children instanceof Array) {
                        rows = child.children;
                    }
                }

                return rows;
            }
        },
        getHeaderColumns(row) {
            let cols = [];

            if (row.children && row.children.default) {
                row.children.default().forEach((child) => {
                    if (child.children && child.children instanceof Array) cols = [...cols, ...child.children];
                    else if (child.type.name === 'Column') cols.push(child);
                });

                return cols;
            }
        }
    },
    components: {
        DTHeaderCell: script$2,
        DTHeaderCheckbox: script$3,
        DTColumnFilter: script$4
    }
};

const _hoisted_1$1 = {
  class: "p-datatable-thead",
  role: "rowgroup"
};
const _hoisted_2$1 = { key: 0 };

function render$1(_ctx, _cache, $props, $setup, $data, $options) {
  const _component_DTHeaderCell = resolveComponent("DTHeaderCell");
  const _component_DTHeaderCheckbox = resolveComponent("DTHeaderCheckbox");
  const _component_DTColumnFilter = resolveComponent("DTColumnFilter");

  return (openBlock(), createElementBlock("thead", _hoisted_1$1, [
    (!$props.columnGroup)
      ? (openBlock(), createElementBlock(Fragment, { key: 0 }, [
          createElementVNode("tr", null, [
            (openBlock(true), createElementBlock(Fragment, null, renderList($props.columns, (col, i) => {
              return (openBlock(), createElementBlock(Fragment, {
                key: $options.columnProp(col, 'columnKey') || $options.columnProp(col, 'field') || i
              }, [
                (!$options.columnProp(col, 'hidden') && ($props.rowGroupMode !== 'subheader' || $props.groupRowsBy !== $options.columnProp(col, 'field')))
                  ? (openBlock(), createBlock(_component_DTHeaderCell, {
                      key: 0,
                      column: col,
                      onColumnClick: _cache[0] || (_cache[0] = $event => (_ctx.$emit('column-click', $event))),
                      onColumnMousedown: _cache[1] || (_cache[1] = $event => (_ctx.$emit('column-mousedown', $event))),
                      onColumnDragstart: _cache[2] || (_cache[2] = $event => (_ctx.$emit('column-dragstart', $event))),
                      onColumnDragover: _cache[3] || (_cache[3] = $event => (_ctx.$emit('column-dragover', $event))),
                      onColumnDragleave: _cache[4] || (_cache[4] = $event => (_ctx.$emit('column-dragleave', $event))),
                      onColumnDrop: _cache[5] || (_cache[5] = $event => (_ctx.$emit('column-drop', $event))),
                      groupRowsBy: $props.groupRowsBy,
                      groupRowSortField: $props.groupRowSortField,
                      reorderableColumns: $props.reorderableColumns,
                      resizableColumns: $props.resizableColumns,
                      onColumnResizestart: _cache[6] || (_cache[6] = $event => (_ctx.$emit('column-resizestart', $event))),
                      sortMode: $props.sortMode,
                      sortField: $props.sortField,
                      sortOrder: $props.sortOrder,
                      multiSortMeta: $props.multiSortMeta,
                      allRowsSelected: $props.allRowsSelected,
                      empty: $props.empty,
                      onCheckboxChange: _cache[7] || (_cache[7] = $event => (_ctx.$emit('checkbox-change', $event))),
                      filters: $props.filters,
                      filterDisplay: $props.filterDisplay,
                      filtersStore: $props.filtersStore,
                      filterInputProps: $props.filterInputProps,
                      onFilterChange: _cache[8] || (_cache[8] = $event => (_ctx.$emit('filter-change', $event))),
                      onFilterApply: _cache[9] || (_cache[9] = $event => (_ctx.$emit('filter-apply'))),
                      onOperatorChange: _cache[10] || (_cache[10] = $event => (_ctx.$emit('operator-change', $event))),
                      onMatchmodeChange: _cache[11] || (_cache[11] = $event => (_ctx.$emit('matchmode-change', $event))),
                      onConstraintAdd: _cache[12] || (_cache[12] = $event => (_ctx.$emit('constraint-add', $event))),
                      onConstraintRemove: _cache[13] || (_cache[13] = $event => (_ctx.$emit('constraint-remove', $event))),
                      onApplyClick: _cache[14] || (_cache[14] = $event => (_ctx.$emit('apply-click', $event))),
                      headerCheckboxIconTemplate: $props.headerCheckboxIconTemplate
                    }, null, 8 /* PROPS */, ["column", "groupRowsBy", "groupRowSortField", "reorderableColumns", "resizableColumns", "sortMode", "sortField", "sortOrder", "multiSortMeta", "allRowsSelected", "empty", "filters", "filterDisplay", "filtersStore", "filterInputProps", "headerCheckboxIconTemplate"]))
                  : createCommentVNode("v-if", true)
              ], 64 /* STABLE_FRAGMENT */))
            }), 128 /* KEYED_FRAGMENT */))
          ]),
          ($props.filterDisplay === 'row')
            ? (openBlock(), createElementBlock("tr", _hoisted_2$1, [
                (openBlock(true), createElementBlock(Fragment, null, renderList($props.columns, (col, i) => {
                  return (openBlock(), createElementBlock(Fragment, {
                    key: $options.columnProp(col, 'columnKey') || $options.columnProp(col, 'field') || i
                  }, [
                    (!$options.columnProp(col, 'hidden') && ($props.rowGroupMode !== 'subheader' || $props.groupRowsBy !== $options.columnProp(col, 'field')))
                      ? (openBlock(), createElementBlock("th", {
                          key: 0,
                          style: normalizeStyle($options.getFilterColumnHeaderStyle(col)),
                          class: normalizeClass($options.getFilterColumnHeaderClass(col))
                        }, [
                          ($options.columnProp(col, 'selectionMode') === 'multiple' )
                            ? (openBlock(), createBlock(_component_DTHeaderCheckbox, {
                                key: 0,
                                checked: $props.allRowsSelected,
                                disabled: $props.empty,
                                onChange: _cache[15] || (_cache[15] = $event => (_ctx.$emit('checkbox-change', $event)))
                              }, null, 8 /* PROPS */, ["checked", "disabled"]))
                            : createCommentVNode("v-if", true),
                          (col.children && col.children.filter)
                            ? (openBlock(), createBlock(_component_DTColumnFilter, {
                                key: 1,
                                field: $options.columnProp(col, 'filterField') || $options.columnProp(col, 'field'),
                                type: $options.columnProp(col, 'dataType'),
                                display: "row",
                                showMenu: $options.columnProp(col, 'showFilterMenu'),
                                filterElement: col.children && col.children.filter,
                                filterHeaderTemplate: col.children && col.children.filterheader,
                                filterFooterTemplate: col.children && col.children.filterfooter,
                                filterClearTemplate: col.children && col.children.filterclear,
                                filterApplyTemplate: col.children && col.children.filterapply,
                                filterIconTemplate: col.children && col.children.filtericon,
                                filterAddIconTemplate: col.children && col.children.filteraddicon,
                                filterRemoveIconTemplate: col.children && col.children.filterremoveicon,
                                filterClearIconTemplate: col.children && col.children.filterclearicon,
                                filters: $props.filters,
                                filtersStore: $props.filtersStore,
                                filterInputProps: $props.filterInputProps,
                                onFilterChange: _cache[16] || (_cache[16] = $event => (_ctx.$emit('filter-change', $event))),
                                onFilterApply: _cache[17] || (_cache[17] = $event => (_ctx.$emit('filter-apply'))),
                                filterMenuStyle: $options.columnProp(col, 'filterMenuStyle'),
                                filterMenuClass: $options.columnProp(col, 'filterMenuClass'),
                                showOperator: $options.columnProp(col, 'showFilterOperator'),
                                showClearButton: $options.columnProp(col, 'showClearButton'),
                                showApplyButton: $options.columnProp(col, 'showApplyButton'),
                                showMatchModes: $options.columnProp(col, 'showFilterMatchModes'),
                                showAddButton: $options.columnProp(col, 'showAddButton'),
                                matchModeOptions: $options.columnProp(col, 'filterMatchModeOptions'),
                                maxConstraints: $options.columnProp(col, 'maxConstraints'),
                                onOperatorChange: _cache[18] || (_cache[18] = $event => (_ctx.$emit('operator-change', $event))),
                                onMatchmodeChange: _cache[19] || (_cache[19] = $event => (_ctx.$emit('matchmode-change', $event))),
                                onConstraintAdd: _cache[20] || (_cache[20] = $event => (_ctx.$emit('constraint-add', $event))),
                                onConstraintRemove: _cache[21] || (_cache[21] = $event => (_ctx.$emit('constraint-remove', $event))),
                                onApplyClick: _cache[22] || (_cache[22] = $event => (_ctx.$emit('apply-click', $event)))
                              }, null, 8 /* PROPS */, ["field", "type", "showMenu", "filterElement", "filterHeaderTemplate", "filterFooterTemplate", "filterClearTemplate", "filterApplyTemplate", "filterIconTemplate", "filterAddIconTemplate", "filterRemoveIconTemplate", "filterClearIconTemplate", "filters", "filtersStore", "filterInputProps", "filterMenuStyle", "filterMenuClass", "showOperator", "showClearButton", "showApplyButton", "showMatchModes", "showAddButton", "matchModeOptions", "maxConstraints"]))
                            : createCommentVNode("v-if", true)
                        ], 6 /* CLASS, STYLE */))
                      : createCommentVNode("v-if", true)
                  ], 64 /* STABLE_FRAGMENT */))
                }), 128 /* KEYED_FRAGMENT */))
              ]))
            : createCommentVNode("v-if", true)
        ], 64 /* STABLE_FRAGMENT */))
      : (openBlock(true), createElementBlock(Fragment, { key: 1 }, renderList($options.getHeaderRows(), (row, i) => {
          return (openBlock(), createElementBlock("tr", { key: i }, [
            (openBlock(true), createElementBlock(Fragment, null, renderList($options.getHeaderColumns(row), (col, j) => {
              return (openBlock(), createElementBlock(Fragment, {
                key: $options.columnProp(col, 'columnKey') || $options.columnProp(col, 'field') || j
              }, [
                (!$options.columnProp(col, 'hidden') && ($props.rowGroupMode !== 'subheader' || $props.groupRowsBy !== $options.columnProp(col, 'field')) && typeof col.children !== 'string')
                  ? (openBlock(), createBlock(_component_DTHeaderCell, {
                      key: 0,
                      column: col,
                      onColumnClick: _cache[23] || (_cache[23] = $event => (_ctx.$emit('column-click', $event))),
                      onColumnMousedown: _cache[24] || (_cache[24] = $event => (_ctx.$emit('column-mousedown', $event))),
                      groupRowsBy: $props.groupRowsBy,
                      groupRowSortField: $props.groupRowSortField,
                      sortMode: $props.sortMode,
                      sortField: $props.sortField,
                      sortOrder: $props.sortOrder,
                      multiSortMeta: $props.multiSortMeta,
                      allRowsSelected: $props.allRowsSelected,
                      empty: $props.empty,
                      onCheckboxChange: _cache[25] || (_cache[25] = $event => (_ctx.$emit('checkbox-change', $event))),
                      filters: $props.filters,
                      filterDisplay: $props.filterDisplay,
                      filtersStore: $props.filtersStore,
                      onFilterChange: _cache[26] || (_cache[26] = $event => (_ctx.$emit('filter-change', $event))),
                      onFilterApply: _cache[27] || (_cache[27] = $event => (_ctx.$emit('filter-apply'))),
                      onOperatorChange: _cache[28] || (_cache[28] = $event => (_ctx.$emit('operator-change', $event))),
                      onMatchmodeChange: _cache[29] || (_cache[29] = $event => (_ctx.$emit('matchmode-change', $event))),
                      onConstraintAdd: _cache[30] || (_cache[30] = $event => (_ctx.$emit('constraint-add', $event))),
                      onConstraintRemove: _cache[31] || (_cache[31] = $event => (_ctx.$emit('constraint-remove', $event))),
                      onApplyClick: _cache[32] || (_cache[32] = $event => (_ctx.$emit('apply-click', $event))),
                      headerCheckboxIconTemplate: $props.headerCheckboxIconTemplate
                    }, null, 8 /* PROPS */, ["column", "groupRowsBy", "groupRowSortField", "sortMode", "sortField", "sortOrder", "multiSortMeta", "allRowsSelected", "empty", "filters", "filterDisplay", "filtersStore", "headerCheckboxIconTemplate"]))
                  : createCommentVNode("v-if", true)
              ], 64 /* STABLE_FRAGMENT */))
            }), 128 /* KEYED_FRAGMENT */))
          ]))
        }), 128 /* KEYED_FRAGMENT */))
  ]))
}

script$1.render = render$1;
script$1.__file = "components/lib/datatable/TableHeader.vue";

var script = {
    name: 'DataTable',
    emits: [
        'value-change',
        'page-change',
        'sort',
        'filter',
        'row-click',
        'row-dblclick',
        'row-focus-changed',
        'row-select-changed',
        'update:contextMenuSelection',
        'row-contextmenu',
        'row-unselect-all',
        'row-select-all',
        'select-all-change',
        'column-resize-end',
        'column-reorder',
        'row-reorder',
        'update:expandedRows',
        'row-collapse',
        'row-expand',
        'update:expandedRowGroups',
        'rowgroup-collapse',
        'rowgroup-expand',
        'update:filters',
        'state-restore',
        'state-save',
        'cell-edit-init',
        'cell-edit-complete',
        'cell-edit-cancel',
        'row-new-init',
        'item-change'
       
    ],
    props: {
        paginator: {
            type: Boolean,
            default: false
        },

        pageSize:{
            type:Number,
            default:20
        },

        paginatorTemplate: {
            type: [Object, String],
            default: 'FirstPageLink PrevPageLink PageLinks NextPageLink LastPageLink CurrentPageReport RowsPerPageDropdown'
        },
        pageLinkSize: {
            type: Number,
            default: 5
        },
        rowsPerPageOptions: {
            type: Array,
            default:()=>[20,50,100,200,500,1000] // 解决直接定义为数组时报以下错误 Type of the default value for 'rowsPerPageOptions' prop must be a function 
        },

        currentPageReportTemplate: {
            type: String,
            default: ' {currentPage} / {pageCount}'
        },

        alwaysShowPaginator: {
            type: Boolean,
            default: true
        },

        loading: {
            type: Boolean,
            default: false
        },
        loadingIcon: {
            type: String,
            default: undefined
        },
        sortField: {
            type: [String, Function],
            default: null
        },
        sortOrder: {
            type: Number,
            default: null
        },
        defaultSortOrder: {
            type: Number,
            default: 1
        },
        multiSortMeta: {
            type: Array,
            default: null
        },
        sortMode: {
            type: String,
            default: 'single'
        },
        removableSort: {
            type: Boolean,
            default: false
        },
        filters: {
            type: Object,
            default: null
        },
        filterDisplay: {
            type: String,
            default: null
        },
        globalFilterFields: {
            type: Array,
            default: null
        },
        filterLocale: {
            type: String,
            default: undefined
        },
      
        selectionMode: {
            type: String,
            default: null
        },
        compareSelectionBy: {
            type: String,
            default: 'deepEquals'
        },
        metaKeySelection: {
            type: Boolean,
            default: true
        },
        contextMenu: {
            type: Boolean,
            default: false
        },
        contextMenuSelection: {
            type: Object,
            default: null
        },

        rowHover: {
            type: Boolean,
            default: false
        },

        currentRowHighlight:{
            type: Boolean,
            default: true
        },

        csvSeparator: {
            type: String,
            default: ','
        },
        exportFilename: {
            type: String,
            default: 'download'
        },
        exportFunction: {
            type: Function,
            default: null
        },
        resizableColumns: {
            type: Boolean,
            default: false
        },
        columnResizeMode: {
            type: String,
            default: 'fit'
        },
        reorderableColumns: {
            type: Boolean,
            default: false
        },
        expandedRows: {
            type: Array,
            default: null
        },
        expandedRowIcon: {
            type: String,
            default: undefined
        },
        collapsedRowIcon: {
            type: String,
            default: undefined
        },
        rowGroupMode: {
            type: String,
            default: null
        },
        groupRowsBy: {
            type: [Array, String, Function],
            default: null
        },
        expandableRowGroups: {
            type: Boolean,
            default: false
        },
        expandedRowGroups: {
            type: Array,
            default: null
        },
        stateStorage: {
            type: String,
            default: 'session'
        },
        stateKey: {
            type: String,
            default: null
        },
       
        rowClass: {
            type: null,
            default: null
        },
        rowStyle: {
            type: null,
            default: null
        },
        scrollable: {
            type: Boolean,
            default: true   /*默认改为滚动显示*/
        },
        
        scrollHeight: {
            type: String,
            default: null
        },
        
        showGridlines: {
            type: Boolean,
            default: false
        },
        stripedRows: {
            type: Boolean,
            default: false
        },
        tableStyle: {
            type: null,
            default: null
        },
        tableClass: {
            type: String,
            default: null
        },
        tableProps: {
            type: null,
            default: null
        },
        filterInputProps: {
            type: null,
            default: null
        },

        /*按回车是跳入下一行还是下一列*/
        enterToNextRow:{
            type: Boolean,
            default: false
        },
        
        disabled:{
             type:Boolean,
             default:false
         },
        readonly:{
            type:Boolean,
            default:false
        }

    },

    provide(){
        return {
               "DataTable":computed(()=> {
                return {
                    "enterToNextRow":this.enterToNextRow,
                    "primaryData": this.d_primaryData,
                    "evaluate":  this.evaluate
                }
            })
        }},

    data() {
        return {
            d_primaryData:null,
            d_currentRow: -1 ,  /*初始值设置为-1,这样才能通过watch 去触发row-focus-changed事件 */
            d_pageParams:{
                current: 1,
                size: this.pageSize ,
                total: 0
            },
            d_sortField: this.sortField,
            d_sortOrder: (this.sortOrder===null) ? this.defaultSortOrder: this.sortOrder ,
            d_multiSortMeta: this.multiSortMeta ? [...this.multiSortMeta] : [],
            d_groupRowsSortMeta: null,
            d_expandedRowKeys: null,
            d_columnOrder: null,
            d_filters: this.cloneFilters(this.filters),
            d_modified:false,
        };
    },
    uniqueRowId:0,
    lastFocusRowId:-1,
    filteredData:null,
    deletedData:null,
    rowTouched: false,
    anchorRowIndex: null,
    rangeRowIndex: null,
    documentColumnResizeListener: null,
    documentColumnResizeEndListener: null,
    lastResizeHelperX: null,
    resizeColumnElement: null,
    columnResizing: false,
    colReorderIconWidth: null,
    colReorderIconHeight: null,
    draggedColumn: null,
    draggedRowIndex: null,
    droppedRowIndex: null,
    rowDragging: null,
    columnWidthsState: null,
    tableWidthState: null,
    columnWidthsRestored: false,

    watch: {
        d_currentRow(newValue){
            if(this.d_primaryData && newValue>=0 && newValue< this.d_primaryData.length){
                this.rowFocusChanged(newValue);
            }
        },

        // sortField(newValue) {
        //     this.d_sortField = newValue;
        // },
        // sortOrder(newValue) {
        //     this.d_sortOrder = newValue;
        // },
        // multiSortMeta(newValue) {
        //     this.d_multiSortMeta = newValue;
        // },
        
        d_sortField(newValue){
           this.sort();
        },
        d_sortOrder(newValue) {
            this.sort();
        },
        // d_multiSortMeta(newValue) {
        //     this.$nextTick(()=>this.sort());
        // },

        d_filters(newValue){
            this.filter();
        },
        // filters: {
        //     deep: true,
        //     handler: function (newValue) {
        //         this.d_filters = this.cloneFilters(newValue);
        //     }
        // }

        pageSize(newValue){
            this.d_pageParams.size = newValue;
        }
    },

    beforeMount() {
        this.uniqueRowId=0;
        this.lastFocusRowId=-1;

        if (this.isStateful()) {
            this.restoreState();
        }
    },
    mounted() {
        this.$el.setAttribute(this.attributeSelector, '');

        if (this.isStateful() && this.resizableColumns) {
            this.restoreColumnWidths();
        }
    },
    beforeUnmount() {
        this.unbindColumnResizeEvents();
        this.destroyStyleElement();
    },

    updated() {
        if (this.isStateful()) {
            this.saveState();
        }
    },
    methods: {
        columnProp(col, prop) {
            return ObjectUtils.getVNodeProp(col, prop);
        },
        onPageChange(event) {

            const {size,total, current} =event;

            this.d_pageParams = { size, total, current};
            this.$emit('page-change', this.d_pageParams);
        },

        onColumnHeaderClick(e) {
            const event = e.originalEvent;
            const column = e.column;

            if (this.columnProp(column, 'sortable')) {
                const targetNode = event.target;
                const columnField = this.columnProp(column, 'sortField') || this.columnProp(column, 'field');

                if (
                    DomHandler.hasClass(targetNode, 'p-sortable-column') ||
                    DomHandler.hasClass(targetNode, 'p-column-title') ||
                    DomHandler.hasClass(targetNode, 'p-column-header-content') ||
                    DomHandler.hasClass(targetNode, 'p-sortable-column-icon') ||
                    DomHandler.hasClass(targetNode.parentElement, 'p-sortable-column-icon')
                ) {
                    DomHandler.clearSelection();

                    if (this.sortMode === 'single') {
                        if (this.d_sortField === columnField) {
                            if (this.removableSort && this.d_sortOrder * -1 === this.defaultSortOrder) {
                                this.d_sortOrder = null;
                                this.d_sortField = null;
                            } else {
                                this.d_sortOrder = this.d_sortOrder * -1;
                            }
                        } else {
                            this.d_sortOrder = this.defaultSortOrder;
                            this.d_sortField = columnField;
                        }

                       this.resetPage();

                    } else if (this.sortMode === 'multiple') {
                        let metaKey = event.metaKey || event.ctrlKey;

                        if (!metaKey) {
                            this.d_multiSortMeta = this.d_multiSortMeta.filter((meta) => meta.field === columnField);
                        }

                        this.addMultiSortField(columnField);
                     }
                }
            }
        },
        sortSingle(value) {
            if (this.groupRowsBy && this.groupRowsBy === this.sortField) {
                this.d_multiSortMeta = [
                    { field: this.sortField, order: this.sortOrder || this.defaultSortOrder },
                    { field: this.d_sortField, order: this.d_sortOrder }
                ];

                return this.sortMultiple(value);
            }

            let data = [...value];
            
            data.sort((data1, data2) => {
                let value1 = ObjectUtils.resolveFieldData(data1, this.d_sortField);
                let value2 = ObjectUtils.resolveFieldData(data2, this.d_sortField);

                let result = null;

                if (value1 == null && value2 != null) result = -1;
                else if (value1 != null && value2 == null) result = 1;
                else if (value1 == null && value2 == null) result = 0;
                else if (typeof value1 === 'string' && typeof value2 === 'string') result = value1.localeCompare(value2, undefined, { numeric: true });
                else result = value1 < value2 ? -1 : value1 > value2 ? 1 : 0;

                return this.d_sortOrder * result;
            });

            return data;
        },
        sortMultiple(value) {

            if (this.groupRowsBy && (this.d_groupRowsSortMeta || (this.d_multiSortMeta.length && this.groupRowsBy === this.d_multiSortMeta[0].field))) {
                const firstSortMeta = this.d_multiSortMeta[0];

                !this.d_groupRowsSortMeta && (this.d_groupRowsSortMeta = firstSortMeta);

                if (firstSortMeta.field !== this.d_groupRowsSortMeta.field) {
                    this.d_multiSortMeta = [this.d_groupRowsSortMeta, ...this.d_multiSortMeta];
                }
            }

            let data = [...value];

            data.sort((data1, data2) => {
                return this.multisortField(data1, data2, 0);
            });

            return data;
        },
        multisortField(data1, data2, index) {
            const value1 = ObjectUtils.resolveFieldData(data1, this.d_multiSortMeta[index].field);
            const value2 = ObjectUtils.resolveFieldData(data2, this.d_multiSortMeta[index].field);
            let result = null;

            if (typeof value1 === 'string' || value1 instanceof String) {
                if (value1.localeCompare && value1 !== value2) {
                    return this.d_multiSortMeta[index].order * value1.localeCompare(value2, undefined, { numeric: true });
                }
            } else {
                result = value1 < value2 ? -1 : 1;
            }

            if (value1 === value2) {
                return this.d_multiSortMeta.length - 1 > index ? this.multisortField(data1, data2, index + 1) : 0;
            }

            return this.d_multiSortMeta[index].order * result;
        },
        addMultiSortField(field) {
            let index = this.d_multiSortMeta.findIndex((meta) => meta.field === field);

            if (index >= 0) {
                if (this.removableSort && this.d_multiSortMeta[index].order * -1 === this.defaultSortOrder) this.d_multiSortMeta.splice(index, 1);
                else this.d_multiSortMeta[index] = { field: field, order: this.d_multiSortMeta[index].order * -1 };
            } else {
                this.d_multiSortMeta.push({ field: field, order: this.defaultSortOrder });
            }

            this.d_multiSortMeta = [...this.d_multiSortMeta];
        },
        
        sort(){
            if(!this.d_primaryData || this.d_primaryData.length ===0 )
                return;

            if (this.sorted) {
               const rowId = this.getRowId();

               if (this.sortMode === 'single')
                   this.d_primaryData = this.sortSingle(this.d_primaryData );
               else if (this.sortMode === 'multiple') 
                   this.d_primaryData=this.sortMultiple(this.d_primaryData );

                if(rowId){
                  this.setRowByRowId(rowId);
                }else {
                    this.d_currentRow=0;
                }

                this.rowFocusChanged(this.d_currentRow);
            }
        },

        filter() {
            const primaryData= this.d_primaryData || [];
            const filteredData = this.filteredData || [];
            let data=[...primaryData ,...filteredData];
            
            if(!data || data.length===0)
                return;

            let globalFilterFieldsArray;

            if (this.filters['global']) {
                globalFilterFieldsArray = this.globalFilterFields || this.columns.map((col) => this.columnProp(col, 'filterField') || this.columnProp(col, 'field'));
            }

            let primaryValue=[];
            let filteredValue = [];
            let rowId = this.getRowId();

            for (let i = 0; i < data.length; i++) {
                let localMatch = true;
                let globalMatch = false;
                let localFiltered = false;

                for (let prop in this.filters) {
                    if (Object.prototype.hasOwnProperty.call(this.filters, prop) && prop !== 'global') {
                        localFiltered = true;
                        let filterField = prop;
                        let filterMeta = this.filters[filterField];

                        if (filterMeta.operator) {
                            for (let filterConstraint of filterMeta.constraints) {
                                localMatch = this.executeLocalFilter(filterField, data[i], filterConstraint);

                                if ((filterMeta.operator === FilterOperator.OR && localMatch) || (filterMeta.operator === FilterOperator.AND && !localMatch)) {
                                    break;
                                }
                            }
                        } else {
                            localMatch = this.executeLocalFilter(filterField, data[i], filterMeta);
                        }

                        if (!localMatch) {
                            break;
                        }
                    }
                }

                if (this.filters['global'] && !globalMatch && globalFilterFieldsArray) {
                    for (let j = 0; j < globalFilterFieldsArray.length; j++) {
                        let globalFilterField = globalFilterFieldsArray[j];

                        globalMatch = FilterService.filters[this.filters['global'].matchMode || FilterMatchMode.CONTAINS](ObjectUtils.resolveFieldData(data[i], globalFilterField), this.filters['global'].value, this.filterLocale);

                        if (globalMatch) {
                            break;
                        }
                    }
                }

                let matches;

                if (this.filters['global']) {
                    matches = localFiltered ? localFiltered && localMatch && globalMatch : globalMatch;
                } else {
                    matches = localFiltered && localMatch;
                }

                if (matches) {
                    primaryValue.push(data[i]);
                }else {
                    filteredValue.push(data[i]);
                }
            }

            if (this.sorted && primaryValue && primaryValue.length>0 ) {
               if (this.sortMode === 'single')
                    primaryValue = this.sortSingle(primaryValue);
               else if (this.sortMode === 'multiple') 
                    primaryValue=this.sortMultiple(primaryValue);
            }

            this.filteredData= filteredValue;
            this.d_primaryData = primaryValue;

            if(rowId){
                this.setCurrentRowByRowId(rowId);
            }
            else {
                this.d_currentRow =0; 
            }

            this.rowFocusChanged(this.d_currentRow);
        },

        executeLocalFilter(field, rowData, filterMeta) {
            let filterValue = filterMeta.value;
            let filterMatchMode = filterMeta.matchMode || FilterMatchMode.STARTS_WITH;
            let dataFieldValue = ObjectUtils.resolveFieldData(rowData, field);
            let filterConstraint = FilterService.filters[filterMatchMode];

            return filterConstraint(dataFieldValue, filterValue, this.filterLocale);
        },

        onRowClick(e) {
           const event = e.originalEvent;
           const rowData = e.data;
           const rowIndex = e.index;
           const { x=0, y=0} = event;


           if (!this.d_primaryData ||  !rowData || DomHandler.isClickable(event.target)) {
               return;
           }
           
           if(this.d_currentRow!==rowIndex){
              this.d_currentRow= rowIndex;
           }
          
           if(x!==0 || y!==0)  //如果x和y都是0,说明是用 DomHandle.invokeElementMethod 执行的click方法
              this.$emit('row-click', e);

           if(!this.selectionMode)
                return;

           const metaSelection = this.rowTouched ? false : this.metaKeySelection;
           const metaKey = event.metaKey || event.ctrlKey;
           const selected = this.isSelected(rowData);

           if (this.isMultipleSelectionMode() && metaSelection && event.shiftKey && this.anchorRowIndex != null) {
                DomHandler.clearSelection();
                this.rangeRowIndex = rowIndex;
                this.selectRange(event);
           }
           else {
                 this.anchorRowIndex = rowIndex;
                 this.rangeRowIndex = rowIndex;

                if (metaSelection && this.isMultipleSelectionMode() ) {
                   if (metaKey) {
                         rowData['isSelected']=!selected;
                    } else {
                       this.doUnSelectAll();
                       rowData['isSelected'] = true;
                   }
                } 
                else {
                    if (this.selectionMode === 'single' && !selected ) {
                        let selectedRow=this.d_primaryData.find((item,index)=> index !=rowIndex && item['isSelected']  );

                        if(selectedRow){
                            selectedRow['isSelected']=false;
                        }

                        rowData['isSelected']=true;
                    }
                    else if (this.selectionMode === 'multiple') {
                        rowData['isSelected']= !selected;
                    }
                }
            }

            if(rowData['isSelected']!==selected){
                this.$emit('row-select-changed',rowData);
            }

            this.rowTouched = false;
        },

        onRowDblClick(e) {
            const event = e.originalEvent;

            if (DomHandler.isClickable(event.target)) {
                return;
            }

            this.$emit('row-dblclick', e);
        },

        onRowRightClick(event) {
            DomHandler.clearSelection();
            event.originalEvent.target.focus();

            this.$emit('update:contextMenuSelection', event.data);
            this.$emit('row-contextmenu', event);
        },

        onRowTouchEnd() {
            this.rowTouched = true;
        },
      
        onArrowDownKey(event, frozenRow ) {
            let data= this.getAllRenderRowData() || [];

            if(this.d_currentRow>=(data.length -1)){
                event.preventDefault();
                event.stopPropagation();

                return;
            }

            this.d_currentRow++;

            if (event.shiftKey && this.isMultipleSelectionMode() ) {
                const rowIndex = this.d_currentRow;

                this.onRowClick({ originalEvent: event, data: data[rowIndex], index: rowIndex });
            }

            event.preventDefault();
            event.stopPropagation();
        },

        onArrowUpKey(event, frozenRow) {
            if(this.d_currentRow<=0) {
                event.preventDefault();
                event.stopPropagation();

                return;
            }
           
            this.d_currentRow -- ;

            if (event.shiftKey && this.isMultipleSelectionMode() ) {
                let data= this.getAllRenderRowData() || [];
                const rowIndex = this.d_currentRow;

                 this.onRowClick({ originalEvent: event, data: data[rowIndex], index: rowIndex });
            }

            event.preventDefault();
            event.stopPropagation();
        },
        onHomeKey(event, frozenRow) {
            // const firstRow = this.findFirstSelectableRow();

            // firstRow && this.focusRowChange(row, firstRow);

            // if (event.ctrlKey && event.shiftKey && this.isMultipleSelectionMode() ) {
            //     ///const data = this.dataToRender(slotProps.rows);
            //    /// this.$emit('update:selection', data.slice(0, rowIndex + 1));
            // }

            // event.preventDefault();
            // event.stopPropagation();
        },
        onEndKey(event, frozenRow) {
            // const lastRow = this.findLastSelectableRow();

            // lastRow && this.focusRowChange(row, lastRow);

            // if (event.ctrlKey && event.shiftKey && this.isMultipleSelectionMode() ) {
            //     //const data = this.dataToRender(slotProps.rows);
            //    /// this.$emit('update:selection', data.slice(rowIndex, data.length));
            // }

            // event.preventDefault();
            // event.stopPropagation();
        },

        onEnterKey(event, frozenRow) {
            
        },
        
        onSpaceKey(event, row,  rowData, rowIndex, slotProps) {
             /*
            this.onEnterKey(event, row, rowData, rowIndex);

            if (event.shiftKey && this.selection !== null) {
                const data = this.dataToRender(slotProps.rows);
                let index;

                if (this.selection.length > 0) {
                    let firstSelectedRowIndex, lastSelectedRowIndex;

                    firstSelectedRowIndex = ObjectUtils.findIndexInList(this.selection[0], data);
                    lastSelectedRowIndex = ObjectUtils.findIndexInList(this.selection[this.selection.length - 1], data);

                    index = rowIndex <= firstSelectedRowIndex ? lastSelectedRowIndex : firstSelectedRowIndex;
                } else {
                    index = ObjectUtils.findIndexInList(this.selection, data);
                }

                const _selection = index !== rowIndex ? data.slice(Math.min(index, rowIndex), Math.max(index, rowIndex) + 1) : rowData;

                this.$emit('update:selection', _selection);
            }
          */
        },
        onTabKey(event, rowIndex) {
            // const body = this.$refs.bodyRef && this.$refs.bodyRef.$el;
            // const rows = DomHandler.find(body, 'tr.p-selectable-row');

            // if (event.code === 'Tab' && rows && rows.length > 0) {
            //     const firstSelectedRow = DomHandler.findSingle(body, 'tr.p-highlight');
            //     const focusedItem = DomHandler.findSingle(body, 'tr.p-selectable-row[tabindex="0"]');

            //     if (firstSelectedRow) {
            //         firstSelectedRow.tabIndex = '0';
            //         focusedItem && focusedItem !== firstSelectedRow && (focusedItem.tabIndex = '-1');
            //     } else {
            //         rows[0].tabIndex = '0';
            //         focusedItem !== rows[0] && (rows[rowIndex].tabIndex = '-1');
            //     }
            // }
        },
        findNextSelectableRow(row) {
            let nextRow = row.nextElementSibling;

            return nextRow;
        },
        findPrevSelectableRow(row) {
            let prevRow = row.previousElementSibling;

            return prevRow;
        },
        findFirstSelectableRow() {
            const body = this.$refs.bodyRef && this.$refs.bodyRef.$el;
            const firstRow = DomHandler.findSingle(body, 'tr');
            
            return firstRow;
        },
        findLastSelectableRow() {
            const body = this.$refs.bodyRef && this.$refs.bodyRef.$el;
            const rows = DomHandler.find(body, 'tr');

            return rows ? rows[rows.length - 1] : null;
        },

        focusRowChange(firstFocusableRow, currentFocusedRow) {
            firstFocusableRow.tabIndex = '-1';
            currentFocusedRow.tabIndex = '0';
            DomHandler.focus(currentFocusedRow);

            console.log("focusRowChange");
        },

        toggleRowWithRadio(event) {
            const rowData = event.data;

            if (rowData && !this.isSelected(rowData)) {
               this.doUnSelectAll();
               rowData['isSelected']=true;
               this.$emit('row-select-changed',rowData);
            }
        },

        toggleRowWithCheckbox(event) {
            const rowData = event.data;

            if(rowData){
               rowData['isSelected']=  !rowData['isSelected'];
               this.$emit('row-select-changed',rowData);
            }
        },

        toggleRowsWithCheckbox(event) {
            const { originalEvent, checked } = event;

            if(checked){
                this.doSelectAll();
                this.$emit('row-select-all');
            }
            else {
                this.doUnSelectAll();
                this.$emit('row-unselect-all');
            }
        },

        isSingleSelectionMode() {
            return this.selectionMode === 'single';
        },
        isMultipleSelectionMode() {
            return this.selectionMode === 'multiple';
        },

        isSelected(rowData) {
            return (rowData && rowData['isSelected']);
        },

        findIndex(rowData, collection) {
            let index = -1;

            if (collection && collection.length) {
                for (let i = 0; i < collection.length; i++) {
                    if (this.equals(rowData, collection[i])) {
                        index = i;
                        break;
                    }
                }
            }

            return index;
        },
        
        updateExpandedRowKeys(expandedRows) {
            if (expandedRows && expandedRows.length) {
                this.d_expandedRowKeys = {};

                for (let data of expandedRows) {
                    this.d_expandedRowKeys[String(ObjectUtils.resolveFieldData(data, this.dataKey))] = 1;
                }
            } else {
                this.d_expandedRowKeys = null;
            }
        },
       
        equals(data1, data2) {
            return this.compareSelectionBy === 'equals' ? data1 === data2 : ObjectUtils.equals(data1, data2, this.dataKey);
        },
        selectRange(event) {
            if(!this.d_primaryData || this.d_primaryData.length ===0 )
                return;

            let rangeStart, rangeEnd;

            if (this.rangeRowIndex > this.anchorRowIndex) {
                rangeStart = this.anchorRowIndex;
                rangeEnd = this.rangeRowIndex;
            } else if (this.rangeRowIndex < this.anchorRowIndex) {
                rangeStart = this.rangeRowIndex;
                rangeEnd = this.anchorRowIndex;
            } else {
                rangeStart = this.rangeRowIndex;
                rangeEnd = this.rangeRowIndex;
            }

            if (this.lazy && this.paginator) {
                rangeStart -= this.first;
                rangeEnd -= this.first;
            }
            
            for (let i =0; i< this.d_primaryData.length; i++) {
                this.d_primaryData[i]['isSelected']=(i>=rangeStart && i<=rangeEnd);
            }
       },

        exportCSV(options, data) {
            let csv = '\ufeff';

            if (!data) {
                data = this.d_primaryData;
            }

            if(!data || this.data.length ===0 )
                return;

            //headers
            let headerInitiated = false;

            for (let i = 0; i < this.columns.length; i++) {
                let column = this.columns[i];

                if (this.columnProp(column, 'exportable') !== false && this.columnProp(column, 'field')) {
                    if (headerInitiated) csv += this.csvSeparator;
                    else headerInitiated = true;

                    csv += '"' + (this.columnProp(column, 'exportHeader') || this.columnProp(column, 'header') || this.columnProp(column, 'field')) + '"';
                }
            }

            //body
            if (data) {
                data.forEach((record) => {
                    csv += '\n';
                    let rowInitiated = false;

                    for (let i = 0; i < this.columns.length; i++) {
                        let column = this.columns[i];

                        if (this.columnProp(column, 'exportable') !== false && this.columnProp(column, 'field')) {
                            if (rowInitiated) csv += this.csvSeparator;
                            else rowInitiated = true;

                            let cellData = ObjectUtils.resolveFieldData(record, this.columnProp(column, 'field'));

                            if (cellData != null) {
                                if (this.exportFunction) {
                                    cellData = this.exportFunction({
                                        data: cellData,
                                        field: this.columnProp(column, 'field')
                                    });
                                } else cellData = String(cellData).replace(/"/g, '""');
                            } else cellData = '';

                            csv += '"' + cellData + '"';
                        }
                    }
                });
            }

            //footers
            let footerInitiated = false;

            for (let i = 0; i < this.columns.length; i++) {
                let column = this.columns[i];

                if (i === 0) csv += '\n';

                if (this.columnProp(column, 'exportable') !== false && this.columnProp(column, 'exportFooter')) {
                    if (footerInitiated) csv += this.csvSeparator;
                    else footerInitiated = true;

                    csv += '"' + (this.columnProp(column, 'exportFooter') || this.columnProp(column, 'footer') || this.columnProp(column, 'field')) + '"';
                }
            }

            DomHandler.exportCSV(csv, this.exportFilename);
        },
        resetPage() {
            const { size } =this.d_pageParams;

            this.d_pageParams={
                size,
                current:1,
                total:0
            };
        },

        onColumnResizeStart(event) {
            let containerLeft = DomHandler.getOffset(this.$el).left;

            this.resizeColumnElement = event.target.parentElement;
            this.columnResizing = true;
            this.lastResizeHelperX = event.pageX - containerLeft + this.$el.scrollLeft;

            this.bindColumnResizeEvents();
        },
        onColumnResize(event) {
            let containerLeft = DomHandler.getOffset(this.$el).left;

            DomHandler.addClass(this.$el, 'p-unselectable-text');
            this.$refs.resizeHelper.style.height = this.$el.offsetHeight + 'px';
            this.$refs.resizeHelper.style.top = 0 + 'px';
            this.$refs.resizeHelper.style.left = event.pageX - containerLeft + this.$el.scrollLeft + 'px';

            this.$refs.resizeHelper.style.display = 'block';
        },
        onColumnResizeEnd() {
            let delta = this.$refs.resizeHelper.offsetLeft - this.lastResizeHelperX;
            let columnWidth = this.resizeColumnElement.offsetWidth;
            let newColumnWidth = columnWidth + delta;
            let minWidth = this.resizeColumnElement.style.minWidth || 15;

            if (columnWidth + delta > parseInt(minWidth, 10)) {
                if (this.columnResizeMode === 'fit') {
                    let nextColumn = this.resizeColumnElement.nextElementSibling;
                    let nextColumnWidth = nextColumn.offsetWidth - delta;

                    if (newColumnWidth > 15 && nextColumnWidth > 15) {
                        this.resizeTableCells(newColumnWidth, nextColumnWidth);
                    }
                } else if (this.columnResizeMode === 'expand') {
                    const tableWidth = this.$refs.table.offsetWidth + delta + 'px';

                    const updateTableWidth = (el) => {
                        !!el && (el.style.width = el.style.minWidth = tableWidth);
                    };

                    updateTableWidth(this.$refs.table);

                    // if (!this.virtualScrollerDisabled) {
                    //     const body = this.$refs.bodyRef && this.$refs.bodyRef.$el;
                    //     const frozenBody = this.$refs.frozenBodyRef && this.$refs.frozenBodyRef.$el;

                    //     updateTableWidth(body);
                    //     updateTableWidth(frozenBody);
                    // }

                    this.resizeTableCells(newColumnWidth);
                }

                this.$emit('column-resize-end', {
                    element: this.resizeColumnElement,
                    delta: delta
                });
            }

            this.$refs.resizeHelper.style.display = 'none';
            this.resizeColumn = null;
            DomHandler.removeClass(this.$el, 'p-unselectable-text');

            this.unbindColumnResizeEvents();

            if (this.isStateful()) {
                this.saveState();
            }
        },
        resizeTableCells(newColumnWidth, nextColumnWidth) {
            let colIndex = DomHandler.index(this.resizeColumnElement);
            let widths = [];
            let headers = DomHandler.find(this.$refs.table, '.p-datatable-thead > tr > th');

            headers.forEach((header) => widths.push(DomHandler.getOuterWidth(header)));

            this.destroyStyleElement();
            this.createStyleElement();

            let innerHTML = '';
            let selector = `.p-datatable[${this.attributeSelector}] > .p-datatable-wrapper ${this.virtualScrollerDisabled ? '' : '> .p-virtualscroller'} > .p-datatable-table`;

            widths.forEach((width, index) => {
                let colWidth = index === colIndex ? newColumnWidth : nextColumnWidth && index === colIndex + 1 ? nextColumnWidth : width;
                let style = `width: ${colWidth}px !important; max-width: ${colWidth}px !important`;

                innerHTML += `
                    ${selector} > .p-datatable-thead > tr > th:nth-child(${index + 1}),
                    ${selector} > .p-datatable-tbody > tr > td:nth-child(${index + 1}),
                    ${selector} > .p-datatable-tfoot > tr > td:nth-child(${index + 1}) {
                        ${style}
                    }
                `;
            });

            this.styleElement.innerHTML = innerHTML;
        },
        bindColumnResizeEvents() {
            if (!this.documentColumnResizeListener) {
                this.documentColumnResizeListener = document.addEventListener('mousemove', () => {
                    if (this.columnResizing) {
                        this.onColumnResize(event);
                    }
                });
            }

            if (!this.documentColumnResizeEndListener) {
                this.documentColumnResizeEndListener = document.addEventListener('mouseup', () => {
                    if (this.columnResizing) {
                        this.columnResizing = false;
                        this.onColumnResizeEnd();
                    }
                });
            }
        },
        unbindColumnResizeEvents() {
            if (this.documentColumnResizeListener) {
                document.removeEventListener('document', this.documentColumnResizeListener);
                this.documentColumnResizeListener = null;
            }

            if (this.documentColumnResizeEndListener) {
                document.removeEventListener('document', this.documentColumnResizeEndListener);
                this.documentColumnResizeEndListener = null;
            }
        },
        onColumnHeaderMouseDown(e) {
            const event = e.originalEvent;
            const column = e.column;

            if (this.reorderableColumns && this.columnProp(column, 'reorderableColumn') !== false) {
                if (event.target.nodeName === 'INPUT' || event.target.nodeName === 'TEXTAREA' || DomHandler.hasClass(event.target, 'p-column-resizer')) event.currentTarget.draggable = false;
                else event.currentTarget.draggable = true;
            }
        },
        onColumnHeaderDragStart(event) {
            if (this.columnResizing) {
                event.preventDefault();

                return;
            }

            this.colReorderIconWidth = DomHandler.getHiddenElementOuterWidth(this.$refs.reorderIndicatorUp);
            this.colReorderIconHeight = DomHandler.getHiddenElementOuterHeight(this.$refs.reorderIndicatorUp);

            this.draggedColumn = this.findParentHeader(event.target);
            event.dataTransfer.setData('text', 'b'); // Firefox requires this to make dragging possible
        },
        onColumnHeaderDragOver(event) {
            let dropHeader = this.findParentHeader(event.target);

            if (this.reorderableColumns && this.draggedColumn && dropHeader) {
                event.preventDefault();
                let containerOffset = DomHandler.getOffset(this.$el);
                let dropHeaderOffset = DomHandler.getOffset(dropHeader);

                if (this.draggedColumn !== dropHeader) {
                    let targetLeft = dropHeaderOffset.left - containerOffset.left;
                    let columnCenter = dropHeaderOffset.left + dropHeader.offsetWidth / 2;

                    this.$refs.reorderIndicatorUp.style.top = dropHeaderOffset.top - containerOffset.top - (this.colReorderIconHeight - 1) + 'px';
                    this.$refs.reorderIndicatorDown.style.top = dropHeaderOffset.top - containerOffset.top + dropHeader.offsetHeight + 'px';

                    if (event.pageX > columnCenter) {
                        this.$refs.reorderIndicatorUp.style.left = targetLeft + dropHeader.offsetWidth - Math.ceil(this.colReorderIconWidth / 2) + 'px';
                        this.$refs.reorderIndicatorDown.style.left = targetLeft + dropHeader.offsetWidth - Math.ceil(this.colReorderIconWidth / 2) + 'px';
                        this.dropPosition = 1;
                    } else {
                        this.$refs.reorderIndicatorUp.style.left = targetLeft - Math.ceil(this.colReorderIconWidth / 2) + 'px';
                        this.$refs.reorderIndicatorDown.style.left = targetLeft - Math.ceil(this.colReorderIconWidth / 2) + 'px';
                        this.dropPosition = -1;
                    }

                    this.$refs.reorderIndicatorUp.style.display = 'block';
                    this.$refs.reorderIndicatorDown.style.display = 'block';
                }
            }
        },
        onColumnHeaderDragLeave(event) {
            if (this.reorderableColumns && this.draggedColumn) {
                event.preventDefault();
                this.$refs.reorderIndicatorUp.style.display = 'none';
                this.$refs.reorderIndicatorDown.style.display = 'none';
            }
        },
        onColumnHeaderDrop(event) {
            event.preventDefault();

            if (this.draggedColumn) {
                let dragIndex = DomHandler.index(this.draggedColumn);
                let dropIndex = DomHandler.index(this.findParentHeader(event.target));
                let allowDrop = dragIndex !== dropIndex;

                if (allowDrop && ((dropIndex - dragIndex === 1 && this.dropPosition === -1) || (dropIndex - dragIndex === -1 && this.dropPosition === 1))) {
                    allowDrop = false;
                }

                if (allowDrop) {
                    ObjectUtils.reorderArray(this.columns, dragIndex, dropIndex);
                    this.updateReorderableColumns();

                    this.$emit('column-reorder', {
                        originalEvent: event,
                        dragIndex: dragIndex,
                        dropIndex: dropIndex
                    });
                }

                this.$refs.reorderIndicatorUp.style.display = 'none';
                this.$refs.reorderIndicatorDown.style.display = 'none';
                this.draggedColumn.draggable = false;
                this.draggedColumn = null;
                this.dropPosition = null;
            }
        },
        findParentHeader(element) {
            if (element.nodeName === 'TH') {
                return element;
            } else {
                let parent = element.parentElement;

                while (parent.nodeName !== 'TH') {
                    parent = parent.parentElement;
                    if (!parent) break;
                }

                return parent;
            }
        },
        findColumnByKey(columns, key) {
            if (columns && columns.length) {
                for (let i = 0; i < columns.length; i++) {
                    let column = columns[i];

                    if (this.columnProp(column, 'columnKey') === key || this.columnProp(column, 'field') === key) {
                        return column;
                    }
                }
            }

            return null;
        },
        onRowMouseDown(event) {
            if (DomHandler.hasClass(event.target, 'p-datatable-reorderablerow-handle')) event.currentTarget.draggable = true;
            else event.currentTarget.draggable = false;
        },
        onRowDragStart(e) {
            const event = e.originalEvent;
            const index = e.index;

            this.rowDragging = true;
            this.draggedRowIndex = index;
            event.dataTransfer.setData('text', 'b'); // For firefox
        },
        onRowDragOver(e) {
            const event = e.originalEvent;
            const index = e.index;

            if (this.rowDragging && this.draggedRowIndex !== index) {
                let rowElement = event.currentTarget;
                let rowY = DomHandler.getOffset(rowElement).top + DomHandler.getWindowScrollTop();
                let pageY = event.pageY;
                let rowMidY = rowY + DomHandler.getOuterHeight(rowElement) / 2;
                let prevRowElement = rowElement.previousElementSibling;

                if (pageY < rowMidY) {
                    DomHandler.removeClass(rowElement, 'p-datatable-dragpoint-bottom');

                    this.droppedRowIndex = index;
                    if (prevRowElement) DomHandler.addClass(prevRowElement, 'p-datatable-dragpoint-bottom');
                    else DomHandler.addClass(rowElement, 'p-datatable-dragpoint-top');
                } else {
                    if (prevRowElement) DomHandler.removeClass(prevRowElement, 'p-datatable-dragpoint-bottom');
                    else DomHandler.addClass(rowElement, 'p-datatable-dragpoint-top');

                    this.droppedRowIndex = index + 1;
                    DomHandler.addClass(rowElement, 'p-datatable-dragpoint-bottom');
                }

                event.preventDefault();
            }
        },
        onRowDragLeave(event) {
            let rowElement = event.currentTarget;
            let prevRowElement = rowElement.previousElementSibling;

            if (prevRowElement) {
                DomHandler.removeClass(prevRowElement, 'p-datatable-dragpoint-bottom');
            }

            DomHandler.removeClass(rowElement, 'p-datatable-dragpoint-bottom');
            DomHandler.removeClass(rowElement, 'p-datatable-dragpoint-top');
        },
        onRowDragEnd(event) {
            this.rowDragging = false;
            this.draggedRowIndex = null;
            this.droppedRowIndex = null;
            event.currentTarget.draggable = false;
        },
        onRowDrop(event) {
            if(!this.d_primaryData ||this.d_primaryData.length ===0  )
                return;

            if (this.droppedRowIndex != null) {
                let dropIndex = this.draggedRowIndex > this.droppedRowIndex ? this.droppedRowIndex : this.droppedRowIndex === 0 ? 0 : this.droppedRowIndex - 1;

                ObjectUtils.reorderArray(this.d_primaryData, this.draggedRowIndex + this.d_first, dropIndex + this.d_first);
            }

            //cleanup
            this.onRowDragLeave(event);
            this.onRowDragEnd(event);
            event.preventDefault();
        },
        toggleRow(event) {
            let rowData = event.data;
            let expanded;
            let expandedRowIndex;
            let _expandedRows = this.expandedRows ? [...this.expandedRows] : [];

            if (this.dataKey) {
                expanded = this.d_expandedRowKeys ? this.d_expandedRowKeys[ObjectUtils.resolveFieldData(rowData, this.dataKey)] !== undefined : false;
            } else {
                expandedRowIndex = this.findIndex(rowData, this.expandedRows);
                expanded = expandedRowIndex > -1;
            }

            if (expanded) {
                if (expandedRowIndex == null) {
                    expandedRowIndex = this.findIndex(rowData, this.expandedRows);
                }

                _expandedRows.splice(expandedRowIndex, 1);
                this.$emit('update:expandedRows', _expandedRows);
                this.$emit('row-collapse', event);
            } else {
                _expandedRows.push(rowData);
                this.$emit('update:expandedRows', _expandedRows);
                this.$emit('row-expand', event);
            }
        },
        toggleRowGroup(e) {
            const event = e.originalEvent;
            const data = e.data;
            const groupFieldValue = ObjectUtils.resolveFieldData(data, this.groupRowsBy);
            let _expandedRowGroups = this.expandedRowGroups ? [...this.expandedRowGroups] : [];

            if (this.isRowGroupExpanded(data)) {
                _expandedRowGroups = _expandedRowGroups.filter((group) => group !== groupFieldValue);
                this.$emit('update:expandedRowGroups', _expandedRowGroups);
                this.$emit('rowgroup-collapse', { originalEvent: event, data: groupFieldValue });
            } else {
                _expandedRowGroups.push(groupFieldValue);
                this.$emit('update:expandedRowGroups', _expandedRowGroups);
                this.$emit('rowgroup-expand', { originalEvent: event, data: groupFieldValue });
            }
        },
        isRowGroupExpanded(rowData) {
            if (this.expandableRowGroups && this.expandedRowGroups) {
                let groupFieldValue = ObjectUtils.resolveFieldData(rowData, this.groupRowsBy);

                return this.expandedRowGroups.indexOf(groupFieldValue) > -1;
            }

            return false;
        },
        isStateful() {
            return this.stateKey != null;
        },
        getStorage() {
            switch (this.stateStorage) {
                case 'local':
                    return window.localStorage;

                case 'session':
                    return window.sessionStorage;

                default:
                    throw new Error(this.stateStorage + ' is not a valid value for the state storage, supported values are "local" and "session".');
            }
        },
        saveState() {
            const storage = this.getStorage();
            let state = {};

            if (this.paginator) {
                state.first = this.d_first;
                state.rows = this.d_rows;
            }

            if (this.d_sortField) {
                state.sortField = this.d_sortField;
                state.sortOrder = this.d_sortOrder;
            }

            if (this.d_multiSortMeta) {
                state.multiSortMeta = this.d_multiSortMeta;
            }

            if (this.hasFilters) {
                state.filters = this.filters;
            }

            if (this.resizableColumns) {
                this.saveColumnWidths(state);
            }

            if (this.reorderableColumns) {
                state.columnOrder = this.d_columnOrder;
            }

            if (this.expandedRows) {
                state.expandedRows = this.expandedRows;
                state.expandedRowKeys = this.d_expandedRowKeys;
            }

            if (this.expandedRowGroups) {
                state.expandedRowGroups = this.expandedRowGroups;
            }


            if (Object.keys(state).length) {
                storage.setItem(this.stateKey, JSON.stringify(state));
            }

            this.$emit('state-save', state);
        },
        restoreState() {
            const storage = this.getStorage();
            const stateString = storage.getItem(this.stateKey);
            const dateFormat = /\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}.\d{3}Z/;

            const reviver = function (key, value) {
                if (typeof value === 'string' && dateFormat.test(value)) {
                    return new Date(value);
                }

                return value;
            };

            if (stateString) {
                let restoredState = JSON.parse(stateString, reviver);

                if (this.paginator) {
                    this.d_first = restoredState.first;
                    this.d_rows = restoredState.rows;
                }

                if (restoredState.sortField) {
                    this.d_sortField = restoredState.sortField;
                    this.d_sortOrder = restoredState.sortOrder;
                }

                if (restoredState.multiSortMeta) {
                    this.d_multiSortMeta = restoredState.multiSortMeta;
                }

                if (restoredState.filters) {
                    this.$emit('update:filters', restoredState.filters);
                }

                if (this.resizableColumns) {
                    this.columnWidthsState = restoredState.columnWidths;
                    this.tableWidthState = restoredState.tableWidth;
                }

                if (this.reorderableColumns) {
                    this.d_columnOrder = restoredState.columnOrder;
                }

                if (restoredState.expandedRows) {
                    this.d_expandedRowKeys = restoredState.expandedRowKeys;
                    this.$emit('update:expandedRows', restoredState.expandedRows);
                }

                if (restoredState.expandedRowGroups) {
                    this.$emit('update:expandedRowGroups', restoredState.expandedRowGroups);
                }

                this.$emit('state-restore', restoredState);
            }
        },
        saveColumnWidths(state) {
            let widths = [];
            let headers = DomHandler.find(this.$el, '.p-datatable-thead > tr > th');

            headers.forEach((header) => widths.push(DomHandler.getOuterWidth(header)));
            state.columnWidths = widths.join(',');

            if (this.columnResizeMode === 'expand') {
                state.tableWidth = DomHandler.getOuterWidth(this.$refs.table) + 'px';
            }
        },
        restoreColumnWidths() {
            if (this.columnWidthsState) {
                let widths = this.columnWidthsState.split(',');

                if (this.columnResizeMode === 'expand' && this.tableWidthState) {
                    this.$refs.table.style.width = this.tableWidthState;
                    this.$refs.table.style.minWidth = this.tableWidthState;
                    this.$el.style.width = this.tableWidthState;
                }

                if (ObjectUtils.isNotEmpty(widths)) {
                    this.createStyleElement();

                    let innerHTML = '';
                    let selector = `.p-datatable[${this.attributeSelector}] > .p-datatable-wrapper ${this.virtualScrollerDisabled ? '' : '> .p-virtualscroller'} > .p-datatable-table`;

                    widths.forEach((width, index) => {
                        let style = `width: ${width}px !important; max-width: ${width}px !important`;

                        innerHTML += `
                            ${selector} > .p-datatable-thead > tr > th:nth-child(${index + 1}),
                            ${selector} > .p-datatable-tbody > tr > td:nth-child(${index + 1}),
                            ${selector} > .p-datatable-tfoot > tr > td:nth-child(${index + 1}) {
                                ${style}
                            }
                        `;
                    });

                    this.styleElement.innerHTML = innerHTML;
                }
            }
        },
        onCellEditInit(event) {
            this.$emit('cell-edit-init', event);
        },
        onCellEditComplete(event) {
            this.$emit('cell-edit-complete', event);
        },
        onCellEditCancel(event) {
            this.$emit('cell-edit-cancel', event);
        },

        onItemChange(event){
            this.$emit('item-change', event);
        },
        hasGlobalFilter() {
            return this.filters && Object.prototype.hasOwnProperty.call(this.filters, 'global');
        },
        getChildren() {
            return this.$slots.default ? this.$slots.default() : null;
        },
        onFilterChange(filters) {
            this.d_filters = filters;
            this.filter();
        },
        
        onFilterApply() {
            this.$emit('update:filters', this.d_filters);
            this.filter();
        },

        cloneFilters() {
            let cloned = {};

            if (this.filters) {
                Object.entries(this.filters).forEach(([prop, value]) => {
                    cloned[prop] = value.operator
                        ? {
                              operator: value.operator,
                              constraints: value.constraints.map((constraint) => {
                                  return { ...constraint };
                              })
                          }
                        : { ...value };
                });
            }

            return cloned;
        },
        updateReorderableColumns() {
            let columnOrder = [];

            this.columns.forEach((col) => columnOrder.push(this.columnProp(col, 'columnKey') || this.columnProp(col, 'field')));
            this.d_columnOrder = columnOrder;
        },
        createStyleElement() {
            this.styleElement = document.createElement('style');
            this.styleElement.type = 'text/css';
            document.head.appendChild(this.styleElement);
        },
       
        destroyStyleElement() {
            if (this.styleElement) {
                document.head.removeChild(this.styleElement);
                this.styleElement = null;
            }
        },
        recursiveGetChildren(children, results) {
            if (!results) {
                results = [];
            }

            if (children && children.length) {
                children.forEach((child) => {
                    if (child.children instanceof Array) {
                        results.concat(this.recursiveGetChildren(child.children, results));
                    } else if (child.type.name == 'Column') {
                        results.push(child);
                    }
                });
            }

            return results;
        },
       
        hasSpacerStyle(style) {
            return ObjectUtils.isNotEmpty(style);
        },

        onBodyKeyDown(e){
            const event = e.originalEvent;
            const frozenRow =e.frozenRow; 

            switch (event.code) {
                case 'ArrowDown':
                    this.onArrowDownKey(event, frozenRow);
                   break;

                case 'ArrowUp':
                    this.onArrowUpKey(event, frozenRow );
                    break;

                /*
                case 'Home':
                    this.onHomeKey(event, frozenRow);
                    break;

                case 'End':
                    this.onEndKey(event, frozenRow);
                    break;

                case 'Enter':
                case 'NumpadEnter':
                    this.onEnterKey(event, frozenRow );
                    break;

              
                case 'Space':
                    this.onSpaceKey(event, frozenRow );
                    break;

                case 'Tab':
                    this.onTabKeyevent, frozenRow);
                    break;
                */
                default:
                    if (event.code === 'KeyA' && metaKey &&  this.isMultipleSelectionMode()) {
                       this.doSelectAll();
                       event.preventDefault();
                       event.stopPropagation();
                    }

                    break;
            }
        },

        doSelectAll(){
            if(!this.d_primaryData || this.d_primaryData.length===0 )
                return;

            for(let i=0; i< this.d_primaryData.length; i++){

                if( !this.d_primaryData[i]['isSelected']){
                   this.d_primaryData[i]['isSelected']=true;

                   this.$emit('row-select-changed',this.d_primaryData[i]);
                }
            }
        },

        doUnSelectAll(){
            if(!this.d_primaryData || this.d_primaryData.length ===0 )
                return;

            for(let i=0; i< this.d_primaryData.length; i++){
                if(this.d_primaryData[i]['isSelected']){
                     this.d_primaryData[i]['isSelected']=false;

                     this.$emit('row-select-changed',this.d_primaryData[i]);
                }
            }
        },

        setData(newData){
            if(Array.isArray(newData)){
                this._interalSetData(newData);
            }
            else {
                if(newData.hasOwnProperty("records") && newData.hasOwnProperty("current") && newData.hasOwnProperty("total")){
                    let { size, current, total }= newData;

                    if(!size){
                        size= this.d_pageParams.size; 
                    }

                    this._interalSetData(newData["records"]);
                    this.d_pageParams = { size, current, total};
                }
                else {
                    this._interalSetData( [ newData] ); 
                }
            }
        },

        _interalSetData(newData){
            this.reset();
            
            if(!newData || !newData.length )
                return;
           
             if (this.sorted) {
                if (this.sortMode === 'single')
                    newData = this.sortSingle(newData);
                else if (this.sortMode === 'multiple') 
                   newData = this.sortMultiple(newData);
             }

           
            this.d_primaryData = newData.map((item)=> DataHandler.Proxy(item, ++this.uniqueRowId, this._interalSetModified ) );
            this.d_currentRow =0;
            this.rowFocusChanged(0);
        },

        _interalSetModified(newData){
           this.d_modified=newData;
        },

        setRow(rowIndex){
            if(!this.d_primaryData || this.d_primaryData.length===0 )
                return;

            if(rowIndex<0 || rowIndex>=( this.d_primaryData.length -1) )
                rowIndex = this.d_primaryData.length -1;

            this.d_currentRow= rowIndex;
        },

        getRow(){
            return this.d_currentRow;
        },

        getRowByRowId(rowId){
            if(!rowId || !this.d_primaryData || this.d_primaryData.length===0  )
                return -1; 
            
            for(let i=0;i<this.d_primaryData.length;i++){
                if( this.d_primaryData[i]['rowId'] === rowId){
                    return i;
                }
            }
            
            return -1;
        },

        getRowData( rowIndex ){
           if(rowIndex===undefined || rowIndex===null || rowIndex===-1  )
              rowIndex = this.d_currentRow;

           if(!this.d_primaryData || this.d_primaryData.length===0 )
                return null;

           if(!this.d_primaryData ||  rowIndex< 0 || rowIndex>=this.d_primaryData.length ){
                throw new RangeError(`RowIndex:${rowIndex}无效`);
            }

            return this.d_primaryData[rowIndex];
        },

        getFirstRowData(){
           if(!this.d_primaryData || this.d_primaryData.length===0 )
               return null;

            return this.getRowData(0);
        },
        getLastRowData(){
            if(!this.d_primaryData || this.d_primaryData.length===0 )
               return null;

            return this.getRowData(this.d_primaryData.length -1 );
        },

        getRowId(){
            const rowData= this.getRowData( -1 );

            if( rowData && ('rowId' in rowData)){
                return rowData['rowId'];
            }
            
            return null;
        },
   
        setRowByRowId(rowId){
            if(!rowId || !this.d_primaryData || this.d_primaryData.length===0  )
                return ; 
            
            for(let i=0;i<this.d_primaryData.length;i++){
                if( this.d_primaryData[i]['rowId'] === rowId){
                    this.d_currentRow = i;
                    break;
                }
            }
        },

        //TODO 未考虑数据过滤和排序的情况，如果设置了过滤和排序，插入后的数据会过滤掉或显示的位置不对
        //所以应该考虑优化 processData,避免每次显示都重新过滤和排序数据
        insertRow(rowIndex){
             
            let rowData=null;
            let _columns= this.columns || [] ;

            if(this.uniqueRowId===null || this.uniqueRowId===undefined){
                this.uniqueRowId=0;
            }

            if(this.d_primaryData && this.d_primaryData.length){
                rowData= DataHandler.Initialize(this.d_primaryData[0], ++this.uniqueRowId , this._interalSetModified);
            }
            else {
                rowData={};  //先不进行Proxy处理，否则后面赋初始值会提示找不到属性
            }
         
            for (let i = 0; i < _columns.length; i++) {
                const column = _columns[i];
                const field= this.columnProp(column, 'field') || '';
                const initial = this.columnProp(column, 'initial')  ;
                const colType =  this.columnProp(column, 'colType') || '';

                if (field) {
                    rowData[field] =(initial!==null && initial!==undefined && initial!=='' )? ObjectUtils.initailValue(initial, colType): null ;
                }
            }

            if(!('__dataProxy' in rowData)){
                rowData=DataHandler.Proxy(rowData, ++this.uniqueRowId ,this._interalSetModified );
            }

            DomHandler.clearSelection();
            this.$emit('row-new-init', rowData);
            rowData['rowStatus'] = RowStatus.New;

            if(!this.d_primaryData){
                this.d_primaryData =[];
            }

            if(rowIndex < 0 || rowIndex>= this.d_primaryData.length ){
                this.d_primaryData.push(rowData);
            }
            else {
                this.d_primaryData.splice(rowIndex  ,0 , rowData);
            }

            if(this.d_currentRow< 0 ){
                this.d_currentRow =0;
            }
            else if(this.d_currentRow>=rowIndex){
               this.d_currentRow++;
            }
            
            return rowData; // 返回新增行的数据，以便对新增行的数据进行赋值 
        },

        deleteRow(rowIndex){
            if( !this.d_primaryData || this.d_primaryData.length===0)
                return;

            //如果传入 -1 ，表示当前行
            if(rowIndex=== -1 ){
                rowIndex = this.d_currentRow;
            }

            if(rowIndex<0 || rowIndex>= this.d_primaryData.length ){
                return ;
            }

            DomHandler.clearSelection();
            let rowData = this.d_primaryData.splice(rowIndex,1)[0] ;
            let { rowStatus=RowStatus.NotModified } = rowData;
            
            //如果删除行之后当前行号大于记录数
            if(this.d_primaryData.length===0){
                this.d_currentRow=-1;
            }
            else if( this.d_currentRow>=rowIndex &&  this.d_currentRow>0 ) {
                this.d_currentRow--;
            }
            else if( this.d_currentRow>=0 && this.d_currentRow< this.d_primaryData.length){ //删除记录后行号没有改变，也解发发row-focus-changed事件
                this.$emit('row-focus-changed',{ rowIndex: this.d_currentRow, rowData: this.d_primaryData[this.d_currentRow ] });
            }

            if(rowStatus===RowStatus.NotModified || rowStatus=== RowStatus.DataModified){
                if(!this.deletedData){
                    this.deletedData=[];
                }

                this.deletedData.push( DataHandler.Restore(rowData));
                this.d_modified=true;
            }
        },

        setItem(rowIndex, fieldName,value){
            if(rowIndex===-1)
                rowIndex = this.d_currentRow;
           
           if(!this.d_primaryData ||  rowIndex< 0 || rowIndex>=this.d_primaryData.length ){
                throw new RangeError(`RowIndex:${rowIndex}无效`);
            }

            this.d_primaryData[rowIndex][fieldName] = value;
        },
        getItem(rowIndex, fieldName){
            if(rowIndex===-1)
                rowIndex = this.d_currentRow;
           
            if(!this.d_primaryData ||  rowIndex< 0 || rowIndex>=this.d_primaryData.length ){
                throw new RangeError(`RowIndex:${rowIndex}无效`);
            }
            
            return this.d_primaryData[rowIndex][fieldName] ;
        },

        reset(){
            this.d_primaryData = null;
            this.filteredData = null;
            this.deletedData = null;
            this.d_currentRow = -1;
            this.d_modified=false;
            this.resetPage();
            this.$emit('update:expandedRows',null);
        },

        resetUpdate(){
            const _resetUpdate=(values)=>{
                if(!values)
                    return ;
                
                for(let i=0;i<values.length;i++){
                    values[i].resetUpdate();
                }
            };

            _resetUpdate(this.d_primaryData);
            _resetUpdate(this.filteredData);
            this.deletedData = null;
            this.d_modified=false;
        },

        setFocus(rowIndex,field){
            if(!field){
                this.setRow(rowIndex);

                return;
            }

            const trNodeList= this.$refs.bodyRef.$el.querySelectorAll("tr");

            if(rowIndex<0 || rowIndex> (trNodeList.length -1) )
                rowIndex =trNodeList.length -1;

            const cellEl = trNodeList[rowIndex].querySelector(`[data-field=${field}`);

            if(cellEl){
                this.d_currentRow = rowIndex;

                //下面这行代码会导致单元在表格中显示的位置发生改变
                //cellEl.scrollIntoView && cellEl.scrollIntoView({ block: 'nearest', inline: 'start' });
                setTimeout(()=>{ DomHandler.invokeElementMethod(cellEl, 'click'); },50);
             }
        },

        rowFocusChanged(rowIndex){
            if(rowIndex>=0 && rowIndex< this.d_primaryData.length){
                const rowData = this.d_primaryData[rowIndex];
                const rowId = ('rowId' in rowData) ? rowData['rowId'] :rowIndex;

                if(rowId!==this.lastFocusRowId){
                    this.lastFocusRowId=rowId;
                    this.$emit('row-focus-changed',{ rowIndex, rowData  } );
                }
            }
        },

        evaluate(funName, field){

            switch(funName.toLowerCase()){
                case 'sum':
                    return this.evaluateSum(field);

                case 'count':
                    return this.evaluateCount();

                case 'average':
                case 'avg':
                   return this.evaluateAvg(field);

                 default:
                    throw(new Error("不支持的计算公式:"+funName));
            }

            throw(new Error("不支持的计算公式:"+funName));
        },
        
        evaluateSum(field){

            let data = this.d_primaryData || [];

            return data.reduce((preVal, curVal)=>{
                let  val= ObjectUtils.resolveFieldData(curVal, field) || 0 ;

                if( ( typeof val )==='string'){
                    val= Number.parseFloat(val);

                    if(Number.isNaN(val)){
                        val= 0 ;
                    }
                }

                return preVal+ val ; 
            },0);
        },

        evaluateCount(){
            return (this.d_primaryData ) ? this.d_primaryData.length :0  ;
        } ,

        evaluateAvg(field){
           
             let sum = this.evaluateSum(field);
             let count = this.evaluateCount();
             
            return (count>0) ? (sum / count ) : 0;
        },

        getData(){
            return this.d_primaryData  ;
        },

        primaryData(){
            return this.d_primaryData  ;
        },

        changedData(){
            let data=[];

            if(this.d_primaryData){
               data= this.d_primaryData.filter((item)=>{ 
                    const status =item['rowStatus'] || RowStatus.NotModified;

                    return ( status=== RowStatus.NewModified || status === RowStatus.DataModified ) ;
                } );
            }

            if(this.deletedData && this.deletedData.length){
                data =[...data, ...this.deletedData];
            }

            return data;
        },
        
        rowCount(){
            return (this.d_primaryData ) ? this.d_primaryData.length: 0  ;
        },

        deletedData(){
            return this.deletedData  ;
        },

        deletedCount(){
             return (this.deletedData ) ? this.deletedData.length :0  ;
        },

        filteredData(){
            return this.filteredData  ;
        },
        filteredCount(){
             return (this.filteredData ) ? this.filteredData.length :0   ;
        },

        isModified(){
          /*
          if(this.deletedData && this.deletedData.length)
              return true;

          if( this.d_primaryData &&
               this.d_primaryData.find( (item)=> item.rowStatus=== RowStatus.DataModified || item.rowStatus=== RowStatus.NewModified )){
            return true;
          }
           return false;
          */
           return this.d_modified;
        },
        
        pageParms(){
            return this.d_pageParams
        }
    },

    computed: {
        test22(){
          return this.d_modified;
        },
        containerClass() {
            return [
                'p-datatable p-component',
                'p-datatable-responsive-scroll',
                {
                    'p-datatable-hoverable-rows': this.rowHover || this.selectionMode,
                    'p-datatable-resizable': this.resizableColumns,
                    'p-datatable-resizable-fit': this.resizableColumns && this.columnResizeMode === 'fit',
                    'p-datatable-scrollable': this.scrollable,
                    'p-datatable-flex-scrollable': this.scrollable && this.scrollHeight === 'flex',
                    'p-datatable-striped': this.stripedRows,
                    'p-datatable-gridlines': this.showGridlines,
                    'p-datatable-grouped-header': this.headerColumnGroup != null,
                    'p-datatable-grouped-footer': this.footerColumnGroup != null
                }
            ];
        },
        tableStyleClass() {
            return [
                'p-datatable-table',
                {
                    'p-datatable-scrollable-table': this.scrollable,
                    'p-datatable-resizable-table': this.resizableColumns,
                    'p-datatable-resizable-table-fit': this.resizableColumns && this.columnResizeMode === 'fit'
                },
                this.tableClass
            ];
        },
        columns() {
            let children = this.getChildren();

            if (!children) {
                return;
            }

            const cols = this.recursiveGetChildren(children, []);

            if (this.reorderableColumns && this.d_columnOrder) {
                let orderedColumns = [];

                for (let columnKey of this.d_columnOrder) {
                    let column = this.findColumnByKey(cols, columnKey);

                    if (column && !this.columnProp(column, 'hidden')) {
                        orderedColumns.push(column);
                    }
                }

                return [...orderedColumns, ...cols.filter((item) => orderedColumns.indexOf(item) < 0)];
            }

            return cols;
        },
        headerColumnGroup() {
            const children = this.getChildren();

            if (children) {
                for (let child of children) {
                    if (child.type.name === 'ColumnGroup' && this.columnProp(child, 'type') === 'header') {
                        return child;
                    }
                }
            }

            return null;
        },
        footerColumnGroup() {
            const children = this.getChildren();

            if (children) {
                for (let child of children) {
                    if (child.type.name === 'ColumnGroup' && this.columnProp(child, 'type') === 'footer') {
                        return child;
                    }
                }
            }

            return null;
        },
        hasFilters() {
            return this.filters && Object.keys(this.filters).length > 0 && this.filters.constructor === Object;
        },

        totalRecordsLength() {
            // if (this.lazy) {
            //     return this.totalRecords;
            // } else {
            //     const data = this.processedData;

            //     return data ? data.length : 0;
            // }
            return 10;
        },
        empty() {
            return (!this.d_primaryData || this.d_primaryData.length === 0) ;
        },

        paginatorTop() {
            return this.paginator && (this.paginatorPosition !== 'bottom' || this.paginatorPosition === 'both');
        },
        paginatorBottom() {
            return this.paginator && (this.paginatorPosition !== 'top' || this.paginatorPosition === 'both');
        },
        sorted() {
            return this.d_sortField || (this.d_multiSortMeta && this.d_multiSortMeta.length > 0);
        },
        allRowsSelected() {
           if(!this.d_primaryData || this.d_primaryData.length===0 )
              return false;

            return ( this.d_primaryData.find((rowData)=>!this.isSelected(rowData))) ? false : true;
        },

        attributeSelector() {
            return UniqueComponentId();
        },
        groupRowSortField() {
            return this.sortMode === 'single' ? this.sortField : this.d_groupRowsSortMeta ? this.d_groupRowsSortMeta.field : null;
        },

        // virtualScrollerDisabled() {
        //     return ObjectUtils.isEmpty(this.virtualScrollerOptions) || !this.scrollable;
        // },

       
    },
    
    components: {
        DTPaginator: Paginator,
        DTTableHeader: script$1,
        DTTableBody: script$7,
        DTTableFooter: script$5,
        DTVirtualScroller: VirtualScroller,
        ArrowDownIcon: ArrowDownIcon,
        ArrowUpIcon: ArrowUpIcon,
        SpinnerIcon: SpinnerIcon
    }
};

const _hoisted_1 = {
  key: 0,
  class: "p-datatable-loading-overlay p-component-overlay"
};
const _hoisted_2 = {
  key: 1,
  class: "p-datatable-header"
};
const _hoisted_3 = { class: "p-datatable-wrapper" };
const _hoisted_4 = {
  key: 2,
  class: "p-datatable-footer"
};
const _hoisted_5 = {
  ref: "resizeHelper",
  class: "p-column-resizer-helper",
  style: {"display":"none"}
};
const _hoisted_6 = {
  key: 4,
  ref: "reorderIndicatorUp",
  class: "p-datatable-reorder-indicator-up",
  style: {"position":"absolute","display":"none"}
};
const _hoisted_7 = {
  key: 5,
  ref: "reorderIndicatorDown",
  class: "p-datatable-reorder-indicator-down",
  style: {"position":"absolute","display":"none"}
};

function render(_ctx, _cache, $props, $setup, $data, $options) {
  const _component_SpinnerIcon = resolveComponent("SpinnerIcon");
  const _component_DTTableHeader = resolveComponent("DTTableHeader");
  const _component_DTTableBody = resolveComponent("DTTableBody");
  const _component_DTTableFooter = resolveComponent("DTTableFooter");
  const _component_DTPaginator = resolveComponent("DTPaginator");

  return (openBlock(), createElementBlock("div", {
    class: normalizeClass($options.containerClass),
    "data-scrollselectors": ".p-datatable-wrapper"
  }, [
    renderSlot(_ctx.$slots, "default"),
    ($props.loading)
      ? (openBlock(), createElementBlock("div", _hoisted_1, [
          (_ctx.$slots.loading)
            ? renderSlot(_ctx.$slots, "loading", { key: 0 })
            : (openBlock(), createElementBlock(Fragment, { key: 1 }, [
                (_ctx.$slots.loadingicon)
                  ? (openBlock(), createBlock(resolveDynamicComponent(_ctx.$slots.loadingicon), {
                      key: 0,
                      class: "p-datatable-loading-icon"
                    }))
                  : ($props.loadingIcon)
                    ? (openBlock(), createElementBlock("i", {
                        key: 1,
                        class: normalizeClass(['p-datatable-loading-icon pi-spin', $props.loadingIcon])
                      }, null, 2 /* CLASS */))
                    : (openBlock(), createBlock(_component_SpinnerIcon, {
                        key: 2,
                        spin: "",
                        class: "p-datatable-loading-icon"
                      }))
              ], 64 /* STABLE_FRAGMENT */))
        ]))
      : createCommentVNode("v-if", true),
    (_ctx.$slots.header)
      ? (openBlock(), createElementBlock("div", _hoisted_2, [
          renderSlot(_ctx.$slots, "header")
        ]))
      : createCommentVNode("v-if", true),
    createElementVNode("div", _hoisted_3, [
      createCommentVNode(":style=\"{ maxHeight: scrollHeight  }\""),
      createElementVNode("table", mergeProps({
        ref: "table",
        class: $options.tableStyleClass,
        style: [$props.tableStyle]
      }, $props.tableProps), [
        createCommentVNode(", slotProps.spacerStyle "),
        createVNode(_component_DTTableHeader, {
          columnGroup: $options.headerColumnGroup,
          columns: $options.columns,
          rowGroupMode: $props.rowGroupMode,
          groupRowsBy: $props.groupRowsBy,
          groupRowSortField: $options.groupRowSortField,
          reorderableColumns: $props.reorderableColumns,
          resizableColumns: $props.resizableColumns,
          allRowsSelected: $options.allRowsSelected,
          empty: $options.empty,
          sortMode: $props.sortMode,
          sortField: $data.d_sortField,
          sortOrder: $data.d_sortOrder,
          multiSortMeta: $data.d_multiSortMeta,
          filters: $data.d_filters,
          filtersStore: $props.filters,
          filterDisplay: $props.filterDisplay,
          filterInputProps: $props.filterInputProps,
          headerCheckboxIconTemplate: _ctx.$slots.headercheckboxicon,
          onColumnClick: _cache[0] || (_cache[0] = $event => ($options.onColumnHeaderClick($event))),
          onColumnMousedown: _cache[1] || (_cache[1] = $event => ($options.onColumnHeaderMouseDown($event))),
          onFilterChange: $options.onFilterChange,
          onFilterApply: $options.onFilterApply,
          onColumnDragstart: _cache[2] || (_cache[2] = $event => ($options.onColumnHeaderDragStart($event))),
          onColumnDragover: _cache[3] || (_cache[3] = $event => ($options.onColumnHeaderDragOver($event))),
          onColumnDragleave: _cache[4] || (_cache[4] = $event => ($options.onColumnHeaderDragLeave($event))),
          onColumnDrop: _cache[5] || (_cache[5] = $event => ($options.onColumnHeaderDrop($event))),
          onColumnResizestart: _cache[6] || (_cache[6] = $event => ($options.onColumnResizeStart($event))),
          onCheckboxChange: _cache[7] || (_cache[7] = $event => ($options.toggleRowsWithCheckbox($event)))
        }, null, 8 /* PROPS */, ["columnGroup", "columns", "rowGroupMode", "groupRowsBy", "groupRowSortField", "reorderableColumns", "resizableColumns", "allRowsSelected", "empty", "sortMode", "sortField", "sortOrder", "multiSortMeta", "filters", "filtersStore", "filterDisplay", "filterInputProps", "headerCheckboxIconTemplate", "onFilterChange", "onFilterApply"]),
        createCommentVNode(":class=\"slotProps.styleClass\" "),
        createVNode(_component_DTTableBody, {
          ref: "bodyRef",
          value: $data.d_primaryData,
          columns: $options.columns,
          empty: $options.empty,
          currentRow: $data.d_currentRow,
          selectionMode: $props.selectionMode,
          contextMenu: $props.contextMenu,
          contextMenuSelection: $props.contextMenuSelection,
          rowGroupMode: $props.rowGroupMode,
          groupRowsBy: $props.groupRowsBy,
          expandableRowGroups: $props.expandableRowGroups,
          rowClass: $props.rowClass,
          rowStyle: $props.rowStyle,
          compareSelectionBy: $props.compareSelectionBy,
          scrollable: $props.scrollable,
          expandedRowIcon: $props.expandedRowIcon,
          collapsedRowIcon: $props.collapsedRowIcon,
          expandedRows: $props.expandedRows,
          expandedRowKeys: $data.d_expandedRowKeys,
          expandedRowGroups: $props.expandedRowGroups,
          disabled: $props.disabled,
          readonly: $props.readonly,
          templates: _ctx.$slots,
          currentRowHighlight: $props.currentRowHighlight,
          onRowgroupToggle: $options.toggleRowGroup,
          onRowClick: _cache[8] || (_cache[8] = $event => ($options.onRowClick($event))),
          onRowDblclick: _cache[9] || (_cache[9] = $event => ($options.onRowDblClick($event))),
          onRowRightclick: _cache[10] || (_cache[10] = $event => ($options.onRowRightClick($event))),
          onRowTouchend: $options.onRowTouchEnd,
          onRowMousedown: $options.onRowMouseDown,
          onRowDragstart: _cache[11] || (_cache[11] = $event => ($options.onRowDragStart($event))),
          onRowDragover: _cache[12] || (_cache[12] = $event => ($options.onRowDragOver($event))),
          onRowDragleave: _cache[13] || (_cache[13] = $event => ($options.onRowDragLeave($event))),
          onRowDragend: _cache[14] || (_cache[14] = $event => ($options.onRowDragEnd($event))),
          onRowDrop: _cache[15] || (_cache[15] = $event => ($options.onRowDrop($event))),
          onRowToggle: _cache[16] || (_cache[16] = $event => ($options.toggleRow($event))),
          onRadioChange: _cache[17] || (_cache[17] = $event => ($options.toggleRowWithRadio($event))),
          onCheckboxChange: _cache[18] || (_cache[18] = $event => ($options.toggleRowWithCheckbox($event))),
          onCellEditInit: _cache[19] || (_cache[19] = $event => ($options.onCellEditInit($event))),
          onCellEditComplete: _cache[20] || (_cache[20] = $event => ($options.onCellEditComplete($event))),
          onCellEditCancel: _cache[21] || (_cache[21] = $event => ($options.onCellEditCancel($event))),
          onBodyKeydown: _cache[22] || (_cache[22] = $event => ($options.onBodyKeyDown($event))),
          onItemChange: $options.onItemChange
        }, null, 8 /* PROPS */, ["value", "columns", "empty", "currentRow", "selectionMode", "contextMenu", "contextMenuSelection", "rowGroupMode", "groupRowsBy", "expandableRowGroups", "rowClass", "rowStyle", "compareSelectionBy", "scrollable", "expandedRowIcon", "collapsedRowIcon", "expandedRows", "expandedRowKeys", "expandedRowGroups", "disabled", "readonly", "templates", "currentRowHighlight", "onRowgroupToggle", "onRowTouchend", "onRowMousedown", "onItemChange"]),
        createVNode(_component_DTTableFooter, {
          columnGroup: $options.footerColumnGroup,
          columns: $options.columns
        }, null, 8 /* PROPS */, ["columnGroup", "columns"])
      ], 16 /* FULL_PROPS */)
    ]),
    (_ctx.$slots.footer)
      ? (openBlock(), createElementBlock("div", _hoisted_4, [
          renderSlot(_ctx.$slots, "footer")
        ]))
      : createCommentVNode("v-if", true),
    ($props.paginator)
      ? (openBlock(), createBlock(_component_DTPaginator, {
          key: 3,
          total: $data.d_pageParams.total,
          size: $data.d_pageParams.size,
          current: $data.d_pageParams.current,
          pageLinkSize: $props.pageLinkSize,
          template: $props.paginatorTemplate,
          rowsPerPageOptions: $props.rowsPerPageOptions,
          currentPageReportTemplate: $props.currentPageReportTemplate,
          class: "p-paginator-bottom",
          onPageChange: _cache[23] || (_cache[23] = $event => ($options.onPageChange($event)))
        }, createSlots({ _: 2 /* DYNAMIC */ }, [
          (_ctx.$slots.paginatorstart)
            ? {
                name: "start",
                fn: withCtx(() => [
                  renderSlot(_ctx.$slots, "paginatorstart")
                ]),
                key: "0"
              }
            : undefined,
          (_ctx.$slots.paginatorend)
            ? {
                name: "end",
                fn: withCtx(() => [
                  renderSlot(_ctx.$slots, "paginatorend")
                ]),
                key: "1"
              }
            : undefined,
          (_ctx.$slots.paginatorfirstpagelinkicon)
            ? {
                name: "firstpagelinkicon",
                fn: withCtx(() => [
                  renderSlot(_ctx.$slots, "paginatorfirstpagelinkicon")
                ]),
                key: "2"
              }
            : undefined,
          (_ctx.$slots.paginatorprevpagelinkicon)
            ? {
                name: "prevpagelinkicon",
                fn: withCtx(() => [
                  renderSlot(_ctx.$slots, "paginatorprevpagelinkicon")
                ]),
                key: "3"
              }
            : undefined,
          (_ctx.$slots.paginatornextpagelinkicon)
            ? {
                name: "nextpagelinkicon",
                fn: withCtx(() => [
                  renderSlot(_ctx.$slots, "paginatornextpagelinkicon")
                ]),
                key: "4"
              }
            : undefined,
          (_ctx.$slots.paginatorlastpagelinkicon)
            ? {
                name: "lastpagelinkicon",
                fn: withCtx(() => [
                  renderSlot(_ctx.$slots, "paginatorlastpagelinkicon")
                ]),
                key: "5"
              }
            : undefined
        ]), 1032 /* PROPS, DYNAMIC_SLOTS */, ["total", "size", "current", "pageLinkSize", "template", "rowsPerPageOptions", "currentPageReportTemplate"]))
      : createCommentVNode("v-if", true),
    createElementVNode("div", _hoisted_5, null, 512 /* NEED_PATCH */),
    ($props.reorderableColumns)
      ? (openBlock(), createElementBlock("span", _hoisted_6, [
          (openBlock(), createBlock(resolveDynamicComponent(_ctx.$slots.reorderindicatorupicon || 'ArrowDownIcon')))
        ], 512 /* NEED_PATCH */))
      : createCommentVNode("v-if", true),
    ($props.reorderableColumns)
      ? (openBlock(), createElementBlock("span", _hoisted_7, [
          (openBlock(), createBlock(resolveDynamicComponent(_ctx.$slots.reorderindicatordownicon || 'ArrowUpIcon')))
        ], 512 /* NEED_PATCH */))
      : createCommentVNode("v-if", true)
  ], 2 /* CLASS */))
}

function styleInject(css, ref) {
  if ( ref === void 0 ) ref = {};
  var insertAt = ref.insertAt;

  if (!css || typeof document === 'undefined') { return; }

  var head = document.head || document.getElementsByTagName('head')[0];
  var style = document.createElement('style');
  style.type = 'text/css';

  if (insertAt === 'top') {
    if (head.firstChild) {
      head.insertBefore(style, head.firstChild);
    } else {
      head.appendChild(style);
    }
  } else {
    head.appendChild(style);
  }

  if (style.styleSheet) {
    style.styleSheet.cssText = css;
  } else {
    style.appendChild(document.createTextNode(css));
  }
}

var css_248z = "\n.p-datatable {\n    position: relative;\n    outline:none;\n    display: flex;\n    flex-direction: column;\n}\n.p-datatable > .p-datatable-wrapper {\n    background-color: #FFFFFF;\n    flex: 1;\n    overflow: auto;\n}\n.p-datatable-table {\n    border-spacing: 0px;\n    width: 100%;\n    table-layout: fixed;\n    white-space: pre-wrap;  /*让空格保留正常的输出*/\n    outline:none;\n}\n.p-datatable-tbody{\n    outline: none;   /*body获得焦点时不显示边框*/\n}\n.p-datatable .p-sortable-column {\n    cursor: pointer;\n    user-select: none;\n}\n.p-datatable .p-sortable-column .p-column-title,\n.p-datatable .p-sortable-column .p-sortable-column-icon,\n.p-datatable .p-sortable-column .p-sortable-column-badge {\n    vertical-align: middle;\n}\n.p-datatable .p-sortable-column .p-sortable-column-badge {\n    display: inline-flex;\n    align-items: center;\n    justify-content: center;\n}\n.p-datatable-hoverable-rows {\n    cursor: pointer;\n}\n\n/* Scrollable */\n.p-datatable-scrollable{\n    overflow: hidden;\n}\n.p-datatable-scrollable > .p-datatable-wrapper {\n    position: relative;\n}\n.p-datatable-scrollable-table > .p-datatable-thead {\n    position: sticky;\n    top: 0;\n    z-index: 1;\n}\n.p-datatable-scrollable-table > .p-datatable-frozen-tbody {\n    position: sticky;\n    z-index: 1;\n}\n.p-datatable-scrollable-table > .p-datatable-tfoot {\n    position: sticky;\n    /*bottom: 0;*/\n    bottom: -1px;  /*设置为 -1px,可解决body滚动时，有部分内容可能会显示在footer底下的问题，应该是设置边框导致的*/\n    z-index: 1;\n}\n.p-datatable-scrollable .p-frozen-column {\n    position: sticky;\n    background: inherit;\n}\n.p-datatable-scrollable th.p-frozen-column {\n    z-index: 1;\n}\n.p-datatable-flex-scrollable {\n    display: flex;\n    flex-direction: column;\n    height: 100%;\n}\n.p-datatable-flex-scrollable > .p-datatable-wrapper {\n    display: flex;\n    flex-direction: column;\n    flex: 1;\n    height: 100%;\n}\n.p-datatable-scrollable-table > .p-datatable-tbody > .p-rowgroup-header {\n    position: sticky;\n    z-index: 1;\n}\n\n/* Resizable */\n.p-datatable-resizable-table > .p-datatable-thead > tr > th,\n.p-datatable-resizable-table > .p-datatable-tfoot > tr > td,\n.p-datatable-resizable-table > .p-datatable-tbody > tr > td {\n    overflow: hidden;\n    white-space: nowrap;\n}\n.p-datatable-resizable-table > .p-datatable-thead > tr > th.p-resizable-column:not(.p-frozen-column) {\n    background-clip: padding-box;\n    position: relative;\n}\n.p-datatable-resizable-table-fit > .p-datatable-thead > tr > th.p-resizable-column:last-child .p-column-resizer {\n    display: none;\n}\n.p-datatable .p-column-resizer {\n    display: block;\n    position: absolute !important;\n    top: 0;\n    right: 0;\n    margin: 0;\n    width: 0.5rem;\n    height: 100%;\n    padding: 0px;\n    cursor: col-resize;\n    border: 1px solid transparent;\n}\n\n/*\n.p-datatable .p-column-header-content {\n    display: flex;\n    align-items: center;\n}\n*/\n.p-datatable .p-column-resizer-helper {\n    width: 1px;\n    position: absolute;\n    z-index: 10;\n    display: none;\n}\n.p-datatable .p-row-editor-init,\n.p-datatable .p-row-editor-save,\n.p-datatable .p-row-editor-cancel {\n    display: inline-flex;\n    align-items: center;\n    justify-content: center;\n    overflow: hidden;\n    position: relative;\n}\n\n/* Expand */\n.p-datatable .p-row-toggler {\n    display: inline-flex;\n    align-items: center;\n    justify-content: center;\n    overflow: hidden;\n    position: relative;\n}\n\n/* Reorder */\n.p-datatable-reorder-indicator-up,\n.p-datatable-reorder-indicator-down {\n    position: absolute;\n    display: none;\n}\n.p-reorderable-column,\n.p-datatable-reorderablerow-handle {\n    cursor: move;\n}\n\n/* Loader */\n.p-datatable .p-datatable-loading-overlay {\n    position: absolute;\n    display: flex;\n    align-items: center;\n    justify-content: center;\n    z-index: 2;\n}\n\n/* Filter */\n.p-column-filter-row {\n    display: flex;\n    align-items: center;\n    width: 100%;\n}\n.p-column-filter-menu {\n    display: inline-flex;\n    margin-left: auto;\n}\n.p-column-filter-row .p-column-filter-element {\n    flex: 1 1 auto;\n    width: 1%;\n}\n.p-column-filter-menu-button,\n.p-column-filter-clear-button {\n    display: inline-flex;\n    justify-content: center;\n    align-items: center;\n    cursor: pointer;\n    text-decoration: none;\n    overflow: hidden;\n    position: relative;\n}\n.p-column-filter-overlay {\n    position: absolute;\n    top: 0;\n    left: 0;\n}\n.p-column-filter-row-items {\n    margin: 0;\n    padding: 0;\n    list-style: none;\n}\n.p-column-filter-row-item {\n    cursor: pointer;\n}\n.p-column-filter-add-button,\n.p-column-filter-remove-button {\n    justify-content: center;\n}\n.p-column-filter-add-button .p-button-label,\n.p-column-filter-remove-button .p-button-label {\n    flex-grow: 0;\n}\n.p-column-filter-buttonbar {\n    display: flex;\n    align-items: center;\n    justify-content: space-between;\n}\n.p-column-filter-buttonbar .p-button:not(.p-button-icon-only) {\n    width: auto;\n}\n\n/* Responsive */\n.p-datatable .p-datatable-tbody > tr > td > .p-column-title {\n    display: none;\n}\n\n/* VirtualScroller */\n.p-datatable-virtualscroller-spacer {\n    display: flex;\n}\n.p-datatable .p-virtualscroller .p-virtualscroller-loading {\n    transform: none !important;\n    min-height: 0;\n    position: sticky;\n    top: 0;\n    left: 0;\n}\n";
styleInject(css_248z);

script.render = render;
script.__file = "components/lib/datatable/DataTable.vue";

export { script as default };
